<?php
if ( ! class_exists( 'WP_List_Table' ) )
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );

class mam_debug_grid extends WP_List_Table {

	public $current_page;
	public $tour_type;

    function __construct() {
		parent::__construct( array(
			'singular' => 'post',
			'plural' => 'posts',
			'ajax' => true,
			'screen' => 'interval_list') );
	}

    function prepare_items() {

        if( isset($_REQUEST['tag']) && $_REQUEST['tag'] == 'No Filter') unset( $_REQUEST['tag'] );

		$per_page = 25;
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = array();

        $this->_column_headers = array($columns, $hidden, $sortable);

		$args = array(
			'posts_per_page' => $per_page,
			'orderby' => 'plugin',
			'order' => 'ASC',
			'offset' => ( $this->get_pagenum() - 1 ) * $per_page );

		if ( ! empty( $_REQUEST['s'] ) )
			$args['s'] = $_REQUEST['s'];

		if ( ! empty( $_REQUEST['orderby'] ) ) {
			if ( 'left_column' == $_REQUEST['left_column'] )
				$args['orderby'] = 'left_column';
		}

		if ( ! empty( $_REQUEST['order'] ) ) {
			if ( 'asc' == strtolower( $_REQUEST['order'] ) )
				$args['order'] = 'ASC';
			elseif ( 'desc' == strtolower( $_REQUEST['order'] ) )
				$args['order'] = 'DESC';
		}

        $per_page     = $this->get_items_per_page( 'analytics_per_page', $per_page );
        $current_page = $this->get_pagenum();

		$total_items = $this->get_grid_data_count();
		$total_pages = ceil( $total_items / $per_page );

		$this->set_pagination_args( array(
            'total_items' => $total_items,
            'per_page'    => $per_page
        ) );

        $this->items = $this->get_grid_data( $per_page , $current_page);

	}

    public function display_tablenav($which) {
        global $wpdb;
        $table_name = $wpdb->prefix . 'tsl_debug_items';
        $data = $wpdb->get_results("SELECT message_from FROM $table_name  GROUP BY message_from ORDER BY message_from", ARRAY_A);
        if( sizeof($data) == 0 ) return;

        ?>
        <div class="tablenav <?php echo esc_attr($which); ?>">
            <div class="alignleft actions">
                <form method="get">
                    <table><tr><td>
                    <?php
                    $this->search_box('Search', 'search_id');
                    echo '</td><td><a href="' . esc_url(add_query_arg(array('s' => ''), admin_url('admin.php?page=' . $_REQUEST['page']))) . '" class="button" id="clear-search-button">' . __('Clear Search') . '</a></td></tr></table>';
                    ?>
                    <input type="hidden" name="page" value="<?php echo esc_attr($_REQUEST['page']) ?>" />
                    </table>
                </form>
            </div>
            <?php
            parent::display_tablenav($which);
            ?>
        </div>
        <?php
    }

	function get_columns() {

        $columns = array(
            'tsl-debug-date-column' => __( 'Date', 'local-app-manger' ),
            'tsl_status_column' =>   __( 'Tag', 'local-app-manger' ),
            'tsl_plugin_column' =>  __( 'Plugin', 'local-app-manger' ),
            'tsl-mam-debug-grid-message' =>   __( 'Message', 'local-app-manger' )
        );

        return $columns;

	}

	function get_sortable_columns() {
		$columns = array(
			'tsl-debug-date-column' => array( 'tsl_name_column', true ) );

		return $columns;
	}

	function get_bulk_actions() {
		$actions = array();

		return $actions;
	}

	function column_default( $item, $column_name ) {
		switch( $column_name ) {
			case 'tsl-debug-date-column':
			case 'tsl_status_column':
            case 'tsl-mam-debug-grid-message':
            case 'tsl_plugin_column':
			return $item[ $column_name ];
            default:
              return print_r( $item, true ) ;
            }
	}

    function get_grid_data( $per_page , $current_page){

        $timezone = get_option('timezone_string');
        if( $timezone == '' ){
            $timezone = 'UTC';
        }
        date_default_timezone_set( $timezone );

	    $items = array();
	    $offset = $current_page * $per_page - $per_page;

	    $tag = '';
        $has_where = false;
	    if( isset($_REQUEST['tag'])){
	        $tag = ' WHERE message_from = "'.$_REQUEST['tag'].'" ';
            $has_where = true;
        }
        $search = isset($_REQUEST['s']) ? $_REQUEST['s'] : '';
        if( $search != '' ){
            if( $has_where ){
                $tag .= ' AND message_info LIKE "%'.$search.'%" ';
            }else{
                $tag = ' WHERE message_info LIKE "%'.$search.'%" ';
            }
        }

	    global $wpdb;
        $table_name = $wpdb->prefix . 'tsl_debug_items';
	    $data = $wpdb->get_results("SELECT * FROM $table_name ". $tag ." ORDER BY this_index DESC LIMIT $offset, $per_page", ARRAY_A);

	    $tz = get_option('gmt_offset');

        foreach ($data as $temp_array)
        {
            try {
                $dt = new DateTime(date('d-m-Y h:i a', strtotime( $temp_array['this_date'])), new DateTimeZone( 'UTC' ));
//                $timezoneName = timezone_name_from_abbr( get_option('timezone_string'), $tz * 3600, true  );
                $dt->setTimezone(new DateTimezone($timezone));
                $temp_array['this_date'] = $dt->format('M d h:i a');
            }catch (Exception $e){

            }

			$items[] = array(
			    'tsl-debug-date-column' => $temp_array['this_date'],
                'tsl_status_column' => $temp_array['message_from'],
                'tsl-mam-debug-grid-message' => '<pre>'. $temp_array['message_info'] . '</pre>',
                'tsl_plugin_column' => $temp_array['message_plugin'],
			);
		}

        return $items;

    }

	function get_grid_data_count(){

        $search = isset($_REQUEST['s']) ? $_REQUEST['s'] : '';

        $tag = '';
        $has_where = false;
	    if( isset($_REQUEST['tag'])){
	        $tag = ' WHERE message_from = "'.$_REQUEST['tag'].'" ';
            $has_where = true;
        }
        if( $search != '' ){
            if( $has_where ){
                $tag .= ' AND message_info LIKE "%'.$search.'%" ';
            }else{
                $tag = ' WHERE message_info LIKE "%'.$search.'%" ';
            }
        }

	    global $wpdb;
        $table_name = $wpdb->prefix . 'tsl_debug_items';
	    $data = $wpdb->get_results("SELECT * FROM $table_name " . $tag, ARRAY_A);

		return sizeof($data);

	}

	function extra_tablenav( $which )
    {

        global $wpdb;
        $table_name = $wpdb->prefix . 'tsl_debug_items';

        if( isset($_REQUEST['tag']) && $_REQUEST['tag'] == 'Dev_Forms')
        {
            $wpdb->get_results("DELETE FROM $table_name WHERE message_from LIKE '".$_REQUEST['tag']."'", ARRAY_A);
            $form_settings = apply_filters( 'mam_gf_get_form_settings', array() );

            foreach( $form_settings as $form_setting )
            {
                $source = 'Unknown';
                $message = 'No Template - ' . $form_setting['id'];
                if( isset($form_setting['template'])) {
                    $template = $form_setting['template'];
                    $source = explode('/', $template['location']);
                    if (sizeof($source) == 2) {
                        $source = $source[0];
                    }
                    $message = 'Template - ' . $template['form_name'];
                }

                $wpdb->insert($table_name, array( 'message_from' => 'Dev_Forms', 'message_info' => $message, 'message_plugin' => $source ));
            }

        }
	    $data = $wpdb->get_results("SELECT message_from FROM $table_name  GROUP BY message_from ORDER BY message_from", ARRAY_A);

        //if( $which == 'top' ){
            $html  = '<form>';
            $html .= '<input type="hidden" name="page" value="'.$_REQUEST['page'].'">';
            $selected = 'No Filter';
            if( isset($_REQUEST['tag'])){
                $selected = $_REQUEST['tag'];
            }
            if( isset($_REQUEST['paged'])) {
                $html .= '<input type="hidden" name="paged" value="'.$_REQUEST['paged'].'">';
            }
            $html .= '<select name="tag" id="tag" onchange="this.form.submit()">';
            if( $selected == 'No Filter'){
                $html .= '<option selected>No Filter</option>';
            }else{
                $html .= '<option>No Filter</option>';
            }
            foreach( $data as $item )
            {
                $set_selected = '';
                if( $selected == $item['message_from']) {
                    $set_selected = 'selected';
                }
                $html .= '<option '.$set_selected.'>'.$item['message_from'].'</option>';
            }
            $html .= '</select>';
            $html .= '</form>';
            echo $html;
        //}

    }
}
