<?php

if ( ! class_exists( 'WP_List_Table' ) ) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class Notifications_List extends WP_List_Table {

    public $notifications_array = array();
    /**
     * Prepare the items for the table to process
     *
     * @return Void
     */
    public function prepare_items()
    {
        $columns = $this->get_columns();
        $hidden = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();

        $data = $this->table_data();

        $perPage = 40;
        $currentPage = $this->get_pagenum();
        $totalItems = count($data);

        $this->set_pagination_args( array(
            'total_items' => $totalItems,
            'per_page'    => $perPage
        ) );

        $data = array_slice($data,(($currentPage-1)*$perPage),$perPage);
        $this->_column_headers = array($columns, $hidden, $sortable);
        $this->items = $data;
    }

    /**
     * Override the parent columns method. Defines the columns to use in your listing table
     *
     * @return Array
     */
    public function get_columns()
    {
        $columns = array(
            //'id'  => 'ID',
            'title' => __( 'Title', 'mam-notifications-manager' ),
            'email' => __( 'Email', 'mam-notifications-manager' ),
            'pn'    => __( 'PN', 'mam-notifications-manager' ),
            'text'  => __( 'Text', 'mam-notifications-manager' ),
        );
        $columns = apply_filters( 'mam_notifications_notification_add_type_column' , $columns );

        return $columns;
    }

    /**
     * Define which columns are hidden
     *
     * @return Array
     */
    public function get_hidden_columns()
    {
        return array();
    }

    /**
     * Define the sortable columns
     *
     * @return Array
     */
    public function get_sortable_columns()
    {
        /*return array('title' => array('title', false));*/
        return array();
    }

    /**
     * Get the table data
     *
     * @return Array
     */
    private function table_data()
    {
        $data = array();

        if(!empty($this->notifications_array)){

            foreach ($this->notifications_array as $item) {

                $email_status_value = get_option( 'tsl_notification_email_status_' . $item['id'] );
                $pn_status_value = get_option( 'tsl_notification_pn_status_' . $item['id'] );
                $text_status_value = get_option( 'tsl_notification_text_status_' . $item['id'] );

                $email_text = get_option( 'tsl_notification_email_text_' . $item['id'] );
                $pn_text = get_option( 'tsl_notification_pn_text_' . $item['id'] );
                $text_text = get_option( 'tsl_notification_text_text_' . $item['id'] );

                if($email_text == '' && $pn_text == '' && $text_text == '' && isset($item['default_content']) && isset($item['default_on'])){
                    if(!empty($item['default_on'])){
                        foreach ($item['default_on'] as $default_on){
                            if($default_on == 'email'){
                                $email_status_value = 1;
                            }elseif($default_on == 'push'){
                                $pn_status_value = 1;
                            }elseif($default_on == 'sms'){
                                $text_status_value = 1;
                            }
                        }
                    }
                }

                $email_status = '<i class="fas fa-times">&nbsp;</i>';
                $pn_status = '<i class="fas fa-times">&nbsp;</i>';
                $text_status = '<i class="fas fa-times">&nbsp;</i>';

                if( $email_status_value == 1 ) $email_status = '<i class="fas fa-check">&nbsp;</i>';
                if( $pn_status_value == 1 ) $pn_status = '<i class="fas fa-check">&nbsp;</i>';
                if( $text_status_value == 1 ) $text_status = '<i class="fas fa-check">&nbsp;</i>';

                $desc = '';
                if( isset($item['description']) && strlen($item['description']) > 0  ) {
                    $desc = '<span title="' . $item['description'] . '" class="local-app-help-button fa fa-question-circle" style="margin-left:1em;"></span> ';
                }

                $this_notification = array();
                $this_notification['title'] = '<a href="?page=mam-notification-manager&slug='.$item['id'].'">' . $item['title'] . '</a>&nbsp;' . $desc;
                $this_notification['email'] = $email_status;
                $this_notification['pn'] = $pn_status;
                $this_notification['text'] = $text_status;
                $this_notification = apply_filters( 'mam_notifications_notification_add_type_column_value' , $this_notification , $item['id'] );
                $data[] = $this_notification;
            }
        }

        return $data;
    }

    /**
     * Define what data to show on each column of the table
     *
     * @param  Array $item        Data
     * @param  String $column_name - Current column name
     *
     * @return Mixed
     */
    public function column_default( $item, $column_name )
    {
        switch( $column_name ) {
            case 'title':
            case 'email':
            case 'pn':
            case 'text':
                return $item[ $column_name ];
            default:
                return  $item[ $column_name ] ;
        }
    }

    function get_table_data($notifications){
        usort($notifications,function ($a, $b)
        {
            return strcmp($a["title"], $b["title"]);
        });
        $this->notifications_array = $notifications;
    }

}
