<?php
if ( ! class_exists( 'WP_List_Table' ) )
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );

if ( !function_exists('get_plugins') ){
    require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
}

class tsl_software_grid extends WP_List_Table {

	public $current_page;
	public $software_list = array();
	private $filtered_software_list = array();
	private $all_plugins = array();

    function __construct() {
		parent::__construct( array(
			'singular' => 'post',
			'plural' => 'posts',
			'ajax' => true,
			'screen' => 'interval_list') );
	}

    function prepare_items() {

        $this->process_bulk_action();

        $this->all_plugins = get_plugins();

		$per_page = 20;
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = array();

        $this->_column_headers = array($columns, $hidden, $sortable);

		$args = array(
			'posts_per_page' => $per_page,
			'orderby' => 'plugin',
			'order' => 'ASC',
			'offset' => ( $this->get_pagenum() - 1 ) * $per_page );

		if ( ! empty( $_REQUEST['s'] ) )
			$args['s'] = $_REQUEST['s'];

		if ( ! empty( $_REQUEST['orderby'] ) ) {
			if ( 'left_column' == $_REQUEST['left_column'] )
				$args['orderby'] = 'left_column';
		}

		if ( ! empty( $_REQUEST['order'] ) ) {
			if ( 'asc' == strtolower( $_REQUEST['order'] ) )
				$args['order'] = 'ASC';
			elseif ( 'desc' == strtolower( $_REQUEST['order'] ) )
				$args['order'] = 'DESC';
		}

        $per_page     = $this->get_items_per_page( 'analytics_per_page', $per_page );
        $current_page = $this->get_pagenum();

		$total_items = $this->get_grid_data_count();
		$total_pages = ceil( $total_items / $per_page );

        $this->items = $this->get_grid_data( $total_items , 1);

	}

	function extra_tablenav( $which ) {

        //$options = $this->get_bulk_actions();

		$html = '<div class="actions">';
		$html .= '	<p class="action">';
		$html .= '		<label for="post-search-input" class="screen-reader-text">Bulk Actions:</label>';


		$html .= '	</p>';
		$html .= '</div>';

		echo $html;
	}

	public function get_bulk_actions() {

        return array(
            'activate-install'   => __( 'Install and Activate', 'mam-main' ),
        );

    }

    public function process_bulk_action() {

        if ( isset( $_POST['_wpnonce'] ) && ! empty( $_POST['_wpnonce'] ) ) {

            $nonce  = filter_input( INPUT_POST, '_wpnonce', FILTER_SANITIZE_STRING );
            $action = 'bulk-' . $this->_args['plural'];

            if ( ! wp_verify_nonce( $nonce, $action ) )
                wp_die( 'Nope! Security check failed!' );

        }

        $action = $this->current_action();
        if( isset($_REQUEST['software-grid']) && is_array( $_REQUEST['software-grid'] ) && !empty( $_REQUEST['software-grid'] ) ) {

            $action = $_REQUEST['action2'];
            foreach ($_REQUEST['software-grid'] as $plugin) {

                $dl_url = '';
                $plugin_slug = $plugin;
                $from_repo = false;
                foreach ($this->software_list as $item) {
                    if( $item['slug'] == $plugin && ! isset( $item['recommended'] )) {
                        $dl_url = $item['dl_url'] . '&api_key=' . get_option('mam-main-software-manager-api');
                    }elseif( $item['slug'] == $plugin && isset( $item['recommended'] ) ){
                        $dl_url = $item['dl_url'];
                        $from_repo = true;
                    }
                }

                if( strlen( $dl_url ) > 0 ) {

                    switch ($action) {
//                        case 'download':
//                            $this->download_plugin($dl_url, $plugin_slug);
//                            break;
                        case 'activate-install':
                            $status = $this->download_plugin($dl_url, $plugin_slug, $from_repo);
                            if ($status == 'Success') {
                                //wp redirect
                                $view = '';
                                if( isset( $_REQUEST['view'] )){
                                    $view = '&view='.$_REQUEST['view'];
                                }
                                $nonce = wp_create_nonce( 'activate-plugin' );
                                wp_redirect( admin_url('admin.php') . '?page=mam-software-manager&activate=1&slug='.$plugin_slug.'&nonce='.$nonce.$view );
                                die();
                            } else {
                                //todo handle download error
                                echo '<div class="error"><p>Failed to download plugin '.$plugin_slug.'</p></div>';
                                mamdebug('status', 'failed to download '.$plugin_slug);
                            }
                            break;
                        case 'deactivate':
                            deactivate_plugins($plugin_slug . '/' . $plugin_slug . '.php');
                            break;
                    }
                }
            }
        }

        return;
    }

    function download_plugin( $fetch_url , $plugin_slug , $from_repo = false )
    {

        //todo - make sure plugin and directory does not exist first?


        $tmp_file = download_url( $fetch_url );
        mamdebug('download', $fetch_url );
        if( $from_repo ){
            $filepath = ABSPATH . 'wp-content/plugins';
        }else {
            $filepath = ABSPATH . 'wp-content/plugins/' . $plugin_slug;

            if (!file_exists($filepath)) {
                mkdir($filepath, 0777, true);
            }
        }

        $filename = $filepath . '/' . $plugin_slug . '.zip';

        copy( $tmp_file, $filename );
        @unlink( $tmp_file );

        $zip = new ZipArchive;
        $res = $zip->open( $filename );
        if ($res === TRUE) {
            if( $from_repo ){
                $zip->extractTo(ABSPATH . 'wp-content/plugins' );
            }else {
                $zip->extractTo(ABSPATH . 'wp-content/plugins/' . $plugin_slug);
            }
            $zip->close();
            @unlink( $filename );
            return 'Success';
        } else {
            return 'Failed';
        }

    }

	function get_columns() {

        $columns = array(
            'select_software_column' => ' ',
            'plugin_name_column' => __( 'Name', 'local-app-manger' ),
            'description_column' =>   __( 'Description', 'local-app-manger' ),
            'version_column' =>   __( 'Version', 'local-app-manger' ),
            'status_column' =>   __( 'Status', 'local-app-manger' ),
        );

        return $columns;

	}

	function get_sortable_columns() {
		$columns = array(
			'plugin_name_column' => array( 'plugin_name_column', true ) );

		return $columns;
	}


	function column_default( $item, $column_name ) {
		switch( $column_name ) {
			case 'plugin_name_column':
			case 'description_column':
			case 'version_column':
			case 'status_column':
            case 'select_software_column':
			return $item[ $column_name ];
            default:
              return print_r( $item, true ) ;
            }
	}

    function get_grid_data( $per_page , $current_page){

	    $items = array();

	    for($x=0;$x<2;$x++) {
            foreach ( $this->filtered_software_list as $plugin) {

                if( $x == 0 && $plugin['slug'] != 'mam-main' ) continue;
                if( $x == 1 && $plugin['slug'] == 'mam-main' ) continue;

                $main_plugin_file = $plugin['slug'] . '/' . $plugin['slug'] . '.php';

                $disabled = '';;

                $plugin['status'] = 'Not Installed';
                if (is_plugin_active($main_plugin_file)) {
                    $plugin['status'] = 'Active';
                    $disabled = 'disabled';;
                }
                if (isset($this->all_plugins[$main_plugin_file]) && $plugin['status'] == 'Active' ){
                    $this_plugin = $this->all_plugins[$main_plugin_file];
                    if ($this_plugin['Version'] != $plugin['version']){

                        $plugin['status'] = '<span style="color:red">Update Available</span>';
                        $disabled = 'disabled';;
                    }
                }
                if( $plugin['slug'] == 'mam-main' ) {
                    $plugin['status'] = 'Required';
                    $disabled = 'disabled';;
                }

                if( $plugin['price'] == 0 ){
                    $plugin['price'] = 'No Charge';
                }else{
                    $plugin['price'] = '$'.number_format( (float)$plugin['price'] , 2, '.' , ',' );
                }

                $checkbox = '<input type="checkbox" name="software-grid[]" value="'.$plugin['slug'].'" '.$disabled.'>';

                $items[] = array(
                    'select_software_column' => $checkbox,
                    'plugin_name_column' => $plugin['name'],
                    'description_column' => $plugin['description'],
                    'version_column' => $plugin['version'],
                    'status_column' => $plugin['status']
                );
            }
        }




        return $items;

    }

	function get_grid_data_count(){

        $this->filtered_software_list = array();
        foreach( $this->software_list as $plugin )
        {
            if( ! isset( $plugin['recommended'] ) )
            {
                $this->filtered_software_list[] = $plugin;
            }
        }

        if( isset( $_REQUEST['view'] ) )
        {
            switch ( $_REQUEST['view'])
            {
                case 'all':
                    $this->filtered_software_list = $this->software_list;
                    break;
                case 'mam':
                    $this->filtered_software_list = array();
                    foreach( $this->software_list as $plugin )
                    {
                        if( ! isset( $plugin['recommended'] ) )
                        {
                            $main_plugin_file = $plugin['slug'] . '/' . $plugin['slug'] . '.php';
                            if ( ! is_plugin_active($main_plugin_file)) {
                                $this->filtered_software_list[] = $plugin;
                            }
                        }
                    }

                    break;
                case 'active':
                    $this->filtered_software_list = array();
                    foreach( $this->software_list as $plugin )
                    {
                        $main_plugin_file = $plugin['slug'] . '/' . $plugin['slug'] . '.php';
                        if (is_plugin_active($main_plugin_file)) {
                                $this->filtered_software_list[] = $plugin;
                            }
                        }
                    break;
                case 'recommended':
                    $this->filtered_software_list = array();
                    foreach( $this->software_list as $plugin )
                    {
                        if( isset( $plugin['recommended'] )  )
                        {
                            $main_plugin_file = $plugin['slug'] . '/' . $plugin['slug'] . '.php';
                            if ( ! is_plugin_active($main_plugin_file)) {
                                $this->filtered_software_list[] = $plugin;
                            }
                        }
                    }
                    break;
            }
        }


		return sizeof( $this->filtered_software_list );

	}

	protected function get_views() {
        $status_links = array(
            "mam-only" => __("<a href='".admin_url('admin.php') . '?page=mam-software-manager&view=mam'."'>Mobile App Manager</a>",'mam-main'),
            "active" => __("<a href='".admin_url('admin.php') . '?page=mam-software-manager&view=active'."'>Active</a>",'mam-main'),
            "recommended"   => __("<a href='".admin_url('admin.php') . '?page=mam-software-manager&view=recommended'."'>Recommended</a>",'mam-main')
        );
        return $status_links;
    }

}
