<?php

if ( ! class_exists( 'WP_List_Table' ) ) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class mam_test_users_grid extends WP_List_Table {
    /**
     * Prepare the items for the table to process
     *
     * @return Void
     */
    public function prepare_items()
    {

        $columns = $this->get_columns();
        $hidden = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();

        $perPage = 20;
        $currentPage = $this->get_pagenum();
        $totalItems = $this->get_total_records();

        $this->set_pagination_args( array(
            'total_items' => $totalItems,
            'per_page'    => $perPage
        ) );

        $data = $this->get_records((($currentPage-1)*$perPage),$perPage);

        $this->_column_headers = array($columns, $hidden, $sortable);
        $this->items = $data;
    }

    /**
     * Override the parent columns method. Defines the columns to use in your listing table
     *
     * @return Array
     */
    public function get_columns()
    {
        $columns = array(
            'first_name'    => __( 'First name', 'mam-notifications-manager' ),
            'last_name'     => __( 'Last Name', 'mam-notifications-manager' ),
            'email'         => __( 'Email address', 'mam-notifications-manager' ),
        );

        return $columns;
    }

    /**
     * Define which columns are hidden
     *
     * @return Array
     */
    public function get_hidden_columns()
    {
        return array();
    }

    /**
     * Define the sortable columns
     *
     * @return Array
     */
    public function get_sortable_columns()
    {
        /*return array('title' => array('title', false));*/
        return array();
    }

    /**
     * Get the total records from table
     *
     * @return Array
     */
    private function get_total_records()
    {
        $all_users = get_option('local-app-test-users', true);
        if(!is_array($all_users)) $all_users = array();
        $data = count($all_users);
        return $data;
    }

    /**
     * Get the table data
     *
     * @return Array
     */
    private function get_records($offset, $limit)
    {
        $data = array();
        $all_users = get_option('local-app-test-users', true);
        if(!is_array($all_users)) $all_users = array();
        if(!empty($all_users)){
            $upto = $offset+$limit;
            $lastKey = key(array_slice($all_users, -1, 1, true));
            if($lastKey <= $upto) $upto = $lastKey;
            for($i=$offset;$i<=$upto;$i++){
                $data[] = $all_users[$i];
            }
        }
        return $data;
    }

    /**
     * Define what data to show on each column of the table
     *
     * @param  Array $item        Data
     * @param  String $column_name - Current column name
     *
     * @return Mixed
     */
    public function column_default( $item, $column_name )
    {
        switch( $column_name ) {
            case 'first_name':
            case 'last_name':
            case 'email':
                return $item[ $column_name ];

            default:
                return print_r( $item, true ) ;
        }
    }

    /**
     * Display extra filter controls between bulk actions and pagination.
     *
     * @since 1.3.1
     */
    function extra_tablenav( $which )
    {

        // Bail on bottom table nav.
        if ('bottom' === $which) {
            return;
        }
        $actual_link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[SCRIPT_NAME]";
        $actual_link .= '?page=local_app_publish&tab=test_users';
        ?>
        <style type="text/css">.tablenav.top{display: block !important;margin-bottom: 1em;}.tablenav.top .alignleft.actions a{ display: inline-block; margin: 0; }</style>
        <div class="alignleft actions">
            <a href="<?php echo $actual_link.'&action=add-new-user';?>" class="button button-primary">Add Test User</a>
            <a href="<?php echo $actual_link.'&action=add-wp-user';?>" class="button button-primary">Add from WP Users list</a>
        </div>
        <?php
    }

}