<?php

if ( ! class_exists( 'WP_List_Table' ) ) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class mam_test_wp_users_grid extends WP_List_Table {

    /**
     * Prepare the items for the table to process
     *
     * @return Void
     */
    public function prepare_items()
    {

        $this->process_bulk_action();

        $columns = $this->get_columns();
        $hidden = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();

        $data = $this->table_data();
        usort( $data, array( &$this, 'sort_data' ) );

        $perPage = 20;
        $currentPage = $this->get_pagenum();
        $totalItems = count($data);

        $this->set_pagination_args( array(
            'total_items' => $totalItems,
            'per_page'    => $perPage
        ) );

        $data = array_slice($data,(($currentPage-1)*$perPage),$perPage);

        $this->_column_headers = array($columns, $hidden, $sortable);
        $this->items = $data;
    }

    /**
     * Override the parent columns method. Defines the columns to use in your listing table
     *
     * @return Array
     */
    public function get_columns()
    {
        $columns = array(
            'cb'    => '<input name type="checkbox" />',
            'first_name'  => 'First Name',
            'last_name'  => 'Last Name',
            'email'  => 'Email'
        );

        return $columns;
    }

    /**
     * Define which columns are hidden
     *
     * @return Array
     */
    public function get_hidden_columns()
    {
        return array();
    }

    /**
     * Define the sortable columns
     *
     * @return Array
     */
    public function get_sortable_columns()
    {
        /*return array('title' => array('title', false));*/
        return array();
    }

    function column_cb( $item ) {
        /* translators: accessibility text */
        printf( '<input type="checkbox" name="ids[]" value="%1$d" id="id-%1$d" />', $item['id'] );
    }

    function get_bulk_actions() {
        $actions = array(
            'add_user' => 'Add Users',
        );

        return $actions;
    }

    function process_bulk_action(){

        $action = $this->current_action();

        if ($action == 'add'){
            $all_users = get_option('local-app-test-users', true);
            if(!is_array($all_users)) $all_users = array();

            if(!empty($_POST['ids'])){
                foreach ($_POST['ids'] as $id) {
                    $user = get_user_by('id', $id);
                    $first_name = get_user_meta($id, 'first_name', true);
                    $last_name = get_user_meta($id, 'last_name', true);
                    $email = $user->data->email;
                    if($email != ''){
                        $all_users[] = array(
                            'first_name' => $first_name,
                            'last_name' => $last_name,
                            'email' => $email
                        );
                        update_option('local-app-test-users', $all_users);
                        $actual_link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[SCRIPT_NAME]";
                        $actual_link .= '?page=local_app_publish&tab=test_users';
                        wp_redirect($actual_link);
                        exit;
                    }
                }
            }
        }
    }

    /**
     * Get the table data
     *
     * @return Array
     */
    private function table_data()
    {
        $data = array();

        $wp_users = get_users();
        if(!empty($wp_users)){
            foreach ($wp_users as $user){
                $data[] = array(
                    'id' => $user->ID,
                    'first_name' => get_user_meta($user->ID, 'first_name', true),
                    'last_name' => get_user_meta($user->ID, 'last_name', true),
                    'email' => $user->data->email,
                );
            }
        }

        return $data;
    }

    /**
     * Define what data to show on each column of the table
     *
     * @param  Array $item        Data
     * @param  String $column_name - Current column name
     *
     * @return Mixed
     */
    public function column_default( $item, $column_name )
    {
        switch( $column_name ) {
            case 'first_name':
            case 'last_name':
            case 'email':
                return $item[ $column_name ];
            default:
                return print_r( $item, true ) ;
        }
    }

    /**
     * Display extra filter controls between bulk actions and pagination.
     *
     * @since 1.3.1
     */
    function extra_tablenav( $which )
    {

        ?><?php
        // Bail on bottom table nav.
        if ('bottom' === $which) {
            return;
        }
        $actual_link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[SCRIPT_NAME]";
        $actual_link .= '?page=local_app_publish&tab=test_users&action=add-wp-user';
        ?><form action="<?php echo $actual_link; ?>" method="post">
        <select name="action" id="bulk-action-selector-top">
            <option value="-1">Bulk actions</option>
            <option value="add_user">Add Users</option>
        </select>
        <input type="submit" id="doaction" class="button action" value="Apply">
        </form><?php
    }

}
