<?php
if ( ! class_exists( 'WP_List_Table' ) )
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );

class local_app_notifications_grid extends WP_List_Table {

	public $current_page;

    function __construct() {
		parent::__construct( array(
			'singular' => 'post',
			'plural' => 'posts',
			'ajax' => true,
			'screen' => 'interval_list') );
	}

    function prepare_items() {

		$per_page = 10;
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = array();

        $this->_column_headers = array($columns, $hidden, $sortable);

		$args = array(
			'posts_per_page' => $per_page,
			'orderby' => 'plugin',
			'order' => 'ASC',
			'offset' => ( $this->get_pagenum() - 1 ) * $per_page );

		if ( ! empty( $_REQUEST['s'] ) )
			$args['s'] = $_REQUEST['s'];

		if ( ! empty( $_REQUEST['orderby'] ) ) {
			if ( 'apple_column' == $_REQUEST['apple_column'] )
				$args['orderby'] = 'apple_column';
		}

		if ( ! empty( $_REQUEST['order'] ) ) {
			if ( 'asc' == strtolower( $_REQUEST['order'] ) )
				$args['order'] = 'ASC';
			elseif ( 'desc' == strtolower( $_REQUEST['order'] ) )
				$args['order'] = 'DESC';
		}

        $per_page     = $this->get_items_per_page( 'analytics_per_page', $per_page );
        $current_page = $this->get_pagenum();

		$total_items = $this->get_grid_data_count();
		$total_pages = ceil( $total_items / $per_page );

		$this->set_pagination_args( array(
			'total_items' => $total_items,
			'total_pages' => $total_pages,
			'per_page' => $per_page ) );

        $this->items = $this->get_grid_data( 10 , $current_page);

	}

	function get_columns() {
        $columns = array(
                'title_column' => __( 'Message Title', 'local-app' ),
                'date_column' => __( 'Date Sent', 'local-app' ),
                'sent_by' => __( 'Sent By', 'local-app' ),
                'target' => __( 'Target', 'local-app' ),
				'ios_phones' => __( 'iOS Users', 'local-app' ),
				'android_phones' => __( 'Android Users', 'local-app' ),
        );

        return $columns;

	}

	function get_sortable_columns() {
		$columns = array(
			'date_column' => array( 'date_column', true ) );

		return $columns;
	}

	function get_bulk_actions() {
		$actions = array();

		return $actions;
	}

	function column_default( $item, $column_name )
    {
		return $item[$column_name];
    }

    function get_grid_data( $per_page , $current_page){

		$history = local_app_get_notification_history(null,null,false);

		if( ! $history) return array();

        $start = ($current_page * $per_page) - $per_page;
        $end = $start + $per_page;
        $current_item = 1;

        $items = array();


		for($x=$start;$x<sizeof($history);$x++){

            if($current_item == $per_page) continue;

            if(strlen($history[$x]['message']) > 25 ){
                $short_message = substr($history[$x]['message'],0,25) . '...';
            }else{
                $short_message = $history[$x]['message'];
            }

            $this_target = 'General Notification';

            if($history[$x]['target_type'] == 'posts'){
                $this_target = '<a href="'.get_permalink($history[$x]['target_source']).'" target="_blank">'.get_the_title($history[$x]['target_source']).'</a>';
            }
            if($history[$x]['target_type'] == 'button'){
                if($history[$x]['target_source'] == '999' ){
                    $this_target = 'General Notification';
                }else{
                    $this_target = '';
                }
            }

            $items[] = array(
                'title_column' => '<strong><span title="'.stripslashes($history[$x]['message']).'">'.stripslashes($short_message).'</span></strong>',
                'date_column' => $history[$x]['send_date'],
                'sent_by' => $history[$x]['sent_by_name'],
                'target' => $this_target,
                'ios_phones' => $history[$x]['ios_phones'],
                'android_phones' => $history[$x]['android_phones']
            );
			$current_item++;
		}

        return $items;
    }

	function get_grid_data_count(){

		$history = local_app_get_notification_history(null,null,false);
		return sizeof($history);

	}

}
