<?php
if (!class_exists('local_app_submit')) {

    class local_app_submit
    {

        function __construct()
        {

        }

        function create_ios_app_icons()
        {

            if( ! extension_loaded('gd')) {
                mamdebug('admin_alert', "The GD extension is required to generate app icons.");
                return;
            }

            $image = wp_get_attachment_image_src(get_option('local-app-images-app-icon'), 'full');
            if ($image)
            {
                $url = $image[0];

                global $tsl_local_app_dir_path;
                $upload_dir = wp_upload_dir();
                $inputImage = str_replace($upload_dir['baseurl'], $upload_dir['basedir'], $url);
                $outputDir = $tsl_local_app_dir_path . 'publish/ios/app_icons/';

                // Ensure the output directory exists
                if (!is_dir($outputDir)) {
                    mkdir($outputDir, 0755, true);
                }

                // List of iOS app icon sizes
                $iconSizes = [
                    ['size' => 20, 'suffix' => 'Notification'],
                    ['size' => 40, 'suffix' => 'Notification@2x'],
                    ['size' => 60, 'suffix' => 'Notification@3x'],
                    ['size' => 29, 'suffix' => 'Settings'],
                    ['size' => 58, 'suffix' => 'Settings@2x'],
                    ['size' => 87, 'suffix' => 'Settings@3x'],
                    ['size' => 40, 'suffix' => 'Spotlight'],
                    ['size' => 80, 'suffix' => 'Spotlight@2x'],
                    ['size' => 120, 'suffix' => 'Spotlight@3x'],
                    ['size' => 60, 'suffix' => 'App'],
                    ['size' => 120, 'suffix' => 'App@2x'],
                    ['size' => 180, 'suffix' => 'App@3x'],
                    ['size' => 76, 'suffix' => 'iPad'],
                    ['size' => 152, 'suffix' => 'iPad@2x'],
                    ['size' => 167, 'suffix' => 'iPadPro'],
                    ['size' => 1024, 'suffix' => 'AppStore'],
                ];

                // Load the input image
                $image = imagecreatefrompng($inputImage);
                if (!$image) {
                    mamdebug('ios icons', "Failed to load input image.");
                    return;
                }

                // Get original image dimensions
                $originalWidth = imagesx($image);
                $originalHeight = imagesy($image);

                // Ensure the input image is square
                if ($originalWidth !== $originalHeight) {
                    //to do - is image is not square, then create new image and add this image on top
                    mamdebug('ios icons', "The input image must be square.");
                    return;
                }

                // Generate resized icons
                foreach ($iconSizes as $icon) {
                    $size = $icon['size'];
                    $suffix = $icon['suffix'];

                    // Create a blank canvas for the resized image
                    $resizedImage = imagecreatetruecolor($size, $size);

                    // Preserve transparency
                    imagesavealpha($resizedImage, true);
                    $transparency = imagecolorallocatealpha($resizedImage, 0, 0, 0, 127);
                    imagefill($resizedImage, 0, 0, $transparency);

                    // Resize the image
                    imagecopyresampled(
                        $resizedImage, $image,
                        0, 0, 0, 0,
                        $size, $size,
                        $originalWidth, $originalHeight
                    );

                    // Save the resized image
                    $outputFile = $outputDir . "icon_{$suffix}.png";
                    imagepng($resizedImage, $outputFile);

                    // Free memory
                    imagedestroy($resizedImage);

                    mamdebug('ios icons', "Generated: $outputFile");
                }

                // Free memory for the original image
                imagedestroy($image);
            }

        }

        function create_android_app_icons()
        {

            if( ! extension_loaded('gd')) {
                mamdebug('admin_alert', "The GD extension is required to generate app icons.");
                return;
            }

            $image = wp_get_attachment_image_src(get_option('local-app-images-app-icon'), 'full');
            if ($image) {
                $url = $image[0];

                global $tsl_local_app_dir_path;
                $upload_dir = wp_upload_dir();
                $inputForeground = str_replace($upload_dir['baseurl'], $upload_dir['basedir'], $url);
                $outputDir = $tsl_local_app_dir_path . 'publish/android/app_icons/';

                //todo set rgb for background from settings
                $img = imagecreatetruecolor(512, 512);
                $color = imagecolorallocate($img, 0, 0, 255);
                imagefill($img, 0, 0, $color);
                $temp_dir = get_temp_dir();
                $inputBackground = $temp_dir . '/icon_background_' . time() . '.png';
                imagesavealpha($img, true);
                imagepng($img, $inputBackground);

                // Ensure the output directory exists
                if (!is_dir($outputDir)) {
                    mkdir($outputDir, 0755, true);
                }

                // List of Android app icon sizes for standard icons
                $iconSizes = [
                    ['size' => 48, 'folder' => 'mipmap-mdpi'],
                    ['size' => 72, 'folder' => 'mipmap-hdpi'],
                    ['size' => 96, 'folder' => 'mipmap-xhdpi'],
                    ['size' => 144, 'folder' => 'mipmap-xxhdpi'],
                    ['size' => 192, 'folder' => 'mipmap-xxxhdpi'],
                ];

                // List of sizes for adaptive icons (foreground and background)
                $adaptiveSizes = [
                    ['size' => 108, 'folder' => 'mipmap-mdpi'],
                    ['size' => 162, 'folder' => 'mipmap-hdpi'],
                    ['size' => 216, 'folder' => 'mipmap-xhdpi'],
                    ['size' => 324, 'folder' => 'mipmap-xxhdpi'],
                    ['size' => 432, 'folder' => 'mipmap-xxxhdpi'],
                ];

                // Generate standard icons
                foreach ($iconSizes as $icon) {
                    $size = $icon['size'];
                    $folder = $outputDir . $icon['folder'];

                    // Ensure the density-specific directory exists
                    if (!is_dir($folder)) {
                        mkdir($folder, 0755, true);
                    }

                    $this->resizeImage($inputForeground, $folder, $size, 'ic_launcher.png');
                }

                // Generate adaptive icons (foreground and background)
                foreach ($adaptiveSizes as $icon) {
                    $size = $icon['size'];
                    $folder = $outputDir . $icon['folder'];

                    // Ensure the density-specific directory exists
                    if (!is_dir($folder)) {
                        mkdir($folder, 0755, true);
                    }

                    // Resize foreground image
                    $this->resizeImage($inputForeground, $folder, $size, 'ic_launcher_foreground.png');

                    // Resize background image
                    $this->resizeImage($inputBackground, $folder, $size, 'ic_launcher_background.png');
                }
            }
        }

        function resizeImage($inputImage, $outputDir, $size, $fileName, $bgColor = '')
        {
            $image = imagecreatefrompng($inputImage);
            if (!$image) {
                mamdebug('android icons', "Failed to load input image: $inputImage");
            }

            // Create a blank canvas for the resized image
            $resizedImage = imagecreatetruecolor($size, $size);

            if($bgColor != '') {
                $app_settings_handler = new tsl_mam_app_settings_main();
                $rgb = $app_settings_handler->hexToRgb($bgColor);
                // Preserve transparency
                imagesavealpha($resizedImage, false);
                $transparency = imagecolorallocate($resizedImage, $rgb['r'], $rgb['g'], $rgb['b']);
                imagefill($resizedImage, 0, 0, $transparency);
            }else{
                // Preserve transparency
                imagesavealpha($resizedImage, true);
                $transparency = imagecolorallocatealpha($resizedImage, 0, 0, 0, 127);
                imagefill($resizedImage, 0, 0, $transparency);
            }

            // Resize the image
            $originalWidth = imagesx($image);
            $originalHeight = imagesy($image);
            imagecopyresampled(
                $resizedImage, $image,
                0, 0, 0, 0,
                $size, $size,
                $originalWidth, $originalHeight
            );

            // Save the resized image
            $outputFile = $outputDir . '/' . $fileName;
            imagepng($resizedImage, $outputFile);

            // Free memory
            imagedestroy($resizedImage);
            imagedestroy($image);

            mamdebug('android icons', "Generated: $outputFile ");

        }

        public function local_app_setup_publish_page(){

            if (isset($_REQUEST['tab'])){
                $current = $_REQUEST['tab'];
            }else{
                $current = 'apple';
            }

            $html_line = '<div class="local-app-main-content-area">';

            if (!current_user_can('manage_options')) return 'No one is home';

            if(!empty($_POST) && isset($_REQUEST['get-api-key'] ) & array_key_exists('get_software_api', $_POST)){
                $nonce = wp_create_nonce( 'set-api-key' );
                $request_url = 'https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=wc_private_repo_get_api_key&return_url='.admin_url( 'admin.php' ).'&site_url='.$_SERVER['SERVER_NAME'].'&nonce='.$nonce.'&ajaxurl='.admin_url('admin-ajax.php');
                wp_redirect( $request_url );
                die();
            }

            $html_line .= '<h1>Publish App For Test or Production</h1>';
            if( strlen( get_option('mam-main-software-manager-api') ) <= 1 ){
                $html_line .= '<form action="" method="POST">';
                $html_line .= '<input type="hidden" id="get-api-key" name="get-api-key" value="1">';
                $html_line .= '<p>API key is required to add publish app for test flight or production. Please click on below button to get API key from Tinyscreenlabs.com.</p>';
                $html_line .= '<p class="submit"><input type="submit" name="get_software_api" id="get_software_api" class="button button-primary" value="Get API Key"></p>';
                $html_line .= '</form>';
            }else{

                switch ($current) {
                    case 'apple':
                        $html_line = $this->local_app_setup_publish_apple();
                        $setup = new local_app_setup();
                        wp_localize_script('localapp', 'localapp_tsl_media_icons', $setup->get_media_library_images());
                        break;
                    case 'android':
                        $html_line = $this->local_app_setup_publish_android();
                        $setup = new local_app_setup();
                        wp_localize_script('localapp', 'localapp_tsl_media_icons', $setup->get_media_library_images());
                        break;
                    case 'test_users':
                        $html_line = $this->local_app_setup_test_users();
                        break;
                    case 'api_key':
                        $html_line = $this->local_app_setup_api_key();
                        break;
                    case 'support':
                        $url = '/wp-admin/admin.php?page=mam-setup-wizard&go=publish';
                        wp_redirect($url);
                        die();
                }
            }

            $html_line .= '</div>';

            return $html_line;
        }

        function local_app_setup_api_key(){

            if(!local_app_is_connected_to_internet()){

                return '<div class="local-app-api-message local-app-content-width-info">'.local_app_get_lang('local-app-publish-localhost').'</div>';

            }

            $html_line = '<div id="local-app-disable-api" style="display:none;">
                 <p>'.local_app_get_lang('disable-api-message').'</p>
            </div>';

            $api_key = local_app_get_api_key();

            $update_message = 'Add API Key';
            $disable_button = '';
            $email = '';

            if($api_key){
                $update_message = 'Update API Key';
                $email = get_option('local-app-api-email');
                $disable_button = '&nbsp;<a id="local-app-disable-api-request" class="button-primary local-app-publish-button" onclick="local_app.disable_api();">Disable API Key</a>&nbsp;<span title="'.local_app_get_qtip_message('local-app-disable-api-key').'" class="fa fa-question-circle local-app-help-button-large">&nbsp;</span>';
                $html_line .= '<br>';
                $html_line .= '<div class="local-app-api-message local-app-content-width-info">You can update or disable your API key here.  Make sure the email address field matches the email you used to purchase the key on the <a href="'.TSL_AFF_URL.'" target="_blank">Tiny Screen Labs</a> website.</div>';
                $html_line .= '<br>';
            }else {
                $html_line .= '<div class="local-app-api-message local-app-content-width-info">'.local_app_get_lang('app-submit-purchase-message').'</div>';
                $html_line .= '<div class="local-app-api-message local-app-content-width-info">'.local_app_get_lang('api-call-to-action').'</div>';
            }

            $addons = tal_mam_main_manager::local_app_premium_add_ons();

                $html_line .= '<table>';
                $html_line .= '<tr><td style="font-weight: bold;font-size: 1.2em;">Email associated with API Keys:</td><td>&nbsp;&nbsp;<input type="text" style="width:25em" value="' . $email . '" id="local-app-update-api-key-email" >&nbsp;</td></td>&nbsp;<td><td></td></tr>';
                $html_line .= '<tr><td style="font-weight: bold;font-size: 1.2em;">App Service API Key:</td><td>&nbsp;&nbsp;<input type="text" style="width:25em" value="' . $api_key . '" id="local-app-update-api-key-key" ></td><td><a id="local-app-update-api-key" disabled="disabled" class="button-primary local-app-publish-button" onclick="local_app.update_api_key();">' . $update_message . '</a>' . $disable_button . '</td></tr>';

            if(sizeof($addons) > 0 ) {
                $html_line .= '<tr><td style="font-weight: bold;font-size: 1.2em;">&nbsp;</td><td></td><td></td></tr>';
                $html_line .= '<tr><td style="font-weight: bold;font-size: 1.2em;">API Keys for Add-Ons</td><td></td><td></td></tr>';


                for ($x = 0; $x < sizeof($addons); $x++) {

                    $addon_api_key = $addons[$x]['api_key'];
                    $addon_api_id = $addons[$x]['id'];
                    if (strlen($addon_api_key) > 0) {
                        $update_message = 'Update API Key';
                        $disable_button = '&nbsp;<a class="button-primary local-app-publish-button" onclick="local_app.disable_addon_api(' . $addon_api_id . ');">Disable API Key</a>&nbsp;<span title="' . local_app_get_qtip_message('local-app-disable-api-key') . '" class="fa fa-question-circle local-app-help-button-large">&nbsp;</span>';
                    } else {
                        $update_message = 'Add API Key';
                        $disable_button = '';
                    }

                    $html_line .= '<tr><td style="font-weight: bold;font-size: 1.2em;">' . $addons[$x]['name'] . '</td><td>&nbsp;&nbsp;<input type="text" style="width:25em" value="' . $addon_api_key . '" id="local-app-update-api-addon-key-' . $addon_api_id . '" ></td><td><a id="local-app-update-api-addon-key-' . $addon_api_id . '" disabled="disabled" class="button-primary local-app-publish-button" onclick="local_app.update_addon_api_key(' . $addon_api_id . ');">' . $update_message . '</a>' . $disable_button . '</td></tr>';
                }
            }

            $html_line .= '<tr><td><span id="local-app-api-error-message" class="local-app-hide"></span></td><td></td><td></td></tr>';
            $html_line .= '</table>';

            return $html_line;

        }

        function can_publish_app($osversion){

            $html_line = '';

            if($osversion == 1){
                //apple
                //text field test
                $return_array = array();

                $return_array[] = get_option('local-app-ios-app-name');
                $return_array[] = get_option('local-app-ios-app-title');
                $return_array[] = get_option('local-app-ios-app-description');
                $return_array[] = get_option('local-app-ios-app-keywords');
                $return_array[] = get_option('local-app-ios-app-support');
                //$return_array[] = get_option('local-app-ios-app-privacy');
                //$return_array[] = get_option('local-app-ios-app-marketing');
                $return_array[] = get_option('local-app-ios-app-copyright');
                $return_array[] = get_option('local-app-ios-primary-cat');
                $return_array[] = get_option('local-app-ios-secondary-cat');

                for($x=0;$x<sizeof($return_array);$x++){
                    if (strlen($return_array[$x]) == 0 ) $html_line = 'disabled="disabled"';
                }


//                for($x=0;$x<sizeof($ratingsArray['ratings']);$x++) {
//                    if (!isset($ratingsArray['ratings'][$x]['checked']) || $ratingsArray['ratings'][$x]['checked'] < 1 ) $html_line = 'disabled="disabled"';
//                }

                //images test
                if(!$this->get_image_url('local-app-images-app-icon')) $html_line = 'disabled="disabled"';
                if(!$this->get_image_url('local-app-images-launch-screen-1')) $html_line = 'disabled="disabled"';
                if(!$this->get_image_url('local-app-images-launch-screen-2')) $html_line = 'disabled="disabled"';

            }else{

                $return_array = array();

                $return_array[] = get_option('local-app-android-app-name');
                $return_array[] = get_option('local-app-android-app-title');
                $return_array[] = get_option('local-app-android-app-full-description');
                $return_array[] = get_option('local-app-android-app-short-description');
                $return_array[] = get_option('local-app-android-use-tsl-name');
                $return_array[] = get_option('local-app-android-app-email');
                $return_array[] = get_option('local-app-android-primary-cat');
                $return_array[] = get_option('local-app-android-secondary-cat');
                $return_array[] = get_option('local-app-android-app-category');

                for($x=0;$x<sizeof($return_array);$x++){
                    if (strlen($return_array[$x]) == 0 ) $html_line = 'disabled="disabled"';
                }

                //images test
                if(!$this->get_image_url('local-app-images-android-app-icon')) $html_line = 'disabled="disabled"';
                if(!$this->get_image_url('local-app-images-launch-screen-1')) $html_line = 'disabled="disabled"';
                if(!$this->get_image_url('local-app-images-featured-graphic')) $html_line = 'disabled="disabled"';

            }


            return $html_line;
        }

        function get_image_url( $image_name ){

            $image_url = null;

            if(is_numeric(get_option( $image_name ))) {
                $image_info_1 = wp_get_attachment_image_src(get_option( $image_name ), 'full');
                if ($image_info_1) $image_url = $image_info_1[0];
            }else{
                if(get_option('local-app-images-launch-screen-1')) $image_url = get_option('local-app-images-launch-screen-1');
            }

            return $image_url;

        }

        function local_app_setup_publish_apple(){

            $publish_button = '';
            $android_publish_button = '';
            $purchase_message = '';
            $preview_button = '';
            $save_settings_button = '';
            $api_key = '';

            $appleStatus = get_option( 'local_app_apple_status' );

            if(empty($appleStatus))     $appleStatus     = 'Not published yet';

            if(!local_app_is_connected_to_internet()){

                return '<div class="local-app-api-message local-app-content-width-info">'.local_app_get_lang('local-app-publish-localhost').'</div>';

            }

            //if (!$button_array) {
            //    $preview_button = '<div class="local-app-button-section-title">You need to setup your app using the "App Setup" tab above and click the "Push to App" button on that page.</div>';
            //} else {

                if ($this->local_app_check_api_key()) {
                    $disabled_attr = $this->can_publish_app(1);
                    if(get_option( 'local-app-apple-id' )) {
                        $publish_button = '<a id="local-app-ios-preview-request"  ' . $disabled_attr . ' class="button-primary local-app-publish-on-app-store-button" onclick="local_app.send_app_publish_request( 1 , 2 );">Update on iTunes</a>';
                    }else{
                        $publish_button = '<a id="local-app-ios-preview-request"  class="button-primary local-app-publish-on-app-store-button" onclick="local_app.send_app_publish_request( 1 , 2 );">Push to iTunes</a>';
                    }

                    $disabled_attr = $this->can_publish_app(2);
                    if(get_option( 'local-app-android-id' )) {
                        $android_publish_button = '<a id="local-app-android-preview-request" ' . $disabled_attr . 'class="button-primary local-app-publish-on-app-store-button" onclick="local_app.send_app_publish_request( 2 , 2 );">Update on Google Play</a>';
                    }else{
                        $android_publish_button = '<a id="local-app-android-preview-request" class="button-primary local-app-publish-on-app-store-button" onclick="local_app.send_app_publish_request( 2 , 2 );">Publish on Google Play</a>';
                    }
                    //$save_draft_button = '<a id="local-app-ios-save-draft"  class="button-primary local-app-publish-save-draft-button" style="background-color:gray" disabled="disabled" onclick="local_app.send_app_publish_request( 1 , 1 );" >Save Draft</a>';
                    //$save_draft_button = '<a id="local-app-ios-save-draft"  class="button-primary local-app-publish-save-draft-button" style="background-color:#0085ba" onclick="local_app.send_app_publish_request( 1 , 1 );" >Save Draft</a>';
                    $publish_status = '<span id="local-app-ios-publish-status" style="margin-top:.5em;"><strong>Status:</strong> '.$appleStatus.'</span>';
                } //else {
//                    $purchase_message = '<div class="local-app-button-section-title">To publish your App enter your API Key on the API Key section.</div>';
//                }
            //}
//            $html_line = '<div class="local-app-button-table-status" >' . $publish_status . '</div>';

            /* check app status */
            $full_url = 'https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=tsl_get_app_publish_status&os_type=ios&ajax_url=' . admin_url('admin-ajax.php');
            $response = wp_remote_get($full_url);
            $body = null;
            if ( is_array( $response ) && ! is_wp_error( $response ) ) {
                $body = $response['body'];
                $body = json_decode($body, true);
            }

            $html_line = '';
            $html_line .= '<div class="local-app-button-table"><div style="float:right">' . $preview_button . $publish_button . $android_publish_button . '</div></div>';

            $html_line .= $purchase_message;

            $html_line .= '<div class="local-app-button-section-title">App Store Info&nbsp;<span title="' . local_app_get_qtip_message('local-app-about-the-app-store') . '" class="fa fa-question-circle local-app-help-button">&nbsp;</span></div>';

            if(is_array($body) && $body['status'] == 'success') {
                $email = $body['data']['email'];
                $publish_message = "<div class='notice notice-success'><p>An iOS app publishing update was requested on ".date('m/d/Y', strtotime($body['data']['added_date']))." at ".date('H:i:s', strtotime($body['data']['added_date']))." by ".$email.". This message will be updated when that request is completed</p></div>";
                $html_line .= '<div class="local-app-publish-status">'.$publish_message.'</div>';
            }

            /* Check Android Update */
            $full_url = 'https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=tsl_get_app_publish_status&os_type=android&ajax_url=' . admin_url('admin-ajax.php');
            $response = wp_remote_get($full_url);
            $body = null;
            if ( is_array( $response ) && ! is_wp_error( $response ) ) {
                $body = $response['body'];
                $body = json_decode($body, true);
                if(is_array($body) && $body['status'] == 'success') {
                    $email = $body['data']['email'];
                    $publish_message = "<div class='notice notice-success'><p>An Android app publishing update was requested on ".date('m/d/Y', strtotime($body['data']['added_date']))." at ".date('H:i:s', strtotime($body['data']['added_date']))." by ".$email.". This message will be updated when that request is completed</p></div>";
                    $html_line .= '<div class="local-app-publish-status">'.$publish_message.'</div>';
                }
            }

            $html_line .= $this->local_app_app_store_info_ios();

            $html_line .= '<div class="local-app-button-section-title">Entitlements</div>';

            $html_line .= $this->local_app_app_entitlements_ios();

            $html_line .= '<div class="local-app-button-section-title">App Icon</div>';

            $html_line .= $this->local_app_app_icon_ios();

            $html_line .= '<div class="local-app-button-section-title">Launch Screen&nbsp;<span title="' . local_app_get_qtip_message('local-app-about-the-launch-screen') . '" class="fa fa-question-circle local-app-help-button">&nbsp;</span></div>';

            $html_line .= $this->local_app_launch_screen();

            $html_line .= '<p class="submit"><a id="local-app-ios-save-settings" class="button-primary local-app-publish-save-settings-button" style="background-color:#0085ba; margin-left: 1em;" onclick="local_app.save_app_settings();" >Save</a></p>';

            return $html_line;
        }

        function local_app_store_info_android_maturity_section(){

            $html_line = '<div class="local-app-setup-store-maturity-message">';
            $html_line .= '<p>For each content description, select the level of frequency that best describes your app. The app rating that will display on the App Store is the same across all of your platforms. It is based on the app’s platform with the highest rating. Learn more about <a href="https://support.google.com/googleplay/android-developer/answer/188189" target="_blank" >Content Ratings for Apps</a>.</p>';
            $html_line .= '<p>Apps must not contain any obscene, pornographic, offensive, or defamatory or materials of any kind (text, graphics, images, photographs, and so on), or other content or materials that in Google’s reasonable judgement may be found objectionable.</p>';

            $cat_info = null;

            $cat_button = '<a class="button-primary local-app-change-android-category-button" onclick="local_app.open_android_category_dialog();">Change App Category</a>';

            $html_line .= '<div class="local-app-android-category-header"><h2>App Category</h2></div>';
            $html_line .= '<div class="local-app-android-category-title">'.$cat_info['title'].'</div>';
            $html_line .= '<div class="local-app-android-category-desc">'.$cat_info['desc'].'</div>';
            $html_line .= '<div class="local-app-android-category-button">'.$cat_button.'</div>';
            $html_line .= '<div id="local-app-android-category-id" class="local-app-android-category-id local-app-hide">'.trim($cat_info['id']).'</div>';

            $html_line .= '<div class="local-app-android-questions-header"><h2>Content Rating Questions</h2></div>';

            $html_line .= '<table class="local-app-ratingsTable local-app-setup-store-table">';
            $html_line .= '<thead>';
            $html_line .= '<tr>';
            $html_line .= '<th style="text-align: left;">Android Content Description</th>';
            $html_line .= '<th style="text-align: center;">Yes</th>';
            $html_line .= '<th style="text-align: center;">No</th>';
            $html_line .= '</tr>';
            $html_line .= '</thead>';
            $html_line .= '<tbody>';

            $html_line .= '</tbody>';
            $html_line .= '</table>';
            $html_line .= '</div>';

            return $html_line;
        }

        function local_app_store_info_ios_maturity_section(){

            $html_line = '<div class="local-app-setup-store-maturity-message">';

            $html_line .= '<p>For each content description, select the level of frequency that best describes your app. The app rating that will display on the App Store is the same across all of your platforms. It is based on the app’s platform with the highest rating. Learn more about <a href="https://developer.apple.com/app-store/review/guidelines/" target="_blank" >Apple Store Review Guidelines</a></a></p>';

            $html_line .= '<p>Apps must not contain any obscene, pornographic, offensive, or defamatory or materials of any kind (text, graphics, images, photographs, and so on), or other content or materials that in Apple’s reasonable judgement may be found objectionable.</p><br>';

            $html_line .= '<table class="local-app-ratingsTable local-app-setup-store-table">';
            $html_line .= '<thead>';
            $html_line .= '<tr>';
            $html_line .= '<th style="text-align: left;">Apple Content Description</th>';
            $html_line .= '<th style="text-align: center;">None</th>';
            $html_line .= '<th style="text-align: center;">Infrequent/Mild</th>';
            $html_line .= '<th style="text-align: center;">Frequent/Intense</th>';
            $html_line .= '</tr>';
            $html_line .= '</thead>';
            $html_line .= '<tbody>';
            $html_line .= '</tbody>';
            $html_line .= '</table>';
            $html_line .= '</div>';

            return $html_line;

        }

        function local_app_app_store_info_ios(){

            $app_name = stripslashes(get_option('local-app-ios-app-name'));
            /*$app_title = get_option('local-app-ios-app-title');
            $app_description = get_option('local-app-ios-app-description');
            $app_keywords = get_option('local-app-ios-app-keywords');
            $app_support = get_option('local-app-ios-app-support');
            $app_privacy = get_option('local-app-ios-app-privacy');
            $app_marketing = get_option('local-app-ios-app-marketing');
            $app_copyright = get_option('local-app-ios-app-copyright');*/
            $use_app_name = '';
            $ios_bundle_id = get_option('ios_pn_app_bundle_id');
            $android_bundle_id = get_option('android_pn_app_bundle_id');
            $team_id = get_option('ios_pn_team_id');
            $version_number = get_option('mam_ios_version_number');
            $android_version_number = get_option('mam_android_version_number');
            $ios_build_number = get_option('mam_ios_build_number');
            $android_build_number = get_option('mam_android_build_number');
            $camera_message = get_option('ios_app_camera_message');
            $gps_message = get_option('ios_app_gps_message');
            $contacts_message = get_option('ios_app_contacts_message');
            $nfc_message = get_option('ios_app_nfc_message');
            $submit_type = get_option('ios_app_submit_type');
            $google_plist = get_option('ios_app_google_plist');
            $google_places_api = get_option('ios_app_google_places_api');
            $app_channel = get_option('local-app-ios-app-channel');
            if( ! $app_channel ) $app_channel = 'Prod';

            if (get_option('local-app-ios-use-tsl-name')) $use_app_name = 'checked';

            $html_line = '<table class="local-app-setup-store-table">';
            $html_line .= '<tr><td class="local-app-setup-store-left">App Name:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-ios-app-name" maxlength="15" value="'.$app_name.'"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-app-name').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">iOS Bundle ID:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-ios-bundle-id" value="'.$ios_bundle_id.'"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-bundle-id').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Android Bundle ID:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-android-bundle-id" value="'.$android_bundle_id.'"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-android-bundle-id').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Team ID:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-ios-team-id" maxlength="12" value="'.$team_id.'"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-bundle-id').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">iOS Version Number:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-ios-version-number" value="'.$version_number.'"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-version-number').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Android Version Number:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-android-version-number" value="'.$android_version_number.'"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-android-version-number').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">iOS Build Number:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-ios-build-number" value="'.$ios_build_number.'"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-build-number').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Android Build Number:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-android-build-number" value="'.$android_build_number.'"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-android-build-number').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Camera Message:</td><td class="local-app-setup-store-center"><textarea class="local-app-app-store-field" id="local-app-ios-camera-message" rows="5" cols="50">'.$camera_message.'</textarea></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-camera-message').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">GPS Message:</td><td class="local-app-setup-store-center"><textarea class="local-app-app-store-field" id="local-app-ios-gps-message" rows="5" cols="50">'.$gps_message.'</textarea></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-gps-message').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Contacts Message:</td><td class="local-app-setup-store-center"><textarea class="local-app-app-store-field" id="local-app-ios-contacts-message" rows="5" cols="50">'.$contacts_message.'</textarea></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-contacts-message').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">NFC Message:<br><span style="font-size: .8em">(Leave blank if your app does not use NFC tag reader functions)</span></td><td class="local-app-setup-store-center"><textarea class="local-app-app-store-field" id="local-app-ios-nfc-message" rows="5" cols="50">'.$nfc_message.'</textarea></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-contacts-message').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Google Plist</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-ios-google-plist" name="local-app-ios-google-plist" value="'.$google_plist.'" onclick="jQuery(this).next().trigger(\'click\')"><input type="file" name="local-app-ios-google-plist-file" value="" onchange="local_app.upload_app_plist_file( this );" style="display: none;"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-google-plist').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Google Places API Key:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-ios-google-places-api" value="'.$google_places_api.'"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-google-places-api').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Channel:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="radio" class="local-app-ios-app-channel" value="Prod" '.(($app_channel == "Prod") ? "checked='checked'" : "").'> Prod &nbsp;<input class="local-app-app-store-field" type="radio" class="local-app-ios-app-channel" value="Beta" '.(($app_channel == "Beta") ? "checked='checked'" : "").'> Beta </td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-ios-google-places-api').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '</table>';

            return $html_line;
        }

        function local_app_app_entitlements_ios(){
            $use_gps = false;
            $use_gps_data = get_option('tsl-setting-use_gps', true);
            if(is_array($use_gps_data) && !empty($use_gps_data)){
                foreach ($use_gps_data as $role_data){
                    if($role_data['general-settings'] == 'yes'){
                        $use_gps = true;
                        break;
                    }
                }
            }
            $use_push_notification = false;
            $use_push_notification_data = get_option('tsl-setting-use_push_notifications', true);
            if(is_array($use_push_notification_data) && !empty($use_push_notification_data)){
                foreach ($use_push_notification_data as $role_data){
                    if($role_data['general-settings'] == 'yes'){
                        $use_push_notification = true;
                        break;
                    }
                }
            }
            $sign_up_with_google = false;
            $sign_up_with_google_data = get_option('tsl-sign_up_with_google', true);
            if(is_array($sign_up_with_google_data) && !empty($sign_up_with_google_data)){
                foreach ($sign_up_with_google_data as $role_data) {
                    foreach ($role_data as $dataValue){
                        if ($dataValue == 'yes') {
                            $sign_up_with_google = true;
                            break;
                        }
                    }
                }
            }
            $entitlements = array(
                'push-notifications' => 'Push Notifications',
                'nfc-reader' => 'NFC Reader',
                'location-services' => 'Location Services',
                'location-services-background' => 'Location Services - Background',
                'sign-in-with-apple' => 'Sign In with Apple',
            );
            $saved_entitlements = get_option('saved_entitlements', true);
            if(!is_array($saved_entitlements)) $saved_entitlements = array();


            $html_line = '<table class="entitlements-table local-app-setup-store-table" cellspacing=0><tbody>';
            foreach ($entitlements as $slug=>$entitlement){
                if($slug == 'location-services' && $use_gps){
                    $html_line .= '<tr><td>' . $entitlement . '</td><td><input name="entitlements[]" type="checkbox" value="' . $slug . '" checked="checked" disabled></td></tr>';
                }else if($slug == 'push-notifications' && $use_push_notification){
                    $html_line .= '<tr><td>' . $entitlement . '</td><td><input name="entitlements[]" type="checkbox" value="' . $slug . '" checked="checked" disabled></td></tr>';
                }else if($slug == 'sign-in-with-apple' && $sign_up_with_google){
                    $html_line .= '<tr><td>' . $entitlement . '</td><td><input name="entitlements[]" type="checkbox" value="' . $slug . '" checked="checked" disabled></td></tr>';
                }else {
                    $html_line .= '<tr><td>' . $entitlement . '</td><td><input name="entitlements[]" type="checkbox" value="' . $slug . '" ' . (in_array($slug, $saved_entitlements) ? "checked='checked'" : "") . '></td></tr>';
                }
            }
            $html_line .= '</tbody></table>';


            return $html_line;
        }

        function local_app_setup_publish_android()
        {

            if(!local_app_is_connected_to_internet()){

                return '<div class="local-app-api-message local-app-content-width-info">'.local_app_get_lang('local-app-publish-localhost').'</div>';

            }

            $publish_button = '';
            $purchase_message = '';
            $androidStatus =    get_option( 'local_app_android_status' );
            if(empty($androidStatus))   $androidStatus   = 'Not published yet';

                if($this->local_app_check_api_key()) {
                    $disabled_attr = $this->can_publish_app(2);
                    if(get_option( 'local-app-android-id' )) {
                        $publish_button = '<a id="local-app-android-preview-request" class="button-primary local-app-publish-on-app-store-button" onclick="local_app.send_app_publish_request( 2 , 2 );">Update on Google Play</a>';
                    }else{
                        $publish_button = '<a id="local-app-android-preview-request" class="button-primary local-app-publish-on-app-store-button" onclick="local_app.send_app_publish_request( 2 , 2 );">Publish on Google Play</a>';
                    }
                    $save_draft_button = '<a id="local-app-android-save-draft"  class="button-primary local-app-publish-save-draft-button" onclick="local_app.send_app_publish_request( 2 , 1 );">Save Draft</a>';
                    $publish_status = '<span id="local-app-android-publish-status" style="margin-top:.5em;"><strong>Status:</strong> '.$androidStatus.'</span>';
                }else{
                    $purchase_message = '<div class="local-app-button-section-title">To publish your App enter your API Key on the API Key section.</div>';
                }
            //}
            $html_line = '';
            //$html_line = '<div class="local-app-button-table-status" >' . $publish_status . '</div>';

            //$html_line .= '<div class="local-app-button-table"><div style="float:right">'.$preview_button.$save_draft_button.$publish_button.'</div></div>';
            $html_line .= '<div class="local-app-button-table"><div style="float:right">'.$publish_button.'</div></div>';

            $html_line .= $purchase_message;

            //$html_line .= '<div class="local-app-button-section-title">Google Play Info&nbsp;<span title="'.local_app_get_qtip_message('local-app-about-the-android-store').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></div>';

            //$html_line .= $this->local_app_app_store_info_android();

            //$html_line .= '<div class="local-app-button-section-title">Content Rating</div>';

            //$html_line .= $this->local_app_store_info_android_maturity_section();

            $html_line .= '<div class="local-app-button-section-title">App Icon</div>';

            $html_line .= $this->local_app_app_icon_android();

            $html_line .= '<div class="local-app-button-section-title">Launch Screens&nbsp;<span title="' . local_app_get_qtip_message('local-app-about-the-launch-screen') . '" class="fa fa-question-circle local-app-help-button">&nbsp;</span></div>';

            $html_line .= $this->local_app_launch_screen(2);

            //$html_line .= '<div class="local-app-button-section-title">Feature Graphic Screen&nbsp;<span title="' . local_app_get_qtip_message('local-app-about-the-featured-graphic') . '" class="fa fa-question-circle local-app-help-button">&nbsp;</span></div>';

            //$html_line .= $this->local_app_featured_graphic();

            return $html_line;
        }

        function local_app_setup_test_users(){

            $html_line = '';
            if(array_key_exists('action', $_REQUEST) && $_REQUEST['action'] == 'add-new-user'){
                if(array_key_exists('mam_add_test_user', $_POST)){
                    $all_users = get_option('local-app-test-users', true);
                    if(!is_array($all_users)) $all_users = array();
                    if(trim($_POST['test_user_fname']) != '' && trim($_POST['test_user_lname']) != '' && trim($_POST['test_user_email']) != ''){
                        $all_users[] = array(
                            'first_name' => trim($_POST['test_user_fname']),
                            'last_name' => trim($_POST['test_user_lname']),
                            'email' => trim($_POST['test_user_email'])
                        );
                        update_option('local-app-test-users', $all_users);
                        $actual_link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[SCRIPT_NAME]";
                        $actual_link .= '?page=local_app_publish&tab=test_users';
                        wp_redirect($actual_link);
                        exit;
                    }
                }
                $html_line .= '<form action="" method="post">';
                $html_line .= '<table class="form-table"><tbody>';
                $html_line .= '<tr><th>First Name</th><td><input type="text" name="test_user_fname" value="" class="regular-text" required></td></tr>';
                $html_line .= '<tr><th>Last Name</th><td><input type="text" name="test_user_lname" value="" class="regular-text" required></td></tr>';
                $html_line .= '<tr><th>First Name</th><td><input type="text" name="test_user_email" value="" class="regular-text" required></td></tr>';
                $html_line .= '</tbody></table>';
                $html_line .= get_submit_button( __( 'Submit' ), 'primary', 'mam_add_test_user', false );
                $html_line .= '</form>';
            }elseif(array_key_exists('action', $_REQUEST) && $_REQUEST['action'] == 'add-wp-user'){
                $list = new mam_test_wp_users_grid();
                $list->prepare_items();
                ob_start();
                echo $list->views();
                echo $list->display();
                $html_line .= ob_get_contents();
                ob_end_clean();
            }else {
                $list = new mam_test_users_grid();
                $list->prepare_items();
                ob_start();
                echo $list->views();
                echo $list->display();
                $html_line .= ob_get_contents();
                ob_end_clean();
            }

            return $html_line;

        }

        function local_app_app_store_info_android(){

            $app_name = get_option('local-app-android-app-name');
            $app_title = get_option('local-app-android-app-title');
            $full_description = get_option('local-app-android-app-full-description');
            $short_description = get_option('local-app-android-app-short-description');
            $app_website = get_option('local-app-android-app-website');
            $app_privacy = get_option('local-app-android-app-privacy');
            $app_email = get_option('local-app-android-app-email');

            $use_app_name = '';

            if (get_option('local-app-android-use-tsl-name')) $use_app_name = 'checked';

            $android_meta_info = '<a href="https://play.google.com/about/storelisting-promotional/metadata/" target="_blank">Meta Data Info</a><br><a href="https://play.google.com/about/developer-content-policy/" target="_blank">Content Policy</a>';

            $html_line = '<table class="local-app-setup-store-table">';
            $html_line .= '<tr><td class="local-app-setup-store-left">App Name:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-android-app-name" maxlength="12" value="'.$app_name.'"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-android-app-name').'"  class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">App Title:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" id="local-app-android-app-title" maxlength="30" value="'.$app_title.'"></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-android-app-title').'"  class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left" colspan="2">Use "<span id="local-app-use-ios-app-tsl-title">My App Title</span> by TSL" if selected app title is taken?:&nbsp;&nbsp;<input type="checkbox" id="local-app-android-use-tsl-title" value="1" ' . $use_app_name . '></td><td>&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-android-app-tsl-title').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left" style="vertical-align: text-top">Short Description:</td><td class="local-app-app-store-field" class="local-app-setup-store-center"><textarea id="local-app-app-short-description" maxlength="80" rows="8" cols="50">'.$short_description.'</textarea></td><td style="vertical-align: text-top">&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-android-app-short-description').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left" style="vertical-align: text-top">Full Description:</td><td class="local-app-app-store-field" class="local-app-setup-store-center"><textarea id="local-app-app-full-description" maxlength="4000" rows="15" cols="50">'.$full_description.'</textarea></td><td style="vertical-align: text-top">&nbsp;<span title="'.local_app_get_qtip_message('local-app-help-android-app-full-description').'" class="fa fa-question-circle local-app-help-button">&nbsp;</span><br><br>'.$android_meta_info.'</td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Website URL:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field local-app-app-store-field-optional" type="text" size="50" id="local-app-android-app-website" value="'.$app_website.'"></td><td>&nbsp;<span id="local-app-help-android-app-website" title="'.local_app_get_qtip_message('local-app-help-android-app-website').'" class="fa fa-question-circle local-app-help-icon">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Privacy Policy URL:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field local-app-app-store-field-optional" type="text" size="50" id="local-app-android-app-privacy" value="'.$app_privacy.'"></td><td>&nbsp;<span id="local-app-help-android-app-privacy" title="'.local_app_get_qtip_message('local-app-help-android-app-privacy').'" class="fa fa-question-circle local-app-help-icon">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Email:</td><td class="local-app-setup-store-center"><input class="local-app-app-store-field" type="text" size="50" id="local-app-android-app-email" value="'.$app_email.'"></td><td>&nbsp;<span id="local-app-help-android-app-email" title="'.local_app_get_qtip_message('local-app-help-android-app-email').'" class="fa fa-question-circle local-app-help-icon">&nbsp;</span></td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Application Type:</td><td class="local-app-setup-store-center">'.$this->create_android_category_dd(true).'</td><td>&nbsp;</td></tr>';
            $html_line .= '<tr><td class="local-app-setup-store-left">Category:</td><td class="local-app-setup-store-center">'.$this->create_android_category_dd(false).'</td><td>&nbsp;</td></tr>';
            $html_line .= '</table>';

            return $html_line;
        }

        function local_app_featured_graphic(){

            $image_1 = '';
            $image_info_1 = wp_get_attachment_image_src(get_option('local-app-images-featured-graphic'), 'full');
            if ($image_info_1) $image_1 = '<img src="' . $image_info_1[0] . '" style="width:512px;height:250px;">';

            $html_line = '<table class="local-app-table-featured-graphic">';
            $html_line .= '<tr>';
            $html_line .= '<td>';
            $html_line .= '<div class="local-app-images-featured-graphic">' . $image_1 . '</div>';
            $html_line .= '<div class="local-app-images-featured-graphic-subtitle">500 x 1,024 pixels</div>';
            $html_line .= '<div class="local-app-images-featured-graphic-subtitle">Click Image to Upload</div>';
            $html_line .= '<div class="local-app-images-featured-graphic-subtitle"><a href="https://www.norio.be/android-feature-graphic-generator/" target="_blank">Create One Here</a></div>';
            $html_line .= '</td>';
            $html_line .= '</tr>';
            $html_line .= '</table>';

            return $html_line;
        }

        function local_app_launch_screen($os_type = 1){

            $image_1 = '';
            $image_2 = '';

            if(is_numeric(get_option('local-app-images-launch-screen-1'))) {
                $launch_screen_1_id = get_option('local-app-images-launch-screen-1');
                $image_info_1 = wp_get_attachment_image_src(get_option('local-app-images-launch-screen-1'), 'full');
                if ($image_info_1) $image_1 = '<img src="' . $image_info_1[0] . '?session='.time().'" style="width:268px;height:476px;">';
            }else{
                $launch_screen_1_id = attachment_url_to_postid(get_option('local-app-images-launch-screen-1'));
                if(get_option('local-app-images-launch-screen-1')) $image_1 = '<img src="' . get_option('local-app-images-launch-screen-1') . '?session='.time().'" style="width:268px;height:476px;">';
            }

            if(is_numeric(get_option('local-app-images-launch-screen-2'))) {
                $launch_screen_2_id = get_option('local-app-images-launch-screen-2');
                $image_info_2 = wp_get_attachment_image_src(get_option('local-app-images-launch-screen-2'), 'full');
                if ($image_info_2) $image_2 = '<img src="' . $image_info_2[0] . '?session='.time().'" style="width:277px;height:600px;">';
            }else{
                $launch_screen_2_id = attachment_url_to_postid(get_option('local-app-images-launch-screen-2'));
                if(get_option('local-app-images-launch-screen-2')) $image_2 = '<img src="' . get_option('local-app-images-launch-screen-2') . '?session='.time().'" style="width:277px;height:600px;">';
            }
            $launch_screen_bg = get_option('ios_app_launch_screen_bg');

            $html_line = '<table class="local-app-table-launch-screen">';
            $html_line .= '<tr><td>Background Color:&nbsp;<input type="color" class="mam-color-picker" id="local-app-ios-launch-screen-bg" name="local-app-ios-launch-screen-bg" value="'.$launch_screen_bg.'">&nbsp;<span title="'.local_app_get_qtip_message("local-app-help-ios-launch-screen-bg").'"  class="fa fa-question-circle local-app-help-button">&nbsp;</span></td></tr>';
            $html_line .= '<tr>';
            $html_line .= '<td>';
            $html_line .= '<div class="local-app-images-launch-screen-1">' . $image_1 . '</div>';
            $html_line .= '<input type="hidden" name="local-app-images-launch-screen-1" value="'.$launch_screen_1_id.'">';
            $html_line .= '<div class="local-app-images-launch-screen-subtitle">640px x 1,136px</div>';
            $html_line .= '<div class="local-app-images-launch-screen-subtitle">Click Image to Upload</div>';
            $html_line .= '</td>';
            if($os_type == 1) {
                $html_line .= '<td>';
                $html_line .= '<div class="local-app-images-launch-screen-2">' . $image_2 . '</div>';
                $html_line .= '<input type="hidden" name="local-app-images-launch-screen-2" value="'.$launch_screen_2_id.'">';
                $html_line .= '<div class="local-app-images-launch-screen-subtitle">1,125px x 2,436px</div>';
                $html_line .= '<div class="local-app-images-launch-screen-subtitle">Click Image to Upload</div>';
                $html_line .= '</td>';
            }
            $html_line .= '<td><input type="button" class="button-primary" id="local-app-generate-launch-screen" value="Generate Launch Screen"></td>';
            $html_line .= '</tr>';
            $html_line .= '</table>';

            return $html_line;
        }

        function local_app_app_icon_ios()
        {


            $image = '';
            $app_icon_bg = get_option('local-app-app-icon-bg');
            $app_icon_id = get_option('local-app-images-app-icon');
            $image_info = wp_get_attachment_image_src($app_icon_id, 'full');
            if ($image_info) $image = '<img src="' . $image_info[0] . '" style="width:250px;height:250px;">';

            $html_line = '';

            $html_line .= '<table class="local-app-setup-store-table"><tbody><tr><td>Background Color:&nbsp;<input type="color" class="mam-color-picker" id="local-app-ios-app-icon-bg" name="local-app-ios-app-icon-bg" value="'.$app_icon_bg.'">&nbsp;<span class="fa fa-question-circle local-app-help-button" data-hasqtip="15" title="This will be used for background of app icon.">&nbsp;</span></td></tr></tbody></table>';

            $html_line .= '<div class="local-app-images-app-icon">' . $image . '</div>';
            $html_line .= '<input type="hidden" name="local-app-images-app-icon" value="'.$app_icon_id.'">';
            $html_line .= '<div class="local-app-images-app-icon-subtitle">1024 x 1024 pixels (PNG image)</div>';
            $html_line .= '<div class="local-app-images-app-icon-subtitle">This image will be automatically scaled for use with all devices and app store profiles.</div>';

            return $html_line;
        }

        function local_app_app_icon_android()
        {


            $image = '';
            $image_info = wp_get_attachment_image_src(get_option('local-app-images-android-app-icon'), 'full');
            if ($image_info) $image = '<img src="' . $image_info[0] . '" style="width:250px;height:250px;">';

            $html_line = '';

            $html_line .= '<div class="local-app-images-android-app-icon">' . $image . '</div>';
            $html_line .= '<div class="local-app-images-app-icon-subtitle">512 x 512 pixels (PNG image, no Alpha)</div>';
            $html_line .= '<div class="local-app-images-app-icon-subtitle">This image will be automatically scaled for use with all devices and profiles.</div>';

            return $html_line;
        }

        public function get_product_id($api_key){

            return substr($api_key , 0, 6);
        }

        public function local_app_activate_api_key($api_key , $email  , $this_product_id = null ){

            $platform = get_site_url();

            $api_password = get_option('local-app-api-password');

            if(!$api_password){
                $api_password = uniqid();
                update_option('local-app-api-password' , $api_password );
            }

            if($this_product_id == null){
                $product_id = $this->get_product_id($api_key);
            }else{
                $product_id = $this_product_id;
            }

            $querystring = 'https://tinyscreenlabs.com/?wc-api=software-api&request=activation&license_key='.$api_key.'&email='.$email.'&product_id='.$product_id.'&platform='.$platform.'instance='.$api_password;

            $response = $this->local_app_get_api_info($querystring);

            if($response['activated']){
                $querystring = 'https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=tsl_update_api&api_action=add&api_email='.$email.'&api_key='.$api_key.'&ajax_url='.admin_url('admin-ajax.php');

                $response = wp_remote_get($querystring);

                if (is_array($response) && !is_wp_error($response)) {
                    $this->update_login_url();

                    //if(!get_option('local-app-api')) local_app_update_content_to_prod(false);

                    update_option('local-app-api-email', $email);
                    if($this_product_id == null) {
                        update_option('local-app-api', $api_key);
                    }
                    update_option('local-app-api-password', $response['instance']);
                    return array('status' => 'success', 'response' => $response);
                }
            }else{
                return array('status' => $response['error']);
            }

        }

        public function local_app_check_api_key(){

//            if(local_app_get_api_key()){
//
//                $email = get_option('local-app-api-email');
//                $api_key = local_app_get_api_key();
//                $api_password = get_option('local-app-api-password');
//                $platform = get_site_url();
//
//                $product_id = $this->get_product_id($api_key);
//
//                $querystring = 'https://tinyscreenlabs.com/?wc-api=software-api&request=check&license_key='.$api_key.'&email='.$email.'&product_id='.$product_id.'&platform='.$platform.'instance='.$api_password;
//                $response = $this->local_app_get_api_info($querystring);
//
//                if($response['success'] ){
//                    $this->update_login_url();
//                    return true;
//                }
//
//            }
//            return false;
            return true;
        }

        public function local_app_deactivate_api_key(){

//            if(local_app_get_api_key()) {
//
//                $email = get_option('local-app-api-email');
//                $api_key = local_app_get_api_key();
//                $api_password = get_option('local-app-api-password');
//                $platform = get_site_url();
//
//                $product_id = $this->get_product_id($api_key);
//
//                $querystring = 'https://tinyscreenlabs.com/?wc-api=software-api&request=deactivation&license_key='.$api_key.'&email='.$email.'&product_id=.'.$product_id.'&platform='.$platform.'instance='.$api_password;
//                $response = $this->local_app_get_api_info($querystring);
//
//                if($response['reset']) {
//                    $querystring = 'https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=tsl_update_api&api_action=delete&api_email='.$email.'&api_key='.$api_key.'&ajax_url='.admin_url('admin-ajax.php');
//
//                    $response = wp_remote_get($querystring);
//
//                    if (is_array($response) && !is_wp_error($response)) {
//                        update_option('local-app-api-email', '');
//                        update_option('local-app-api', '');
//                        update_option('local-app-api-password', '');
//                        return array('status' => 'success');
//                    }else{
//                        return array('status' => 'Failed');
//                    }
//                }else{
//                    $this->update_login_url(true);
//                    return array('status' => 'Failed');
//                }
//            }
            return array('status' => 'success');
        }

        public function update_login_url($deacivate = false){
            if($deacivate){
                update_option( 'local-app-ajax-url' , '' );
            }else {
                $account_code = local_app_get_api_key();
                $querystring = 'https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=tsl_update_ajax_url&api_key='.$account_code.'&ajax_url='.admin_url('admin-ajax.php');

                $response = wp_remote_get($querystring);

                if (is_array($response) && !is_wp_error($response)) {
                    update_option('local-app-login-alert-sent' , '');
                    update_option('local-app-ajax-url' ,admin_url('admin-ajax.php') );
                }
            }
        }


        public function local_app_get_api_info($querystring){

            $response = wp_remote_get($querystring);

            if (is_array($response) && !is_wp_error($response)) {
                $data = json_decode(wp_remote_retrieve_body( $response ) , true);
                return $data;

            } else {
                return array();
            }
        }

        public function create_android_category_dd($primary = true){

            $selected = null;
            if($primary){
                $html_line = '<select id="local-app-primary-category" class="local-app-app-store-field" >';
                if(get_option('local-app-android-primary-cat')) $selected = get_option('local-app-android-primary-cat');
            }else{
                $html_line = '<select id="local-app-secondary-category" class="local-app-app-store-field" >';
                if(get_option('local-app-android-secondary-cat')) $selected = get_option('local-app-android-secondary-cat');
            }
            if($selected == null) $selected = 0;

            $options = $this->get_android_catagory_options($primary);

            foreach($options as $index => $value){
                $selected_text = '';
                if($selected == $index) $selected_text = ' selected';
                $html_line .= '<option value="'.$index.'" '.$selected_text.'>'.$value.'</option>';
            }

            $html_line .= '</select>';

            return $html_line;

        }

        public function get_android_catagory_options($primary = true){

            if($primary) {
                $options[] = 'Applications';
                $options[] = 'Games';

            }else{

                $options[] = 'Art & Design';
                $options[] = 'Auto & Vehicles';
                $options[] = 'Beauty';
                $options[] = 'Books & Reference';
                $options[] = 'Business';
                $options[] = 'Comics';
                $options[] = 'Communication';
                $options[] = 'Dating';
                $options[] = 'Education';
                $options[] = 'Entertainment';
                $options[] = 'Events';
                $options[] = 'Finance';
                $options[] = 'Food & Drink';
                $options[] = 'Health & Fitness';
                $options[] = 'House & Home';
                $options[] = 'Libraries & Demo';
                $options[] = 'Lifestyle';
                $options[] = 'Maps & Navigation';
                $options[] = 'Medical';
                $options[] = 'Music & Audio';
                $options[] = 'News & Magazines';
                $options[] = 'Parenting';
                $options[] = 'Personalization';
                $options[] = 'Photography';
                $options[] = 'Productivity';
                $options[] = 'Shopping';
                $options[] = 'Social';
                $options[] = 'Sports';
                $options[] = 'Tools';
                $options[] = 'Travel & Local';
                $options[] = 'Video Players & Editors';
                $options[] = 'Weather';
            }

            return $options;
        }

        public function create_category_dd($primary = true){

            $selected = -1;
            $disabled = -1;
            if($primary){
                $html_line = '<select id="local-app-primary-category" class="local-app-app-store-field" >';
                if(get_option('local-app-ios-primary-cat')) $selected = get_option('local-app-ios-primary-cat');
                if(get_option('local-app-ios-secondary-cat')) $disabled = get_option('local-app-ios-secondary-cat');
            }else{
                $html_line = '<select id="local-app-secondary-category" class="local-app-app-store-field" >';
                if(get_option('local-app-ios-secondary-cat')) $selected = get_option('local-app-ios-secondary-cat');
                if(get_option('local-app-ios-primary-cat')) $disabled = get_option('local-app-ios-primary-cat');
                if($selected < 0) $selected = 1;
                if($disabled < 0) $disabled = 0;
            }

            $options = $this->get_ios_catagory_options();

            foreach($options as $index => $value){
                $selected_text = '';
                $disabled_text = '';
                if($selected == $index) $selected_text = ' selected';
                if($disabled == $index) $disabled_text = ' disabled="disabled"';
                $html_line .= '<option value="'.$index.'" '.$selected_text.' '.$disabled_text.'>'.$value.'</option>';
            }

            $html_line .= '</select>';

            return $html_line;

        }

        public function get_ios_catagory_options(){

            $options[] = 'Book';
            $options[] = 'Business';
            $options[] = 'Catalogs';
            $options[] = 'Education';
            $options[] = 'Entertainment';
            $options[] = 'Finance';
            $options[] = 'Food & Drink';
            $options[] = 'Games';
            $options[] = 'Health & Fitness';
            $options[] = 'Lifestyle';
            $options[] = 'Medical';
            $options[] = 'Music';
            $options[] = 'Navigation';
            $options[] = 'News';
            $options[] = 'Newsstand';
            $options[] = 'Photo & Video';
            $options[] = 'Productivity';
            $options[] = 'Reference';
            $options[] = 'Social Networking';
            $options[] = 'Sports';
            $options[] = 'Travel';
            $options[] = 'Utilities';
            $options[] = 'Weather';

            return $options;
        }

    }
}
