<?php

class tsl_mam_mail_manager
{

    private $from_name;
    private $from_email;
    public $plugin_slug = 'mam-mail-manager';
    public $plugin_name = 'Mail Manager';

    public static function init()
    {
        $class = __CLASS__;
        new $class;
    }

    public function __construct()
    {

        add_filter( 'wp_mail_from_name', array( $this, 'from_mail_name'));
        add_filter( 'wp_mail_from', array( $this, 'from_mail_email' ));
        add_filter( 'admin_menu' , array( $this , 'mam_admin_menu_manager' ) );
        add_filter( 'mam_mail_handler', array($this, 'mail_cron'));
        add_action( 'wp_mail_failed', array( $this, 'log_mailer_errors' ), 10, 1);

    }

    public function mam_admin_menu_manager()
    {

        add_submenu_page('local_app_setup', __($this->plugin_name, $this->plugin_slug), __($this->plugin_name, $this->plugin_slug), 'manage_options', $this->plugin_slug, array($this, 'add_settings_page'));

    }

    function add_settings_page()
    {

		$admin_settings_page = new mam_mail_manager_admin_settings();
		echo $admin_settings_page->create_admin_page();

	}

    function log_mailer_errors( $wp_error )
    {
        global $tsl_mail_error;
        $tsl_mail_error =  $wp_error->get_error_message();
    }

    function mam_cron_manager( $data_array )
    {
        $data_array[] = array('action' => 'mam_mail_handler', 'name' => 'Send notifications - Email and PNs', 'frequency' => '1' );
        return $data_array;
    }

    function from_mail_name($name) {
        if($this->from_name) return $this->from_name;
        return $name;
     }

     function from_mail_email( $original_email_address ) {
        if($this->from_email) return $this->from_email;
        return $original_email_address;
    }

    function mail_cron()
    {

        global $wpdb;

        $email_table = $wpdb->prefix . 'tsl_mail_queue';
        $attachment_table = $wpdb->prefix . 'tsl_mail_attachments';

        $send_info = array();
        $emails = $wpdb->get_results("SELECT * FROM $email_table WHERE send_status IS NULL OR send_status = 20 LIMIT 50", ARRAY_A);

        foreach( $emails as $email ){

            //set field to sending to datetime stamp

            $is_sent = $wpdb->get_results("SELECT * FROM $email_table WHERE this_index = ". $email['this_index'] . " AND ( send_status IS NULL OR send_status = 20 )", ARRAY_A);
            if( sizeof( $is_sent) == 0 ) continue;

            //add retry delay here
            if( $email['send_status'] == 20 )
            {

                $interval = 10;
                if( get_option( 'mam-mail-retry-interval' )) $interval = get_option( 'mam-mail-retry-interval' );

                $last_touched = date( 'U', strtotime( $email['modified']  . ' +'.$interval.' minutes' ));

                if( $last_touched > date( 'U' , strtotime( current_time( 'mysql' ))) ) continue;

            }

            $this->from_email = null;
            $this->from_name = null;

            $attachments = $wpdb->get_results( "SELECT * FROM $attachment_table WHERE email_index = " . $email['this_index'], ARRAY_A);

            $attach_array = array();

            foreach ( $attachments as $attachment ){
                $attach_array[] = $attachment['attachment'];
            }

            if( $email['from_name'] ) $this->from_name = $email['from_name'];
            if( $email['from_email'] ) $this->from_email= $email['from_email'];

            $success = wp_mail($email['email'], $email['subject'], $email['content'], array( $email['headers'] ), $attach_array );

            if( $success ) {
                $wpdb->update($email_table, array( 'send_status' => 2), array('this_index' => $email['this_index']));
                $send_info['email']++;
            }else{
                global $tsl_mail_error;

                $retries = 3;
                if( get_option( 'mam-mail-num-retries' )) $retries = get_option( 'mam-mail-num-retries' );

                if( $email['retry_count'] > $retries ){
                    $wpdb->update($email_table, array( 'send_status' => 30, 'last_error' => $tsl_mail_error ), array('this_index' => $email['this_index']));
                }else{
                    if( ! $email['retry_count'] ) $email['retry_count'] = 0;
                    $retry_count = $email['retry_count'] + 1;
                    $wpdb->update($email_table, array( 'send_status' => 20, 'retry_count' => $retry_count, 'last_error' => $tsl_mail_error , 'modified' =>  current_time( 'mysql' ) ), array('this_index' => $email['this_index']));
                }

            }

        }

        $notification_table = $wpdb->prefix . 'tsl_pn_text_queue';

        $notifications = $wpdb->get_results("SELECT * FROM $notification_table WHERE send_status IS NULL LIMIT 20", ARRAY_A);

        foreach( $notifications as $notification )
        {

            $this_array = array();

            if( $notification['send_pn'] == 1 ) {
                $this_array['title'] = wp_strip_all_tags($notification['subject']);
                $this_array['message'] = $notification['content'];
                if( $notification['is_silent'] == 1){
                    $this_array['target'] = 'refresh';
                }else {
                    $this_array['target'] = get_permalink($notification['target']);
                }
                $this_array['user_id'] = $notification['recipient_id'];
                $this_array['immediate'] = true;
                $this_array['is_silent'] = $notification['is_silent'];

                $this_pn_array = array();

                $this_pn_array[] = $this_array;

                do_action('mam_notification_send_pn', $this_pn_array);

                $send_info['push_notifications']++;

            }

            if( $notification['send_text'] == 1 ) {

                $to = '1' . str_replace('-', '', get_user_meta( $notification['recipient_id'], 'billing_phone', true));

                global $sms;
                if($sms) {
                    $sms->to = array($to);
                    $sms->msg = $notification['content'];
                    $sms->SendSMS();

                    $send_info['texts']++;
                }
            }

            $wpdb->update( $notification_table, array('send_status' => 2 ),  array('this_index' => $notification['this_index']));

        }

        $json['status'] = 'success';

        return $json;

    }

}
