<?php

if ( ! class_exists( 'WP_List_Table' ) ) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class Custom_Notifications_List extends WP_List_Table {
    /**
     * Prepare the items for the table to process
     *
     * @return Void
     */
    public function prepare_items()
    {
        $this->process_delete_action();

        $columns = $this->get_columns();
        $hidden = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();

        $perPage = 20;
        $currentPage = $this->get_pagenum();
        $totalItems = $this->get_total_records();

        $this->set_pagination_args( array(
            'total_items' => $totalItems,
            'per_page'    => $perPage
        ) );

        $data = $this->get_records((($currentPage-1)*$perPage),$perPage);

        $this->_column_headers = array($columns, $hidden, $sortable);
        $this->items = $data;
    }

    function process_delete_action(){
        if(isset($_REQUEST['nmNonce']) && $_REQUEST['action'] == 'delete' && $_REQUEST['id'] > 0 && $_REQUEST['nmNonce'] != ''){
            if ( ! wp_verify_nonce( $_REQUEST['nmNonce'], 'notificationsManagerNonce' ) || ! isset( $_REQUEST['nmNonce'] )){
                wp_redirect('?page=mam-notification-manager&tab=notifications-listing');
                exit;
            }

            global $wpdb;
            $table_name = $wpdb->prefix . 'mam_custom_notifications';
            $wpdb->query("DELETE FROM $table_name WHERE id='".$_REQUEST["id"]."'");
            if($wpdb->last_error === ''){
                wp_redirect('?page=mam-notification-manager&tab=notifications-listing');
                exit;
            }
        }
    }

    /**
     * Override the parent columns method. Defines the columns to use in your listing table
     *
     * @return Array
     */
    public function get_columns()
    {
        $columns = array(
            'title'      => __( 'Title', 'mam-notifications-manager' ),
            'trigger'    => __( 'Trigger', 'mam-notifications-manager' ),
            'frequency' => __( 'Frequency', 'mam-notifications-manager' ),
        );

        return $columns;
    }

    /**
     * Define which columns are hidden
     *
     * @return Array
     */
    public function get_hidden_columns()
    {
        return array();
    }

    /**
     * Define the sortable columns
     *
     * @return Array
     */
    public function get_sortable_columns()
    {
        /*return array('title' => array('title', false));*/
        return array();
    }

    /**
     * Define the title columns row actions
     *
     * @return Array
     */
    public function column_title( $item )
    {

        $output = '<a href="admin.php?page=mam-notification-manager&tab=notifications-listing&action=edit&id=' . $item['id'] . '">' . $item['title'] . '</a> ';

        $actions = array(
            'edit' => '<a href="admin.php?page=mam-notification-manager&tab=notifications-listing&action=edit&id=' . $item['id'] . '" title="' . __('Edit', 'notifications-manager') . '">' . __('Edit', 'notifications-manager') . '</a>',
            'trash' => '<a class="wdt-submit-delete" title="' . __('Delete', 'wpdatatables') . '" href="'. wp_nonce_url('admin.php?page=mam-notification-manager&tab=notifications-listing&action=delete&id=' . $item['id'] . '', 'notificationsManagerNonce', 'nmNonce' ) .'" rel="' . $item['id'] . '">' . __('Delete', 'notifications-manager') . '</a>'
        );

        $row_actions = array();

        foreach ( $actions as $action => $link ) {
            $row_actions[] = '<span class="' . esc_attr( $action ) . '">' . $link . '</span>';
        }

        $output .= '<div class="row-actions">' . implode( ' | ', $row_actions ) . '</div>';

        return $output;

    }

    /**
     * Get the total records from table
     *
     * @return Array
     */
    private function get_total_records()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'mam_custom_notifications';
        $data = $wpdb->get_var("SELECT COUNT(*) FROM $table_name");
        return $data;
    }

    /**
     * Get the table data
     *
     * @return Array
     */
    private function get_records($offset, $limit)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'mam_custom_notifications';
        $data = $wpdb->get_results("SELECT `id`, `title`, `trigger`, `frequency` FROM $table_name LIMIT ".$offset.", ".$limit, ARRAY_A);
        return $data;
    }

    /**
     * Define what data to show on each column of the table
     *
     * @param  Array $item        Data
     * @param  String $column_name - Current column name
     *
     * @return Mixed
     */
    public function column_default( $item, $column_name )
    {
        switch( $column_name ) {
            case 'title':
            case 'trigger':
            case 'frequency':
                return $item[ $column_name ];

            default:
                return print_r( $item, true ) ;
        }
    }
}