<?php
if ( ! class_exists( 'WP_List_Table' ) )
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );

class mam_notification_sent_grid extends WP_List_Table {

	public $current_page;

    function __construct() {
		parent::__construct( array(
			'singular' => 'post',
			'plural' => 'posts',
			'ajax' => true,
			'screen' => 'interval_list') );
	}

    function prepare_items() {

		$per_page = 10;
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = array();

        $this->_column_headers = array($columns, $hidden, $sortable);

		$args = array(
			'posts_per_page' => $per_page,
			'orderby' => 'plugin',
			'order' => 'ASC',
			'offset' => ( $this->get_pagenum() - 1 ) * $per_page );

		if ( ! empty( $_REQUEST['s'] ) )
			$args['s'] = $_REQUEST['s'];

		if ( ! empty( $_REQUEST['orderby'] ) ) {
			if ( 'left_column' == $_REQUEST['left_column'] )
				$args['orderby'] = 'left_column';
		}

		if ( ! empty( $_REQUEST['order'] ) ) {
			if ( 'asc' == strtolower( $_REQUEST['order'] ) )
				$args['order'] = 'ASC';
			elseif ( 'desc' == strtolower( $_REQUEST['order'] ) )
				$args['order'] = 'DESC';
		}

        $per_page     = $this->get_items_per_page( 'analytics_per_page', $per_page );
        $current_page = $this->get_pagenum();

		$total_items = $this->get_grid_data_count();
		$total_pages = ceil( $total_items / $per_page );

		$this->set_pagination_args( array(
            'total_items' => $total_items,
            'per_page'    => $per_page
        ) );

        $this->items = $this->get_grid_data( $per_page , $current_page);

	}

	function get_columns() {

        $columns = array(
            'column-one' => __( 'Date Sent', 'local-app-manger' ),
            'column-two' =>   __( 'Title', 'local-app-manger' ),
			'column-three' =>   __( 'To', 'local-app-manger' ),
            'column-four' =>   __( 'Type', 'local-app-manger' )
        );

        return $columns;

	}

	function get_sortable_columns() {
		$columns = array(
			'mam-ninja-link' => array( 'mam-ninja-link', true ) );

		return $columns;
	}

	function get_bulk_actions() {
		$actions = array();

		return $actions;
	}

	function column_default( $item, $column_name ) {
		switch( $column_name ) {
			case 'column-one':
			case 'column-two':
			case 'column-three':
    	    case 'column-four':
			return $item[ $column_name ];
            default:
              return print_r( $item, true ) ;
            }
	}

    function get_grid_data( $per_page , $current_page){

        $items = array();

        global $wpdb;

        $record_offset = $current_page * $per_page - $per_page;

        $table_name = $wpdb->prefix . 'mam_email_history';
        $posts = $wpdb->get_results( 'SELECT * FROM ' . $table_name . ' ORDER BY notification_date DESC LIMIT '. $record_offset . ' , ' . $per_page, ARRAY_A);

        foreach($posts as $post ){

            $from = '';
            if( $post['user_id'] > 0 ){
                $user = get_userdata( $post['user_id'] );
                if($user) $from = $user->display_name;
            }

            $to = $post['to_email'];
            if( empty($post['to_email'])){
                if( $post['recipient_id'] > 0 ){
                    $user = get_userdata( $post['recipient_id'] );
                    if($user) $to = $user->display_name;
                }
            }


            $items[] = array(
                'column-one' => $post['notification_date'],
                'column-two' => $post['title'],
                'column-three' => $to,
                'column-four' => strtoupper( $post['msg_type'] ),
            );
        }

        return $items;

    }

    function get_users_with_role( $slug ){

        return 0;
    }

	function get_grid_data_count(){

        global $wpdb;

        $table_name = $wpdb->prefix . 'mam_email_history';
        $posts = $wpdb->get_results( 'SELECT * FROM ' . $table_name , ARRAY_A);

        if(is_array($posts)) {
            return sizeof($posts);
        }
        return 0;

	}

}
