<?php

use chillerlan\QRCode\QRCode;
use chillerlan\QRCode\QROptions;
use chillerlan\QRCODe\Output\QRCodeOutputException;
use chillerlan\QRCODe\Output\QROutputInterface;
//use chillerlan\QRCode\Output\{QRCodeOutputException, QROutputInterface};
use chillerlan\QRCode\Data\QRMatrix;
use chillerlan\QRCode\Common\EccLevel;

class mam_qr_code_manager
{
    private $user;
    private $user_id;

    function __construct()
    {

        do_action('mam_update_current_user');
        global $mam_user, $mam_user_id;

        if ($mam_user_id > 0) {
            $this->user = $mam_user;
            $this->user_id = $mam_user_id;
        }

    }

    function hexToRgb($hex, $alpha = false) {
       $hex      = str_replace('#', '', $hex);
       $length   = strlen($hex);
       $rgb['r'] = hexdec($length == 6 ? substr($hex, 0, 2) : ($length == 3 ? str_repeat(substr($hex, 0, 1), 2) : 0));
       $rgb['g'] = hexdec($length == 6 ? substr($hex, 2, 2) : ($length == 3 ? str_repeat(substr($hex, 1, 1), 2) : 0));
       $rgb['b'] = hexdec($length == 6 ? substr($hex, 4, 2) : ($length == 3 ? str_repeat(substr($hex, 2, 1), 2) : 0));
       if ( $alpha ) {
          $rgb['a'] = $alpha;
       }
       return $rgb;
    }

    function create_qrcode( $url , $image_id = '', $icon_image = '' , $dot_red = '#000000')
    {

        if( $image_id == '' ) {
            $image_id = uniqid();
        }

        global $mam_mam_plugin_folder_path, $mam_mam_plugin_folder_url;

        $options = new QROptions;

        $options->outputType = QROutputInterface::GDIMAGE_PNG;
        $options->quality = 100;
        $options->eccLevel = EccLevel::H;

        $options->qrCodeHeight = 1200;
        $options->qrCodeWidth = 1200;

        $options->scale = 5;

        $options->keepAsSquare = [
            QRMatrix::M_FINDER_DARK,
            QRMatrix::M_FINDER_DOT,
            QRMatrix::M_ALIGNMENT_DARK,
        ];

        $rgb = $this->hexToRgb($dot_red);
        if( $rgb['r'] == 0 ) $rgb['r'] = 1;
        if( $rgb['g'] == 0 ) $rgb['g'] = 1;
        if( $rgb['b'] == 0 ) $rgb['b'] = 1;

        $dot_red = (int)$rgb['r'];
        $dot_green = (int)$rgb['g'];
        $dot_blue = (int)$rgb['b'];

        $reg_red = (int)$rgb['r'];
        $reg_green = (int)$rgb['g'];
        $reg_blue = (int)$rgb['b'];

        $dark_red = (int)$rgb['r'];
        $dark_green = (int)$rgb['g'];
        $dark_blue = (int)$rgb['b'];

        mamdebug('qrcode', $dot_red . ' ' . $dot_green . ' ' . $dot_blue);

        $options->moduleValues = [
            QRMatrix::M_FINDER_DARK => [$dot_red, $dot_green, $dot_blue], // dark (true)
            QRMatrix::M_FINDER_DOT     => [$reg_red, $reg_green, $reg_blue], // finder dot, dark (true)
//                    QRMatrix::M_FINDER         => [$dot_red, $dot_green, $dot_blue], // light (false)
            QRMatrix::M_ALIGNMENT_DARK => [$dark_red, $dark_green, $dark_blue],
//                    QRMatrix::M_ALIGNMENT      => [$dot_red, $dot_green, $dot_blue],
            QRMatrix::M_DATA_DARK => [$dark_red, $dark_green, $dark_blue],  //the dots on teh code
//                    QRMatrix::M_TIMING => [$dark_red, $dark_green, $dark_blue],
//                    QRMatrix::M_FORMAT => [$dark_red, $dark_green, $dark_blue],
            QRMatrix::M_TIMING_DARK => [$dark_red, $dark_green, $dark_blue],
            QRMatrix::M_FORMAT_DARK => [$dark_red, $dark_green, $dark_blue]
        ];

        $imagePath = $mam_mam_plugin_folder_path . 'images/qr_' . $image_id . '.png';
        if (file_exists($imagePath)) {
            if (unlink($imagePath)) {
                mamdebug('qrcode', 'Deleted old file');
            } else {
                mamdebug('qrcode', 'Could not delete old file');
            }
        }

        $qrCode = new QRCode($options);
        $qrCode->render($url, $imagePath);

        mamdebug('path', $mam_mam_plugin_folder_url . 'images/qr_' . $image_id . '.png');

        return $mam_mam_plugin_folder_url . 'images/qr_' . $image_id . '.png';
    }

}
