<?php

use Pushok\AuthProvider;
use Pushok\Client;
use Pushok\Notification;
use Pushok\Payload;
use Pushok\Payload\Alert;

use Jose\Component\Core\AlgorithmManager;
use Jose\Component\Core\JWK;
use Jose\Component\Signature\JWSBuilder;
use Jose\Component\Signature\Serializer\CompactSerializer;
use Jose\Component\Signature\Algorithm\RS256;
use GuzzleHttp\Client as GuzzleHttpClient;
use GuzzleHttp\Exception\RequestException;

class mam_main_send_ios_push_notification
{

    private $ranProd = false;
    private $ranTest = false;


    function send_voip_notification( $args = array() )
    {

        $this->ranProd = true;

        $message = 'Calling';
        $title = 'Incoming Call';
        $user_id = 0;
        $data = array();

        if( isset($args['user_id'] )) $user_id = $args['user_id'];
        if( isset($args['message'] )) $message = $args['message'];
        if( isset($args['title'] )) $title = $args['title'];
        if( isset($args['data'] )) $data = $args['data'];

        if( $user_id == 0 ) return;

        $pem_cert_file = $this->get_p8_file();
        if( ! $pem_cert_file ) return;

        $voip_token = get_user_meta( $user_id, 'mam_ios_voip_token', true );
        if(  ! $voip_token )
        {
            $voip_token = get_user_meta( $user_id, 'mam_app_android_pn_token', true );
            $this->SendAndroidPushNotifications( $message, '', $title, $voip_token, $data, true );

            return;
        }

        if( ! get_option( 'ios_pn_key_id') ) {
            update_option('ios_pn_key_id', '569JPQ4BRJ');
            update_option('ios_pn_team_id', '977BJ3E5P8');
            update_option('ios_pn_app_bundle_id', 'tinyscreenlabs.com.57fbd2a7218af');
        }

        if( get_option( 'ios_pn_key_id') && get_option( 'ios_pn_team_id') && get_option( 'ios_pn_app_bundle_id')  ){
            $options['key_id'] = get_option( 'ios_pn_key_id');
            $options['team_id'] = get_option( 'ios_pn_team_id');
            $options['app_bundle_id'] = get_option( 'ios_pn_app_bundle_id');
            $options['private_key_path'] = $pem_cert_file;
            $options['private_key_secret'] = 'a101b102c103';

        }else{
            mamdebug( 'VOIP Failed', 'No Key Data ' . $pem_cert_file );
            return;
        }

        if( ! $this->ranProd ) {
            $production = true;
            $this->ranProd = true;
        }else{
            $production = false;
            $this->ranTest = true;
        }

        $authProvider = AuthProvider\Token::create($options);
        // The VoIP push token of the recipient device
        $deviceToken = $voip_token;

        // Create alert payload
        $alert = Alert::create()->setTitle($title)
                                ->setBody($message);


        // Create payload
        $payload = Payload::create()->setAlert($alert);
        $payload->setPushType('voip');
        $payload->setContentAvailability(true); // VoIP-specific
        try {
            foreach ($data as $key => $value) {
                $payload->setCustomValue($key, $value);
            }
        }catch (Exception $exception)
        {
            mamdebug('VOIP Error', $exception);
        }



        // Create notification
        $notification = new Notification($payload, $deviceToken );
        $notifications[] = $notification;

        try {
            // Create and send notification
            $client = new Client($authProvider, $production );
            $client->addNotifications($notifications);
            $responses = $client->push(); // Returns an array of responses from APNs

            $this->ranProd = true;

            // Handle responses
            foreach ($responses as $response) {

                mamdebug('response', $response);

                $response_data = array();
                $response_data['options'] = $options;
                $response_data['token'] = $voip_token;
                $response_data['apnsid'] = $response->getApnsId();
                $response_data['status'] = $response->getStatusCode();
                $response_data['reason'] = $response->getReasonPhrase();
                $response_data['errorreason'] = $response->getErrorReason();
                $response_data['error'] = $response->getErrorDescription();
                mamdebug('voip response', $response_data );

                if ($response->getStatusCode() === 200) {
                    mamdebug('VOIP iOS', "Push sent successfully!");
                } else {
                    if( ! $this->ranTest ) {
                        mamdebug('VOIP Error', 'trying sandbox' );
                        $this->send_voip_notification( $args );
                        return;
                    }

                    mamdebug('VOIP Error', "Push failed: " . $response->getReasonPhrase());
                }
            }
        }catch (Exception $exception)
        {
            mamdebug('VOIP Error', $exception);
        }

        $voip_token = get_user_meta( $user_id, 'mam_app_android_pn_token', true );
        if( $voip_token) {
            $this->SendAndroidPushNotifications($message, '', $title, $voip_token, $data, true);
        }

    }

    function sendiosPush( $message, $url, $title, $phone_id = null , $data = null , $is_silent = 0 )
    {

        $pem_cert_file = $this->get_p8_file();

        if( ! $pem_cert_file ) return;

//        mamdebug('pndata', $message . ' - ' . $title );
//        mamdebug('pndata', $data );
//        mamdebug('pndata', $phone_id );

        global $wpdb;
        $table_name = $wpdb->prefix . 'tsl_phone_tokens';

        $test_mode = false;
        if( isset($_REQUEST['test_silent_pn']) )
        {
            $test_mode = true;
        }

        if( get_option( 'ios_pn_use_sandbox' ) == 'yes' ){
//            $this->ranProd = true;
        }

        if( ! $this->ranProd ) {
            $production = true;
            $this->ranProd = true;
        }else{
            $production = false;
            $this->ranTest = true;
        }

        if( ! get_option( 'ios_pn_key_id') ) {
            update_option('ios_pn_key_id', '569JPQ4BRJ');
            update_option('ios_pn_team_id', '977BJ3E5P8');
            update_option('ios_pn_app_bundle_id', 'tinyscreenlabs.com.57fbd2a7218af');
        }

        if( get_option( 'ios_pn_key_id') && get_option( 'ios_pn_team_id') && get_option( 'ios_pn_app_bundle_id')  ){
            $options['key_id'] = get_option( 'ios_pn_key_id');
            $options['team_id'] = get_option( 'ios_pn_team_id');
            $options['app_bundle_id'] = get_option( 'ios_pn_app_bundle_id');
            $options['private_key_path'] = $pem_cert_file;
            $options['private_key_secret'] = 'a101b102c103';

        }else{
            mamdebug( 'pnfailed', 'No Key Data ' . $pem_cert_file );
            return;
        }

        mamdebug('PN options', $options );

        $authProvider = AuthProvider\Token::create($options);

        $alert = Alert::create()->setTitle( $title );
        $alert = $alert->setBody(stripslashes( $message ) );

        $payload = Payload::create()->setAlert($alert);
        if( $test_mode ) $payload->setContentAvailability( true );

        //set notification sound to default
        $payload->setSound('default' );

        if( $data ) {
            $payload->setCustomValue('data', $data );
        }
        if( $url ) {
            $payload->setCustomValue('target', $url );
        }
        if( apply_filters( 'mam_use_custom_sound_for_pn', false ) ) {
            $payload->setSound(apply_filters( 'mam_custom_sound_for_pn', 'default' ));
        }

        $deviceTokens = array();

        if( ! $phone_id ) {
            return array( 'status' => 'failed' );
        }else if( is_array( $phone_id )) {
            for ($x = 0;  $x < sizeof($phone_id);  $x++) {
                $deviceTokens[] = $phone_id[$x];
            }
        }else{
            $deviceTokens[] = $phone_id;
        }

        $notifications = [];
        foreach ($deviceTokens as $deviceToken) {
            $notifications[] = new Notification($payload,$deviceToken);
        }

        $client = new Client($authProvider, $production  );
        $client->addNotifications($notifications);

        try {
            $responses = $client->push();
        }catch (Exception $e ){
            mamdebug('PN error' , $e );

        }

        $response_data = array();

        $some_failed = false;
        $run_in_sandbox = array();

        foreach ($responses as $response) {
            $response_data['apnsid'] = $response->getApnsId();
            $response_data['status'] = $response->getStatusCode();
            $response_data['reason'] = $response->getReasonPhrase();
            $response_data['errorreason'] = $response->getErrorReason();
            $response_data['error'] = $response->getErrorDescription();

            if( $response->getStatusCode() > 200 ) {
                if( $this->ranTest ) {
                    //mamdebug('PN Delete Token', $response);
                    $wpdb->get_results("DELETE FROM $table_name WHERE token = '".$response->getDeviceToken()."'");
                }else {
                    $some_failed = true;
                    $run_in_sandbox[] = $response->getDeviceToken();
                }
            }

        }

        if( $some_failed ) {
            if( ! $this->ranTest ){
            //    mamdebug('rerun response', $run_in_sandbox);
                $this->sendiosPush( $message, $url, $title, $run_in_sandbox  , $data  );
            }else {
              //  mamdebug('response data', $response_data);
            }
        }

    }


    function SendAndroidPushNotifications( $message, $target, $title, $phone_id = null , $data = array() , $is_voip = false )
    {

        global $tsl_local_app_dir_path;
        $serviceAccountPath = $tsl_local_app_dir_path . 'pemcerts/firebase-adminsdk.json';

        global $local_app_plugins_dir;

        $key = get_option( 'android_pn_pem_certs');
        $pem_cert_file = $local_app_plugins_dir . 'mam-main/pemcerts/' . $key;

        if( !file_exists( $pem_cert_file )) {
            mamdebug('admin_alert', 'Cannot send Android push notifications. Missing Firebase Service account json.');
            return false;
        }

        if( !file_exists($serviceAccountPath)) {
            mamdebug('firebase_creds', 'Firebase credentials file not found: ' . $serviceAccountPath);
            return false;
        }

        $accessToken = $this->getGoogleAccessToken($serviceAccountPath);

        $firebaseProjectId = get_option( 'mam_firebase_project_id' );

        if( ! $firebaseProjectId ) {
            mamdebug('firebase_creds', 'Firebase project ID not found in options');
            return false;
        }

        if (!$phone_id)
        {
            return false;
        }

        $registrationIds = array();
        if (is_array($phone_id)) {
            $registrationIds = $phone_id;
        } else {
            $registrationIds[] = $phone_id;
        }

        if (sizeof($registrationIds) == 0) return array('success' => 0);

        $url = "https://fcm.googleapis.com/v1/projects/{$firebaseProjectId}/messages:send";

        foreach( $registrationIds as $registrationId ) {
            $payload = array();
            $payload['message']['token'] = $registrationId;

            if ($is_voip) {

                $payload['message']['notification'] = [
                    'title' => $title,
                    'body' => $data['caller_name'] . " is calling you...",
                ];

                // Data payload (sent in the "data" object so the app can handle it in the background)
                $payload['message']['data'] = [
                    'type' => 'incoming_call',
                    'callerName' => $data['caller_name'],
                    'recipientID' => $data['recipientID'],
                    'channelName' => $data['channel'],
                    'agoraToken' => $data['token'],
                    'action' => 'CALL_INCOMING'
                ];

                $notification = array('sound' => 'ringtone.mp3', 'click_action' => 'OPEN_CALL_ACTIVITY');

                // FCM message format
                $payload['message']['android'] = [
                    'priority' => 'high',
//                    'notification' => $notification,
                ];


            } else {

                $payload['message']['notification'] = [
                    'title' => $title,
                    'body' => $message,
                ];

                if (!$target){
                    $payload['message']['data']['target'] = site_url() . '?tsl_button=1000';
                }else{
                    $payload['message']['data']['target'] = $target;
                }

                if (isset($data['action'])){
                    $payload['message']['data']['action'] = $data['action'];
                }else{
                    $payload['message']['data']['action'] = 'message';
                }

                if (get_option('android_notification_icon')) {
                    $payload['message']['data']['image'] = get_option('android_notification_icon');
                }

                if (is_array($data)) {
                    foreach ($data as $index => $item) {
                        $payload['message']['data'][$index] = (string)$item;
                    }
                }

            }

            $guzzleClient = new GuzzleHttpClient();
            try {
                $response = $guzzleClient->post($url, [
                    "headers" => [
                        "Authorization" => "Bearer " . $accessToken,
                        "Content-Type" => "application/json"
                    ],
                    "json" => $payload
                ]);

                $responseBody = json_decode($response->getBody(), true);
                mamdebug('notification', $payload);
                mamdebug("Notification sent successfully: ", $responseBody);
                return $response->getBody();
            } catch (RequestException $e) {
                mamdebug('notification', $payload);
                $errorBody = $e->getResponse()->getBody()->getContents();
                mamdebug("Error sending notification: ", $errorBody);
                return false;
            }
        }

    }

    function get_p8_file()
    {

        global $local_app_plugins_dir;

        $key = get_option( 'ios_pn_pem_certs');
        $pem_cert_file = $local_app_plugins_dir . 'mam-main/pemcerts/' . $key;

        if( file_exists( $pem_cert_file )) {
            update_option('pem_cert_date', date('U'));
            return $pem_cert_file;
        }

        $pem_cert_file = dirname(__FILE__) . '/pemcerts/57fbd2a7218af.p8';
        if( file_exists( $pem_cert_file )) {
            return $pem_cert_file;
        }else{
            return false;
        }


    }

    function getGoogleAccessToken($serviceAccountPath)
    {
    $serviceAccount = json_decode(file_get_contents($serviceAccountPath), true);

        $now = time();
        $jwtPayload = [
            "iss" => $serviceAccount["client_email"],
            "scope" => "https://www.googleapis.com/auth/firebase.messaging",
            "aud" => $serviceAccount["token_uri"],
            "iat" => $now,
            "exp" => $now + 3600,
        ];

        // Convert private key to JWK format

        try {
            $jwk = $this->getJWKFromPEM($serviceAccount["private_key"]);
        }catch (Exception $e) {
            mamdebug('JWK Error', $e->getMessage());
            return false;
        }

        // Initialize the JWT algorithm manager
        $algorithmManager = new AlgorithmManager([new RS256()]);
        $jwsBuilder = new JWSBuilder($algorithmManager);
        $serializer = new CompactSerializer();

        // Create the JWT
        $jws = $jwsBuilder
            ->create()
            ->withPayload(json_encode($jwtPayload))
            ->addSignature($jwk, ["alg" => "RS256", "typ" => "JWT"])
            ->build();

        $jwt = $serializer->serialize($jws, 0);

        // Exchange JWT for an OAuth token
        $guzzleClient = new GuzzleHttpClient();
        try {
            $response = $guzzleClient->post($serviceAccount["token_uri"], [
                "form_params" => [
                    "grant_type" => "urn:ietf:params:oauth:grant-type:jwt-bearer",
                    "assertion" => $jwt
                ]
            ]);

            $data = json_decode($response->getBody(), true);
            return $data["access_token"];
        } catch (RequestException $e) {
            die("Error getting access token: " . $e->getMessage());
        }
    }

    function getJWKFromPEM($pemPrivateKey)
    {
        // Load the private key using OpenSSL
        $key = openssl_pkey_get_private($pemPrivateKey);
        if (!$key) {
            throw new Exception("Failed to extract private key from PEM format.");
        }

        // Extract key details
        $keyDetails = openssl_pkey_get_details($key);

        if (!$keyDetails) {
            throw new Exception("Failed to extract key details.");
        }

        if (isset($keyDetails["rsa"])) {
            return new JWK([
                "kty" => "RSA",
                "n"   => base64_encode($keyDetails["rsa"]["n"]),
                "e"   => base64_encode($keyDetails["rsa"]["e"]),
                "d"   => base64_encode($keyDetails["rsa"]["d"]),
                "p"   => base64_encode($keyDetails["rsa"]["p"]),
                "q"   => base64_encode($keyDetails["rsa"]["q"]),
                "dp"  => base64_encode($keyDetails["rsa"]["dmp1"]),
                "dq"  => base64_encode($keyDetails["rsa"]["dmq1"]),
                "qi"  => base64_encode($keyDetails["rsa"]["iqmp"]),
            ]);
        } else {
            throw new Exception("Unsupported key type. Expected RSA.");
        }
    }

}
