<?php

if ( ! class_exists( 'WP_List_Table' ) ) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class Cron_List extends WP_List_Table {

    /**
     * Prepare the items for the table to process
     *
     * @return Void
     */
    public function prepare_items()
    {
        $columns = $this->get_columns();
        $hidden = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();

        $data = $this->table_data();
        local_app_sksort( $data, 'name' , true  );

        $perPage = 20;
        $currentPage = $this->get_pagenum();
        $totalItems = count($data);

        $this->set_pagination_args( array(
            'total_items' => $totalItems,
            'per_page'    => $perPage
        ) );

        $data = array_slice($data,(($currentPage-1)*$perPage),$perPage);

        $this->_column_headers = array($columns, $hidden, $sortable);
        $this->items = $data;
    }

    /**
     * Override the parent columns method. Defines the columns to use in your listing table
     *
     * @return Array
     */
    public function get_columns()
    {
        $columns = array(
            //'id'      => 'ID',
            'name'      => __( 'Name', 'mam-setcron-manager' ),
            'action'    => __( 'Hook Name', 'mam-setcron-manager' ),
            'frequency' => __( 'Frequency', 'mam-setcron-manager' ),
            'status'    => __( 'Status', 'mam-setcron-manager' ),
            'last_run'  => __( 'Last Run', 'mam-setcron-manager' ),
            'next_run'  => __( 'Next Run', 'mam-setcron-manager' ),
        );

        return $columns;
    }

    /**
     * Define which columns are hidden
     *
     * @return Array
     */
    public function get_hidden_columns()
    {
        return array();
    }

    /**
     * Define the sortable columns
     *
     * @return Array
     */
    public function get_sortable_columns()
    {
        /*return array('title' => array('title', false));*/
        return array();
    }

    /**
     * Get the table data
     *
     * @return Array
     */
    private function table_data()
    {
        $data = array();

        $available_crons = apply_filters( 'mam_cron_manager', array() );
        if(!empty($available_crons) && is_array($available_crons)){

            for($i=0;$i<count($available_crons);$i++){

                $status = '';
                $last_run = '';
                $next_run = '';
                $this_cron = get_option($available_crons[$i]['action'], true);

                if( is_array( $this_cron )){
                    if(array_key_exists('status', $this_cron)){
                        $status = ucfirst( $this_cron['status']);
                        if(array_key_exists('message', $this_cron)){
                            $status .= ': ' . $this_cron['message'];
                        }
                    }
                    if(array_key_exists('last_run', $this_cron)){
                        $last_run = date( 'M d, Y g:i:s A', strtotime($this_cron['last_run']));
                    }
                    if(array_key_exists('next_run', $this_cron)){
                        $next_run = date( 'M d, Y g:i:s A', strtotime($this_cron['next_run']));
                    }

                }
                if(array_key_exists('run_at', $available_crons[$i])){
                    $frequency = 'Every day at '.$available_crons[$i]['run_at'];
                }elseif($available_crons[$i]['frequency'] == 1){
                    $frequency = 'Every '.$available_crons[$i]['frequency'].' minute';
                }else{
                    $frequency = 'Every '.$available_crons[$i]['frequency'].' minutes';
                }

                $available_crons[$i]['frequency'] = $frequency;
                $available_crons[$i]['status'] = $status;
                $available_crons[$i]['last_run'] = $last_run;
                $available_crons[$i]['next_run'] = $next_run;
                $data[] = $available_crons[$i];
            }
        }

        return $data;
    }

    /**
     * Define what data to show on each column of the table
     *
     * @param  Array $item        Data
     * @param  String $column_name - Current column name
     *
     * @return Mixed
     */
    public function column_default( $item, $column_name )
    {
        switch( $column_name ) {
            case 'name':
            case 'action':
            case 'frequency':
            case 'status':
            case 'last_run':
            case 'next_run':
                return $item[ $column_name ];

            default:
                return print_r( $item, true ) ;
        }
    }

}
