<?php
/*
Plugin Name: Mobile App Manager User Roles
Description: User Roles and Login Manager for Mobile App Manager
Version: 1.0
Author: Tiny Screen Labs, LLC
Author URI: https://tinyscreenlabs.com
License: TSL Standard Software License
*/

require_once 'vendor/autoload.php';
include_once 'mam-settings-class.php';
include_once 'manage-phone-json.php';
include_once 'mam-user-role-button-array-class.php';
include_once 'user-content-functions.php';
include_once 'anon-user-manager.php';
include_once 'punch-out-content-class.php';
include_once 'manage-punchin-and-out.php';
include_once 'form-manager.php';
include_once 'phone-code-validation.php';

use Firebase\JWT\JWK;
use Firebase\JWT\JWT;
use Firebase\JWT\Key;

add_action( 'plugins_loaded', array( 'mam_user_roles_manager', 'init' ), 2 );

class mam_user_roles_manager
{

    private $app_version = '1.0';
    public $plugin_slug = 'mam-user-roles';
    private $new_user_role = 'subscriber';
    private $additional_post_meta = array();
    private $user;
    private $user_id;
    private $allow = false;

    public static function init()
    {
        $class = __CLASS__;
        new $class;
    }

    public function __construct(){

        //delete after refactoring
        update_option( 'user_roles_v_2', true );

        do_action( 'mam_update_current_user' );
        global $mam_user, $mam_user_id;

        add_action('edit_user_profile_update', array( $this, 'update_custom_field_on_profile_save'));

        $this->user = $mam_user;
        $this->user_id = $mam_user_id;

        if( function_exists( 'tsl_mam_get_option_value' )) {
            $this->new_user_role = tsl_mam_get_option_value('tsl-setting-new_user_role');
        }else{
            $this->new_user_role = get_option( 'tsl-setting-new_user_role' );
        }
        if( ! isset( $this->new_user_role ) || strlen($this->new_user_role) < 2 ) $this->new_user_role = 'subscriber';

    }

    function update_custom_field_on_profile_save($user_id)
    {
        delete_user_meta($user_id, 'admin_is_cloning' );

        if( isset( $_POST['admin_is_cloning'])) {

            $args = array(
                'meta_query' => array(
                    array(
                        'key' => 'admin_is_cloning' ,
                        'value' => get_current_user_id(),
                    )
                )
            );
            $all_users = get_users($args);
            if (!empty($all_users)) {
                foreach ($all_users as $user) {
                    delete_user_meta($user->ID, 'admin_is_cloning' );
                }
            }

            update_user_meta( $user_id , 'admin_is_cloning' , get_current_user_id() );
        }
    }

    function get_p8_file()
    {

        global $pem_cert_file;

        $key = get_option( 'ios_pn_pem_certs');
        $pem_cert_file = WP_PLUGIN_DIR . '/mam-main/pemcerts/' . $key;
        if( file_exists( $pem_cert_file )) {
            update_option('pem_cert_date', date('U'));
            return $pem_cert_file;
        }

        $pem_cert_file = dirname(__FILE__) . '/pemcerts/AuthKey_' . $key . '.p8';
        if( file_exists( $pem_cert_file )) {
            update_option('pem_cert_date', date('U'));
            return $pem_cert_file;
        }

        $account = get_option('local-app-account_code');
        $pem_cert_file = dirname(__FILE__) . '/pemcerts/' . $account . '.p8';
        if( file_exists( $pem_cert_file )) {
            update_option('pem_cert_date', date('U'));
            return $pem_cert_file;
        }

        $pem_cert_file = dirname(__FILE__) . '/pemcerts/57fbd2a7218af.p8';
        if( file_exists( $pem_cert_file )) {
            return $pem_cert_file;
        }else{
            $pem_cert_file = null;
            return $pem_cert_file;
        }

    }

    function mam_notification_list( $data_array )
    {

        $replacements = array('ID','user_login','user_nicename','user_email','user_url','user_registered','display_name','first_name','last_name','nickname','description');
        $data_array[] = array('source' => $this->plugin_slug, 'title' => 'User Roles - Welcome Email', 'slug' => 'mam-user-roles-welcome_email', 'replacements' => $replacements , 'description' => 'This notification is sent to the user when they first register' );
        $replacements = array( 'passwordcode' );
        $data_array[] = array('source' => $this->plugin_slug, 'title' => 'User Roles - Lost Password', 'slug' => 'mam-user-roles-lost_password', 'replacements' => $replacements, 'default_content' => 'Here is your password [passwordcode]', 'default_on' => array('email') , 'description' => 'This notification is sent to the user when they request a lost password. It includes a code to use to reset their password.' );

        //not used at teh moment
        //$data_array[] = array('source' => $this->plugin_slug, 'title' => 'User Roles - Email Only Login Message', 'slug' => 'mam-user-roles-login-with-email', 'replacements' => $replacements, 'default_content' => 'Here is your code to access your account [passwordcode]', 'default_on' => array('email') );

//        $replacements = array();
//        $replacements['display_name'] = 'display_name';
//        $data_array[] = array('source' => $this->plugin_slug, 'title' => 'User has not logged in for five days', 'slug' => 'mam-not-mgr-user-no-login-five-days', 'replacements' => $replacements , 'frequency' => 'daily' , 'time' => '0600' );

        $replacements = array();
        $replacements['title'] = 'title';
        $replacements['content'] = 'content';
        $data_array[] = array('source' => $this->plugin_slug, 'title' => 'General Push Notification', 'slug' => 'mam-push-notification', 'replacements' => $replacements , 'description' => 'This notification is used by some plugins to send push notifications to users' );

        return $data_array;
    }

    function empty_email_error( $arg ) {
        if ( !empty( $arg->errors['empty_email'] ) ) unset( $arg->errors['empty_email'] );
    }

    function custom_user_profile_fields( $user )
    {
        if( ! get_user_meta( $user->id, '_user_lat' , true ) && class_exists( 'mam_geocoding_manager' )) {

            $apikey = get_option('tsl-google-api-key');
            $user_info = get_user_meta($user->ID);

            if (isset($user_info['billing_postcode'][0]) || isset($user_info['billing_address_1']) && $apikey) {

                $address = $user_info['billing_address_1'][0] . ' ' . $user_info['billing_city'][0] . ' ' . $user_info['billing_state'][0] . ' ' . $user_info['billing_postcode'][0];
                $address = trim($address);

                if (strlen($address) > 5) {
                    $handler = new mam_geocoding_manager();
                    $geo_array = $handler->geocode_address($address);
                    if (isset($geo_array['lat'])) {
                        update_user_meta($user->ID, '_user_lat', $geo_array['lat']);
                        update_user_meta($user->ID, '_user_lon', $geo_array['lon']);
                    }
                }
            }
        }

        ?>
        <h2>Mobile App Information</h2>
        <table class="form-table">
            <tr>
                <th>
                    <label for="code"><?php _e('User Latitude'); ?></label>
                </th>
                <td>
                    <span><?php echo esc_attr(get_user_meta( $user->ID, '_user_lat', true )); ?></span>
                </td>
            </tr>
            <tr>
                <th>
                    <label for="code"><?php _e('User Longitude:'); ?></label>
                </th>
                <td>
                    <span><?php echo esc_attr( get_user_meta( $user->ID, '_user_lon', true )); ?></span>
                </td>
            </tr>
            <tr>
                <th>
                    <label for="code"><?php _e('User Phone ID:'); ?></label>
                </th>
                <td>
                    <span><?php echo get_user_meta( $user->ID , 'mam_app_token', true ); ?></span>
                </td>
            </tr>
            <tr>
                <th>
                    <label for="code"><?php _e('User iOS Token:'); ?></label>
                </th>
                <td>
                    <span><?php echo get_user_meta( $user->ID , 'mam_app_ios_pn_token', true ); ?></span>
                </td>
            </tr>
            <tr>
                <th>
                    <label for="code"><?php _e('User Android Token:'); ?></label>
                </th>
                <td>
                    <span><?php echo get_user_meta( $user->ID , 'mam_app_android_pn_token', true ); ?></span>
                </td>
            </tr>
            <tr>
                <th>
                    <label for="code"><?php _e('User iOS Version:'); ?></label>
                </th>
                <td>
                    <span><?php echo get_user_meta( $user->ID , 'iosversion', true ); ?></span>
                </td>
            </tr>
                        <tr>
                <th>
                    <label for="code"><?php _e('User Android Version:'); ?></label>
                </th>
                <td>
                    <span><?php echo get_user_meta( $user->ID , 'andversion', true ); ?></span>
                </td>
            </tr>
        </table>
        <?php

    }

    function check_role( $user_id )
    {

        if( $user_id< 1 ) return;

        global $wp_roles;
        $user = get_user_by( 'ID', $user_id );
        $valid_user_role = true;

        $all_roles = $wp_roles->roles;

        foreach($all_roles as $role_index => $role ) {

            $this_role_slug = 'mam-user-roles-to-manage-' . $role_index;

            $value = 'no';
            if (get_option($this_role_slug)) {
                $value = get_option($this_role_slug);
            }

            if( in_array( $role_index, $user->roles ))
            {
                if( $value == 'no') $valid_user_role = false;
                if( $value == 'yes'){
                    $valid_user_role = true;
                    break;
                }
            }

        }

        if( ! $valid_user_role ) {
            delete_user_meta($user_id, 'mam_app_token');
            global $is_logging_in;
            if( $is_logging_in )
            {
                $data_array = array('status' => 'failed', 'message' => 'The role for this user login is not setup on the app');

                wp_send_json( $data_array );
                die();
            }
        }

    }

    public function ajax_cred_handler(){

        global $is_logging_in;
        $is_logging_in = true;

        $data_array['status'] = 'shove off';

        $formdata = json_decode( file_get_contents('php://input'), true );

        $actual_link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

        mamdebug( 'login', $actual_link );
        mamdebug( 'login', $formdata );
        mamdebug( 'login_r', $_REQUEST );

        if(is_array($formdata)) $formdata = $formdata[0];

        do_action( 'mam_before_login_start', $formdata );

        if( $formdata['user'] == 'Doctor' || $formdata['user'] == 'Airline' ) die();

        $this->allow = false;
        if( isset( $_REQUEST['login_nonce']) )
        {
            $this->allow = wp_verify_nonce( $_REQUEST['login_nonce'] , $_REQUEST['pid'] );
            if( ! $this->allow )
            {
                $data_array = array('status' => 'failed', 'message' => 'Login failed.  Restart app and try again' );
                wp_send_json($data_array);
                die();
            }
        }

//        $this->clean_users( $formdata['pid'] );

        if( isset( $formdata['resetPasswordCode'] )  )
        {
            $user = get_user_by('login', $formdata['user']);

            if (!$user) {
                $user = get_user_by('email', trim($formdata['user']));
            }

            if ($user)
            {

                $secret_code = get_user_meta($user->ID, 'mam_pw_reset_code' );
                if( $secret_code == $_REQUEST['resetPasswordCode'] ) {
                    wp_set_password($formdata['pass'], $user->ID);
                }
            }
        }

        if( isset( $formdata['client_code'] ) && ! isset( $_REQUEST['client_code'] ) ){

            $_REQUEST['client_code'] = $formdata['client_code'];
            if( ! isset( $_REQUEST['createGoogleAccount'])) $_REQUEST['createGoogleAccount'] = 0;

            if( isset( $formdata['deviceandroid'] )) $formdata['deviceand'] = $formdata['deviceandroid'];

            if( isset( $formdata['deviceand'] )){
                foreach( $formdata as $index => $item )
                {
                    $_REQUEST[$index] = $item;
                }
            }
        }

        if( isset($_REQUEST['logout']) && (isset($formdata['pid']) || isset($_REQUEST['pid'] ))) {
            $this->do_logout( $data_array , $formdata );
        }

        if( $this->user ) {
            $data_array['status'] = 'success';
            if (function_exists('local_app_get_phone_data')) {
                $jsonData = local_app_get_phone_data(true);
                $data_array['jsonData'] = $jsonData;
                mamdebug('login', __LINE__);
            }

            $this->set_current_user_using_pid();

            $this->check_role( $this->user->ID);
            wp_send_json($data_array);
            die();
        }

//        do_action( 'mam_before_login_start', $formdata );

        if( $_REQUEST['subaction'] == 'lost_password' ){
            $this->do_lost_password( $data_array , $formdata );
        }
        if($_REQUEST['subaction'] == 'update_lost_password'){
            $this->update_lost_password( $data_array, $formdata );
        }

        if( isset( $formdata['pass'] ) &&  strlen($formdata['pass']) < 2 ){
            $data_array = array('status' => 'failed', 'message' => apply_filters( 'mam_login_missing_password_message', 'Missing password'));
            wp_send_json($data_array);
            die();
        }

        if( isset($_REQUEST['isSignUp'] ) ) $_REQUEST['isSigningUp'] = $_REQUEST['isSignUp'];

        if(!isset($_REQUEST['pid']) && isset( $formdata['pid'] )) $_REQUEST['pid'] = $formdata['pid'];
        if(!isset($_REQUEST['deviceios']) && isset( $formdata['deviceios'] )) $_REQUEST['deviceios'] = $formdata['deviceios'];

        if( !isset( $_REQUEST['createGoogleAccount'] ) && isset( $formdata['createGoogleAccount'])) $_REQUEST['createGoogleAccount'] = $formdata['createGoogleAccount'];

        if( isset($_REQUEST['subaction']) && $_REQUEST['subaction'] == 'check_phone_number' ){
            $this->check_phone_number( $data_array );
        }
        if( isset($_REQUEST['subaction']) && $_REQUEST['subaction'] == 'verify_phone_number' ){
            $this->verify_phone_number( $data_array );
        }
        if( isset($_REQUEST['subaction']) && $_REQUEST['subaction'] == 'check_email_address' ){
            $this->check_email_address( $data_array );
        }
        if( isset($_REQUEST['subaction']) && $_REQUEST['subaction'] == 'verify_email_address' ){
            $this->verify_email_address( $data_array );
        }

        $user = get_users(array('meta_key' => 'mam_app_token', 'meta_value' => $_REQUEST['pid']));
        if($user) {
            $user =  $user[0];
            if ( $user->ID > 0) {
                mamdebug('LOGIN', 'is logged in');
                $data_array['status'] = 'success';
                $jsonData = local_app_get_phone_data(true);
                $data_array['jsonData'] = $jsonData;
                mamdebug('login', __LINE__);
                $this->check_role( $user->ID);
                wp_send_json( $data_array );
                die();
            }
        }

        if( isset( $formdata['appleUserId'] ) && strlen( $formdata['appleUserId'] ) > 0 ) {

            global $wpdb;
            $table_name = $wpdb->prefix . 'mo_openid_linked_user';
            $social_login = $wpdb->get_results('SELECT * FROM ' . $table_name . ' WHERE identifier LIKE "' . $formdata['appleUserId'] . '"', ARRAY_A);

            if( sizeof( $social_login ) > 0 ) {

                $user = get_user_by('ID', $social_login[0]['user_id']);

                if ($user) {

                    $this->clean_users( $formdata['pid'] );

                    update_user_meta( $user->ID, 'mam_app_token', $formdata['pid']);

                    if (strlen($formdata['deviceios']) > 0) {
                        update_user_meta($user->ID, 'mam_app_ios_pn_token', $formdata['deviceios']);
                    }
                    if (strlen($formdata['deviceand']) > 0) {
                        update_user_meta($user->ID, 'mam_app_android_pn_token', $formdata['deviceand']);
                    }
                    if (strlen($_REQUEST['deviceios'])) {
                        update_user_meta($user->ID, 'mam_app_ios_pn_token', $_REQUEST['deviceios']);
                    }
                    if (strlen($_REQUEST['deviceand'])) {
                        update_user_meta($user->ID, 'mam_app_android_pn_token', $_REQUEST['deviceand']);
                    }

                    $data_array['status'] = 'success';
                    $jsonData = local_app_get_phone_data(true);
                    $data_array['jsonData'] = $jsonData;
                    mamdebug('login', __LINE__);
                    $this->check_role( $user->ID);
                    wp_send_json($data_array);
                    die();
                }
            }else if( isset($formdata['appleToken'] )) {

                mamdebug('appleToken', $formdata['appleToken']);

                //validate without mini-orange plugin

                $identityToken = $formdata['appleToken'];

                $appleJwks = file_get_contents('https://appleid.apple.com/auth/keys');
                $jwksArray = json_decode($appleJwks, true);

                $decodedToken = null;

                foreach ($jwksArray['keys'] as $keyData) {
                    $key = JWK::parseKey($keyData);  // Convert JWK to usable key format

                    try {
                        $decodedToken = JWT::decode($identityToken, $key );
                        break;  // If successful, exit the loop
                    } catch (\Exception $e) {
                        // This key didn't work, try the next one.
                    }
                }

                if ($decodedToken) {
                    mamdebug('token',  "Token is valid! Payload:");
                } else {
                    $data_array['status'] = 'error';
                    $data_array['message'] = 'Sign In with Apple not configured.  Please contact the Admin';
                    wp_send_json($data_array);
                    die();
                }
            }
        }

        if ( ( $this->allow || isset($formdata['credkey'] ) ) && isset($formdata['user']) && isset($formdata['pass']) && isset($formdata['pid'])) {

            if( $this->allow || $formdata['credkey'] == 'udsf97h35iletushfo8yeu5aksbvo7w6t3d47tfqux65rfi' ) {

                if( isset( $formdata['appleUserId'] ) && strlen( $formdata['appleUserId'] ) > 0 ){

                    $user = get_user_by('login', trim($formdata['appleUserId']) );
                }
                if( ! $user ) {
                    $user = get_user_by('login', trim($formdata['user']));

                    if (!$user) {
                        $user = get_user_by('email', trim($formdata['user']));
                    }
                }

                if( $user )
                {
                    $roles = $user->roles;
                    if( in_array( 'invited' , $roles ))
                    {
                        wp_set_password( $formdata['pass'], $user->ID );
                        $user->remove_role( 'invited' );
                        $user->add_role( $this->new_user_role );
                    }
                }

                if ( $user && ( wp_check_password( $formdata['pass'], $user->user_pass, $user->ID) || ( isset( $_REQUEST['createGoogleAccount'] ) && $_REQUEST['createGoogleAccount'] == 1 ) )) {

                    $args['user_role'] = $this->new_user_role;
                    $this->do_role_multisite( $user->ID , $args );

                    $this->clean_users( $formdata['pid'] );

                    update_user_meta( $user->ID, 'mam_app_token', $formdata['pid']);

                    if( isset( $_REQUEST['firstName'] ) && strlen( $_REQUEST['firstName']  ) > 0  ){
                        update_user_meta($user->ID, 'first_name', $_REQUEST['firstName'] );
                        update_user_meta($user->ID, 'billing_first_name', $_REQUEST['firstName'] );
                    }

                    if( isset($_REQUEST['lastName'])  && strlen( $_REQUEST['lastName']  ) > 0 ){
                        update_user_meta($user->ID, 'last_name', $_REQUEST['lastName'] );
                        update_user_meta($user->ID, 'billing_last_name', $_REQUEST['lastName'] );
                    }
                    if( isset($_REQUEST['phoneNumber']) && strlen( $_REQUEST['phoneNumber']  ) > 0 ){
                        update_user_meta($user->ID, 'billing_phone', $_REQUEST['phoneNumber'] );
                    }

                    if ( isset( $formdata['deviceios'] ) && strlen($formdata['deviceios']) > 0) {
                        update_user_meta($user->ID, 'mam_app_ios_pn_token', $formdata['deviceios'] );
                    }
                    if ( isset( $formdata['deviceand'] ) && strlen($formdata['deviceand']) > 0) {
                        update_user_meta($user->ID, 'mam_app_android_pn_token', $formdata['deviceand'] );
                    }
                    if ( isset( $_REQUEST['deviceios'] ) && strlen($_REQUEST['deviceios'])) {
                        update_user_meta($user->ID, 'mam_app_ios_pn_token', $_REQUEST['deviceios']);
                    }
                    if (  isset( $_REQUEST['deviceand'] ) && strlen($_REQUEST['deviceand'])) {
                        update_user_meta($user->ID, 'mam_app_android_pn_token', $_REQUEST['deviceand']);
                    }

                    update_user_meta( $user->ID, 'last_login', time() );

                    $data_array['status'] = 'success';
                    $data_array['message'] = 'here';
                    $data_array['isNewUser'] = 1;
                    if (function_exists('local_app_get_phone_data')) {
                        //commented out on Feb 11 2025 to fix double json generation
//                        $jsonData = local_app_get_phone_data(true);
//                        $data_array['jsonData'] = $jsonData;
//                        mamdebug('login', __LINE__);
                    }

                }else if($user){

                    $data_array['status'] = 'failed';
                    $data_array['message'] = 'This email address has an account and the password entered is incorrect. Please try again';


                } else {

                    if( isset( $_REQUEST['createGoogleAccount'] ) && $_REQUEST['createGoogleAccount'] == 1 ){

                        if( isset( $formdata['appleUserId'] ) && strlen( $formdata['appleUserId'] ) > 0 ){
                            $formdata['user_name'] = $formdata['appleUserId'];
                        }

                        $args['user_email'] = $formdata['user'];
                        $args['password'] = substr( $formdata['pass'] ,0 , 20 );
                        if( isset($formdata['user_name'])){
                            $args['user_name'] = $formdata['user_name'];
                        }else {
                            $args['user_name'] = $formdata['user'];
                        }
                        $args['user_role'] = $this->new_user_role;

                        $data_array = $this->add_or_update_user_account( $args );

                        if(isset($data_array['data'])) {

                            $user_ID = $data_array['data'];

                            $this->clean_users( $formdata['pid'] );

                            update_user_meta( $user_ID, 'mam_app_token', $formdata['pid']);

                            if( isset( $_REQUEST['firstName'] ) && strlen( $_REQUEST['firstName']  ) > 0 ){
                                update_user_meta($user_ID, 'first_name', $_REQUEST['firstName'] );
                                update_user_meta($user_ID, 'billing_first_name', $_REQUEST['firstName'] );
                            }
                            if( isset($_REQUEST['lastName']) && strlen( $_REQUEST['lastName']  ) > 0  ){
                                update_user_meta($user_ID, 'last_name', $_REQUEST['lastName'] );
                                update_user_meta($user_ID, 'billing_last_name', $_REQUEST['lastName'] );
                            }
                            if( isset($_REQUEST['phoneNumber']) && strlen( $_REQUEST['phoneNumber']  ) > 0  ){
                                update_user_meta($user_ID, 'billing_phone', $_REQUEST['phoneNumber'] );
                            }

                            if (strlen($formdata['deviceios'])) {
                                update_user_meta( $user_ID, 'mam_app_ios_pn_token', $formdata['deviceios']);
                            }
                            if (strlen($formdata['deviceand'])) {
                                update_user_meta( $user_ID, 'mam_app_android_pn_token', $formdata['deviceand']);
                            }
                            if (strlen($_REQUEST['deviceios'])) {
                                update_user_meta($user_ID, 'mam_app_ios_pn_token', $_REQUEST['deviceios']);
                            }
                            if (strlen($_REQUEST['deviceand'])) {
                                update_user_meta($user_ID, 'mam_app_android_pn_token', $_REQUEST['deviceand']);
                            }

                            if( isset( $formdata['appleUserId'] ) && strlen( $formdata['appleUserId'] ) > 0 ){

                                global $wpdb;
                                $table_name = $wpdb->prefix.'mo_openid_linked_user';
                                $wpdb->get_results( 'DELETE FROM ' . $table_name .' WHERE identifier LIKE "' . $formdata['appleUserId'] . '"', ARRAY_A );
                                $wpdb->insert( $table_name, array( 'linked_social_app' => 'apple', 'linked_email' => $formdata['user'] , 'user_id' => $user_ID , 'identifier' => $formdata['appleUserId'] ));

                                $social_login = $wpdb->get_results( 'SELECT * FROM ' . $table_name .' WHERE identifier LIKE "' . $formdata['appleUserId'] . '"', ARRAY_A );

                            }

                        }

                        if (function_exists('local_app_get_phone_data')) {
                            $jsonData = local_app_get_phone_data(true);
                            $data_array['jsonData'] = $jsonData;
//                            mamdebug('login', __LINE__);
                        }

                    }else{

                        if( isset($_REQUEST['isSigningUp']) && $_REQUEST['isSigningUp'] == 1 ) {

                            $args['user_email'] = $formdata['user'];
                            $args['password'] = $formdata['pass'];
                            if( isset($formdata['user_name'])){
                                $args['user_name'] = $formdata['user_name'];
                            }else {
                                $args['user_name'] = $formdata['user'];
                            }
                            $args['user_role'] = $this->new_user_role;

                            $data_array = $this->add_or_update_user_account($args);

                            if (isset($data_array['data'])) {
                                $user_ID = $data_array['data'];

                                $this->clean_users($formdata['pid']);

                                update_user_meta($user_ID, 'mam_app_token', $formdata['pid']);

                                if( isset( $_REQUEST['firstName'] ) && strlen( $_REQUEST['firstName']  ) > 0 ){
                                    update_user_meta($user_ID, 'first_name', $_REQUEST['firstName'] );
                                    update_user_meta($user_ID, 'billing_first_name', $_REQUEST['firstName'] );
                                }

                                if( isset($_REQUEST['lastName']) && strlen( $_REQUEST['lastName']  ) > 0 ){
                                    update_user_meta($user_ID, 'last_name', $_REQUEST['lastName'] );
                                    update_user_meta($user_ID, 'billing_last_name', $_REQUEST['lastName'] );
                                }
                                if( isset($_REQUEST['phoneNumber']) && strlen( $_REQUEST['phoneNumber']  ) > 0 ){
                                    update_user_meta($user_ID, 'billing_phone', $_REQUEST['phoneNumber'] );
                                }

                                if (strlen($formdata['deviceios'])) {
                                    update_user_meta($user_ID, 'mam_app_ios_pn_token', $formdata['deviceios']);
                                }
                                if (strlen($formdata['deviceand'])) {
                                    update_user_meta($user_ID, 'mam_app_android_pn_token', $formdata['deviceand']);
                                }
                                if (strlen($_REQUEST['deviceios'])) {
                                    update_user_meta($user_ID, 'mam_app_ios_pn_token', $_REQUEST['deviceios']);
                                }
                                if (strlen($_REQUEST['deviceand'])) {
                                    update_user_meta($user_ID, 'mam_app_android_pn_token', $_REQUEST['deviceand']);
                                }
                                $data_array['status'] = 'success';

                                if( isset( $formdata['appleUserId'] ) && strlen( $formdata['appleUserId'] ) > 0 && $formdata['user'] == $formdata['appleUserId'] ){
                                    mamdebug('apple ' . __LINE__, $formdata );
                                }

                                update_user_meta( $user_ID, 'last_login', time() );
                            }

                            if (function_exists('local_app_get_phone_data')) {
                                $jsonData = local_app_get_phone_data(true);
                                $data_array['jsonData'] = $jsonData;
//                                mamdebug('login', __LINE__);
                            }
                        }else{

                            //login failed
                            $data_array['status'] = 'failed';
                            $user = get_user_by('login', trim($formdata['user']) );

                            if( !$user ){
                                $user = get_user_by( 'email', trim($formdata['user']) );
                            }

                            if( $user ) {
                                $data_array['message'] = 'This email address has an account and the password entered is incorrect. Please try again';
                            }else{
                                $data_array['message'] = 'Your email address or password is incorrect. Please try again or sign up.';
                            }

                        }
                    }
                }
            }
        }

        if( $data_array['status'] == 'success' )
        {
            do_action( 'mam_user_logged_in' , $data_array , $user->ID );
        }
        $user = get_users(array('meta_key' => 'mam_app_token', 'meta_value' => $_REQUEST['pid']));

        $jsonData = local_app_get_phone_data(true);
        $data_array['jsonData'] = $jsonData;
//        mamdebug('login', __LINE__);

        $this->check_role( $user[0]->ID);

        wp_send_json( $data_array );
        die();

    }

    function add_or_update_user_account( $args = array() ){

        if(empty($args['user_email']) ) return array('status' => 'failed', 'message' => 'Missing email');

        $_REQUEST['user_email'] = $args['user_email'];

        $args = apply_filters( 'mam_user_roles_before_create_user_args' , $args );

        $error = array();
        $error = apply_filters( 'mam_user_roles_before_create_user' , $error );

        if( sizeof( $error ) > 0 ){
            if( $error['status'] == 'failed' ){
                return $error;
            }
            if( isset( $error['post_meta'])) $this->additional_post_meta = $error['post_meta'];

            mamdebug('loginerror', $error );
        }

        $user_id = username_exists( $args['user_email'] );

        if(!$user_id){
            $user_id = email_exists( $args['user_email'] );
        }

        $new_user = 0;

        if ( ! $user_id ) {

            if (empty($args['password']) && ! $_REQUEST['subaction'] == 'lost_password' ) return array('status' => 'failed', 'message' => 'Missing password');

//            if (strlen($args['password']) < 7 && ! $_REQUEST['subaction'] == 'lost_password' ) return array('status' => 'failed', 'message' => 'Password must be at least 8 characters');

            if( isset( $args['override_role'] ) ){
                $this->new_user_role = $args['user_role'];
            }

            if( ! isset($args['user_name'])){
                $args['user_name'] = $args['user_email'];
            }

            $userdata = array(
                'user_login' => $args['user_name'],
                'user_pass' => $args['password'],
                'user_email' => $args['user_email'],
                'role' => $this->new_user_role
            );

            $user_id = wp_insert_user( $userdata );
            $new_user = 1;

            if( ! is_wp_error($user_id)) {

                if (isset($_REQUEST['firstName']) && strlen( $_REQUEST['firstName']  ) > 0 ) {
                    $user_display = $_REQUEST['firstName'] . ' ' . $_REQUEST['lastName'];
                    $user_id = wp_update_user(array('ID' => $user_id, 'display_name' => $user_display ));
                }
                $message = array();
                $message['message_type'] = 'mam-user-roles-welcome_email';
                $message['send_text'] = false;
                $message['send_email'] = true;
                $message['send_pn'] = false;
                $message['recipient_id'] = $user_id;
                $message['email'] = $args['user_email'];
                $message['send_now'] = true;
                $message['target'] = 'none';
                $message['force_email'] = true;
                $message['replacements'] = $this->create_replacements( $user_id );

                if( ! get_user_meta( $user_id , 'welcome-email-sent', true )) {

                    update_user_meta( $user_id , 'welcome-email-sent', 'yes' );
                    $messages = array();
                    $messages[] = $message;

                    apply_filters('mam_notification_send_message', $messages);
                }

                do_action( 'mam_new_user_added_complete' , $user_id );

                $complete = apply_filters( 'mam_user_roles_after_create_user' , array(), $user_id );

            }else{
                mamdebug('error ' . __LINE__ , $user_id->get_error_message() );
                return array('status' => 'failed', 'message' => $user_id->get_error_message());
            }
        }else{

            $new_user = 0;

            update_user_meta( $user_id , 'tsl_activity_user_role', $this->new_user_role );

            $this->do_role_multisite( $user_id , $args );

            if(isset($args['update_role'])){
                $user_id = wp_update_user( array( 'ID' => $user_id, 'role' => $this->new_user_role ));
            }

            if (is_wp_error($user_id)) {
                mamdebug('error ' . __LINE__ , $user_id->get_error_message() );
                return array('status' => 'failed', 'message' => $user_id->get_error_message());
            }

        }

         return array('status' => 'success', 'message' => 'User created and or role added', 'data' => $user_id , 'isNewUser' => $new_user );

    }

    public function clean_users( $tokan = null){

        if(!$tokan) return;

        $users = get_users(array('meta_key' => 'mam_app_token', 'meta_value' => $tokan ));

        foreach($users as $user ){

            delete_user_meta( $user->ID, 'mam_app_token');
            delete_user_meta( $user->ID, 'mam_app_ios_pn_token');
            delete_user_meta( $user->ID, 'mam_app_android_pn_token');
        }
    }

    public function ajax_handler(){

        if(current_user_can( 'manage_options' )) {

            $variable_array = array();
            $variable_array['mam-user-roles-require-login'] = 'mam-user-roles-require-login';

            global $wp_roles;
            $all_roles = $wp_roles->roles;
            $has_role_management = 'no';

            foreach($all_roles as $role_index => $role ) {

                if (isset($_REQUEST['mam-user-roles-to-manage-' . $role_index])) {
                    update_option( 'mam-user-roles-to-manage-' . $role_index, $_REQUEST['mam-user-roles-to-manage-' . $role_index] );
                }
                if(get_option( 'mam-user-roles-to-manage-' . $role_index ) == 'yes' ) $has_role_management = 'yes';
            }

            update_option( 'mam-user-roles-has-managed-roles', $has_role_management );

            foreach($variable_array as $item  ) {
                if (isset($_REQUEST[$item])) {
                    update_option( $item, $_REQUEST[$item] );
                }
            }
        }

        $data_array['status'] = 'success';

        $this->set_current_user_using_pid();

        wp_send_json( $data_array );
        die();

    }

    public function plugin_settings_section( $html_line ){


        $mam_user_roles_settings_page = new mam_user_roles_settings_page();
        $html_line .=  $mam_user_roles_settings_page->create_settings_section();

        return $html_line;

    }

    function admin_manager_scripts()
    {

        if (is_admin()) {
            if (isset($_REQUEST['page']) && ($_REQUEST['page'] == 'local_app_setup' || $_REQUEST['page'] == 'mam-advanced-app-manager-settings')) {
                wp_register_script('mam-user-roles', plugins_url('js/mam-user-role-functions.js', __FILE__), array(), $this->app_version, true);
                wp_enqueue_script(array('jquery', 'mam-user-roles'));

            }
        }
    }

    function manage_phone_data( $data_array ){

        $manage_phone_json = new mam_user_roles_manage_phone_json();
		return $manage_phone_json->update_phone_data( $data_array );

    }

    function check_phone_number( $data_array )
    {

        $data_array['status'] = 'success';

        $to = '1' . str_replace('-','',$_REQUEST['phone_number']);

        $Code_1 = rand(100, 999);
        $Code_2 = rand(100, 999);

        if( str_replace( '-', '', $_REQUEST['phone_number'] ) == '6305322014' ){
            $Code_1 = 107;
            $Code_2 = 965;
        }


        global $sms;
        if( $sms ) {
            $sms->to = array($to);
            $sms->msg = 'Your secret code is ' . $Code_1 . ' ' . $Code_2;
            $sms->SendSMS();
        }

        $data_array['code'] = $Code_1 . '-' . $Code_2;
        $data_array['goto'] = 'onboarding';

        $this->set_current_user_using_pid();

        wp_send_json( $data_array );
        die();

    }

    function verify_phone_number( $data_array )
    {
        $data_array['status'] = 'success';
        $data_array['goto'] = 'onboarding';

        //login or create account
        $user = get_users(array('meta_key' => 'mam_app_token', 'meta_value' => $_REQUEST['pid']));

        if( !$user ){
            $handler = new tsl_mam_anon_user_manager();
            $handler->create_anon_user();
        }else{
            update_user_meta( $user[0]->ID, 'last_login', time() );
        }

        $jsonData = local_app_get_phone_data(true);
        $data_array['jsonData'] = $jsonData;
        mamdebug('login', __LINE__);

        $this->set_current_user_using_pid();

        wp_send_json( $data_array );
        die();

    }

    function check_email_address( $data_array )
    {

        $data_array['status'] = 'success';

        $Code_1 = rand(100, 999);
        $Code_2 = rand(100, 999);

        if( ! $this->user )
        {
            //add a user using this email address
        }

        //send email message to user with code
        $message = array();
        $data_array['code'] = $Code_1 . '-' . $Code_2;

        update_user_meta( $this->user->ID, 'mam_pw_reset_code', $data_array['code'] );

        $message['message_type'] = 'mam-user-roles-login-with-email';
        $message['send_text'] = false;
        $message['send_email'] = true;
        $message['send_pn'] = true;
        $message['recipient_id'] = $this->user->ID;
        $message['email'] = $this->user->user_email;
        $message['send_now'] = true;
        $message['target'] = 'none';
        $message['replacements'] = array( 'passwordcode' => $data_array['code'] );
        $message['status'] = false;
        $messages[] = $message;

        apply_filters( 'mam_notification_send_message' , $messages );

        $data_array['goto'] = 'onboarding';

        $this->set_current_user_using_pid();

        wp_send_json( $data_array );
        die();

    }

    function verify_email_address( $data_array )
    {
        $data_array['status'] = 'success';
        $data_array['goto'] = 'onboarding';

        //login or create account
        $user = get_users(array('meta_key' => 'mam_app_token', 'meta_value' => $_REQUEST['pid']));

        if( !$user ){
            $handler = new tsl_mam_anon_user_manager();
            $handler->create_anon_user();
        }else{
            update_user_meta( $user[0]->ID, 'last_login', time() );
        }

        $jsonData = local_app_get_phone_data(true);
        $data_array['jsonData'] = $jsonData;
        mamdebug('login', __LINE__);

        $this->set_current_user_using_pid();

        wp_send_json( $data_array );
        die();

    }

    function do_logout( $data_array , $formdata )
    {
        if (isset($_REQUEST['pid'])) {
            $pid = $_REQUEST['pid'];
        } else {
            $pid = $formdata['pid'];
        }

        $user = get_users(array('meta_key' => 'mam_app_token', 'meta_value' => $pid));

        if ($user) {
            delete_user_meta($user[0]->ID, 'mam_app_token');
            delete_user_meta($user[0]->ID, '_tsl_anon_user_pid');
        }

        global $mam_user, $mam_user_id;
        $mam_user = null;
        $mam_user_id = null;

        $data_array['status'] = 'success';
        $jsonData = local_app_get_phone_data(true);
        $data_array['jsonData'] = $jsonData;
        mamdebug('login', __LINE__);

        wp_send_json($data_array);
        die();

    }

    function do_lost_password( $data_array, $formdata )
    {

         if (  $this->allow || $formdata['credkey'] == 'udsf97h35iletushfo8yeu5aksbvo7w6t3d47tfqux65rfi') {

            $user = get_user_by('login', $formdata['user']);

            if( !$user ){
                $user = get_user_by( 'email', trim($formdata['user']) );
            }

            if($user){

                $secret_code = mt_rand(100000, 999999);

                $data_array['code'] = $secret_code;

                update_user_meta( $user->ID, 'mam_pw_reset_code', $secret_code );

                $message = array();

                $message['message_type'] = 'mam-user-roles-lost_password';
                $message['send_text'] = false;
                $message['send_email'] = true;
                $message['send_pn'] = true;
                $message['recipient_id'] = $user->ID;
                $message['email'] = $user->user_email;
                $message['send_now'] = true;
                $message['target'] = 'none';
                $message['replacements'] = array( 'passwordcode' => $secret_code );
                $message['status'] = false;

                $message = apply_filters( 'mam_notification_send_message' , $message );

                //send email message
                if( $message['status'] == false ) {
                    $message = 'Please use the following code to reset your password.  Code: ' . $secret_code;
                    wp_mail($user->user_email, 'Password Reset for ', get_bloginfo('name'), $message);
                }

                $data_array['status'] = 'password_success';
                $data_array['message'] = 'We sent a code to your email address.  Enter the code and a new password to reset your password.';
            }else{

                $data_array['status'] = 'failed';
                $data_array['message'] = 'We did not find an account with that email address.  Please re-enter it in the field above to try again or sign up.';
            }
        }

        $this->set_current_user_using_pid();

        wp_send_json($data_array);
        die();

    }

    function update_lost_password( $data_array, $formdata )
    {

        if ( $this->allow || $formdata['credkey'] == 'udsf97h35iletushfo8yeu5aksbvo7w6t3d47tfqux65rfi') {

            $user = get_user_by('login', $formdata['user']);

            if( !$user ){
                $user = get_user_by( 'email', trim($formdata['user']) );
            }

            if($user){

                $secret_code = get_user_meta( $user->ID, 'mam_pw_reset_code', true );

                if( strlen($_REQUEST['passcode']) > 0 && $secret_code == $_REQUEST['passcode'] ) {

                    wp_set_password( $_REQUEST['password'], $user->ID );

                    $this->clean_users( $formdata['pid'] );

                    //register this phone to the user
                    update_user_meta( $user->ID, 'mam_app_token', $formdata['pid']);

                    if (strlen($formdata['deviceios'])) {
                        update_user_meta( $user->ID, 'mam_app_ios_pn_token', $formdata['deviceios']);
                    }
                    if (strlen($formdata['deviceand'])) {
                        update_user_meta( $user->ID, 'mam_app_android_pn_token', $formdata['deviceand']);
                    }
                    if (strlen($_REQUEST['deviceios'])) {
                        update_user_meta($user->ID, 'mam_app_ios_pn_token', $_REQUEST['deviceios']);
                    }
                    if (strlen($_REQUEST['deviceand'])) {
                        update_user_meta($user->ID, 'mam_app_android_pn_token', $_REQUEST['deviceand']);
                    }
                    $data_array['status'] = 'success';

                    update_user_meta( $user->ID, 'last_login', time() );

                    if (function_exists('local_app_get_phone_data')) {
                        $jsonData = local_app_get_phone_data(true);
                        $data_array['jsonData'] = $jsonData;
                        mamdebug('login', __LINE__);
                    }

                }else{
                    $data_array['status'] = 'failed';
                    $data_array['message'] = 'The code you entered is not valid.  Either check to make sure the code is correct or request a new code.';
                }
            }else{
                $data_array['status'] = 'failed';
                $data_array['message'] = 'We did not find an account with that email address.  Please re-enter it in the field above to try again to try again or sign up.';
            }
        }

        $this->set_current_user_using_pid();

        wp_send_json($data_array);
        die();
    }

    function create_replacements( $user_id )
    {

        $userdata = get_userdata( $user_id );
        $userdata = (array)$userdata;

        $user_array = array();

        foreach ( $userdata['data'] as $key => $value ){

            if( $key == 'user_pass' ) continue;
            $user_array[$key] = $value;
        }

        return $user_array;

    }


    private function do_role_multisite( $user_id, $args ){

        if(is_multisite()){

            $blog_id = get_current_blog_id();
            $user_blogs = get_blogs_of_user( $user_id );

            foreach($user_blogs as $blog ){
                if( $blog->userblog_id  == $blog_id ) return;
            }

            add_user_to_blog($blog_id, $user_id, $this->new_user_role );

        }

    }

    function modify_initial_phone_data(){

        if( isset( $_REQUEST['pid'] )) {
            $user = get_users(array('meta_key' => 'mam_app_token', 'meta_value' => $_REQUEST['pid']));
            if ($user) {

                $user_id = $user[0]->ID;
                $this->check_role($user_id);

                $args = array(
                    'meta_query' => array(
                        array(
                            'key' => 'admin_is_cloning',
                            'value' => $user_id,
                        )
                    )
                );
                $all_users = get_users($args);

                if (!empty($all_users))
                {
//                    if( ! isset( $_REQUEST['original_pid'] ) ) {
//                        $_REQUEST['original_pid'] = $_REQUEST['pid'];
//                    }
                    $_REQUEST['pid'] = get_user_meta($all_users[0]->ID, 'mam_app_token', true);
                    global $is_cloning;
                    $is_cloning = true;
                }
            }
        }
    }

    function set_current_user_using_pid()
    {

        global $current_user, $mam_user, $mam_user_id;

        if( isset( $_REQUEST['pid'] ) ) {
            $user = get_users(array('meta_key' => 'mam_app_token', 'meta_value' => $_REQUEST['pid']));
            if ($user) {
                $current_user = $user[0];
                $mam_user = $user[0];
                $mam_user_id = $user[0]->ID;
            }else {
                $current_user = null;
                $mam_user = null;
                $mam_user_id = 0;
            }
        }

    }

    function mam_cron_manager( $data_array )
    {

//        if(get_option('tsl_notification_email_status_mam-not-mgr-user-no-login-five-days', true) == 1 || get_option('tsl_notification_pn_status_mam-not-mgr-user-no-login-five-days', true) == 1 || get_option('tsl_notification_text_status_mam-not-mgr-user-no-login-five-days', true) == 1) {
//            $data_array[] = array('action' => 'mam_not_logged_within_last_five_days', 'name' => 'User Not Logged within 5 Days', 'frequency' => '1440', 'run_at' => '0600');
//        }

        return $data_array;
    }

    function mam_not_logged_within_last_five_days( $result ){

        $slug =  'mam-not-mgr-user-no-login-five-days';
        $all_users = get_users( array( 'meta_key' => 'mam_last_active', 'meta_value' => date('U', strtotime('-7 days')), 'meta_compare' => '<' ) );

        if(!empty($all_users)){
            foreach ($all_users as $user){
                if( $slug ==  'mam-not-mgr-user-no-login-five-days' && get_user_meta($user->ID, 'mam_not_mgr_five_day_message',true) != '' && get_user_meta($user->ID, 'mam_not_mgr_five_day_message',true) != '1'){
                    //fetch users that have not logged in in the past five days on the app and who have not yet received this message
                    //send messages
                    $message['message_type'] = 'unlockdbox-user-following-selected';
                    $message['send_text'] = false;
                    $message['send_email'] = true;
                    $message['send_pn'] = true;
                    $message['recipient_id'] = $user->ID;
                    $message['email'] = $user->data->user_email;
                    $message['send_now'] = true;
                    $message['target'] = $user->ID;
                    $message['replacements'] = array( 'display_name' => $user->data->display_name );
                    apply_filters('mam_notification_send_message', $message);
                    //update user meta for mam_not_mgr_five_day_message  to show it was sent
                    update_user_meta($user->ID, 'mam_not_mgr_five_day_message', date('U'));
                }
            }
        }

        $result['last_run'] = date( 'M d, Y H:i:s' );
        //$result['message'] = 'Notification sent to all users who are not logged in within 5 days';
        $result['status'] = 'success';

        return $result;
    }

    function mam_clone_role_content(){
        global $wpdb;
        parse_str($_POST['form'], $form_data);
        $clone_to_role = $form_data['role'];
        $clone_from_role = $form_data['clone-content-from-role'];
        $location = $form_data['location'];
        $table_name = $wpdb->prefix . 'mam_app_settings_buttons';
        $button_array = apply_filters( 'mam_app_settings_get_buttons', false, $clone_from_role, $location );
        if($button_array){
            $app_settings_handler = new tsl_mam_app_settings_main();
            $wpdb->query('DELETE FROM ' . $table_name .' WHERE role = "'.$clone_to_role.'" AND location = "'.$location.'"');
            foreach($button_array as $button){
                $button_id = $app_settings_handler->local_app_get_new_guid();
                $button['id'] = $button_id;
                apply_filters( 'mam_app_settings_set_button', $button_id, $clone_to_role, $location, $button );
            }
            $data_array = array('status' => 200, 'message' => 'Role content cloned successfully');
        }else{
            $data_array = array('status' => 400, 'message' => 'Please setup content for '.$clone_from_role.' role.');
        }

        wp_send_json($data_array);
        die();
    }

}
