<?php

use Pushok\AuthProvider;
use Pushok\Client;
use Pushok\Notification;
use Pushok\Payload;
use Pushok\Payload\Alert;

class mam_send_ios_push_notification
{

    private $ranProd = false;
    private $ranTest = false;

    function sendiosPush( $message, $url, $title, $phone_id = null , $data = null , $is_silent = 0 )
    {

        $handler = new mam_user_roles_manager();
        $pem_cert_file = $handler->get_p8_file();

        $org_phone_id = $phone_id;

        if( get_option( 'ios_pn_use_sandbox' ) == 'yes' ){
            $this->ranProd = true;
        }

        if( ! $this->ranProd ) {
            $production = true;
            $this->ranProd = true;
        }else{
            $production = false;
            $this->ranTest = true;
        }

        if( ! get_option( 'ios_pn_key_id') ) {
            update_option('ios_pn_key_id', '569JPQ4BRJ');
            update_option('ios_pn_team_id', '977BJ3E5P8');
            update_option('ios_pn_app_bundle_id', 'tinyscreenlabs.com.57fbd2a7218af');
        }

        if( $pem_cert_file && get_option( 'ios_pn_key_id') && get_option( 'ios_pn_team_id') && get_option( 'ios_pn_app_bundle_id')  ){
            $options['key_id'] = get_option( 'ios_pn_key_id');
            $options['team_id'] = get_option( 'ios_pn_team_id');
            $options['app_bundle_id'] = get_option( 'ios_pn_app_bundle_id');
            $options['private_key_path'] = $pem_cert_file;
            $options['private_key_secret'] = 'a101b102c103';

        }else{
            mamdebug( 'pnfailed', 'No Key Data ' . $pem_cert_file );
            return;
        }

        $alert = Alert::create()->setTitle( $title );
        $alert = $alert->setBody(stripslashes( $message ) );
        $payload = Payload::create()->setAlert($alert);
//        $payload->setContentAvailability( true );

        //set notification sound to default
        $payload->setSound('default' );

        if( $data ) {
            $payload->setCustomValue('data', $data );
        }
        if( $url ) {
            $payload->setCustomValue('target', $url );
        }
        mamdebug('pntest', __LINE__);
        if( apply_filters( 'mam_use_custom_sound_for_pn', false ) ) {
            $payload->setSound(apply_filters( 'mam_custom_sound_for_pn', 'default' ));

            mamdebug('pntest', apply_filters( 'mam_custom_sound_for_pn', 'default' ));

        }

        $deviceTokens = array();

        if( ! $phone_id ) {
            return array( 'status' => 'failed' );
        }else if( is_array( $phone_id )) {
            for ($x = 0;  $x < sizeof($phone_id);  $x++) {
                $deviceTokens[] = $phone_id[$x];
            }
        }else{
            $deviceTokens[] = $phone_id;
        }

        $notifications = [];
        foreach ($deviceTokens as $deviceToken) {
            $notifications[] = new Notification($payload,$deviceToken);
        }

        $authProvider = AuthProvider\Token::create($options);
        $client = new Client($authProvider, $production  );
        $client->addNotifications($notifications);

        try {
            $responses = $client->push();
        }catch (Exception $e ){
            mamdebug('PN error' , $e->getMessage() );
        }

        $response_data = array();

        foreach ($responses as $response) {
            $response_data['apnsid'] = $response->getApnsId();
            $response_data['status'] = $response->getStatusCode();
            $response_data['reason'] = $response->getReasonPhrase();
            $response_data['errorreason'] = $response->getErrorReason();
            $response_data['error'] = $response->getErrorDescription();

        }

        if( $response_data['status'] > 200 ) {
            if( ! $this->ranTest ){
                $this->sendiosPush( $message, $url, $title, $org_phone_id  , $data  );
            }else {
                mamdebug('response', $response_data);
            }
        }

    }


    function SendAndroidPushNotifications( $message, $target, $title, $phone_id = null , $data = null , $is_silent = 0 )
    {

        if( get_option( 'android_pn_key' )){
            $apikey = get_option( 'android_pn_key' );
        }else{
            //todo set defaults for TSL Previewer app after it is published


            return false;
        }

        if( ! $phone_id ) return false;

        $registrationIds = array();
        if( is_array( $phone_id )) {
            $registrationIds = $phone_id;
        }else{
            $registrationIds[] = $phone_id;
        }

        if (sizeof($registrationIds) == 0) return array('success' => 0);

        $url = 'https://fcm.googleapis.com/fcm/send';

        if( ! $target ) $target = site_url() . '?tsl_button=1000';

        $action = 'message';
        if( isset( $data['action'])) $action = $data['action'];

        $msg_data = array (
            "target" => $target,
            'title'	 => $title,
            'body' 	 => $message,
            'action' => $action
        );

        if( get_option('android_notification_icon') ){
            $msg_data['image'] = get_option('android_notification_icon');
        }

        if( $data ){
            foreach ( $data as $index => $item ) {
                $msg_data[ $index ] = $item;
            }
        }

        $fields = array
        (
            'registration_ids' 	=> $registrationIds,
            'data'      => $msg_data
        );

        $headers = array
        (
            'Authorization: key=' . $apikey,
            'Content-Type: application/json'
        );

        $ch = curl_init();
        curl_setopt( $ch,CURLOPT_URL, $url );
        curl_setopt( $ch,CURLOPT_POST, true );
        curl_setopt( $ch,CURLOPT_HTTPHEADER, $headers );
        curl_setopt( $ch,CURLOPT_RETURNTRANSFER, true );
        curl_setopt( $ch,CURLOPT_SSL_VERIFYPEER, false );
        curl_setopt( $ch,CURLOPT_POSTFIELDS, json_encode( $fields ) );
        $result = curl_exec($ch );
        curl_close( $ch );

        $result_msg['phones'] = $fields;
        $result_msg['api'] = $apikey;

        return $result_msg;


    }


}
