<?php
/*
Plugin Name: Mobile App Manager Main
Plugin URI: https://tinyscreenlabs.com
Description: Mobile App Manager makes it easy to publish your WordPress website content to iOS and Android mobile apps.
Version: 2.1
Author: Tiny Screen Labs, LLC
Author URI: https://tinyscreenlabs.com
License: https://tinyscreenlabs.com/license
Text Domain: tsl-local-app
*/


$currentVersion = phpversion();

if (version_compare($currentVersion, '8.1', '>')) {

    define('TSL_AFF_URL', 'https://tinyscreenlabs.com/pricing/');

    global $local_app_db_version;
    $local_app_db_version = 48;

    global $local_app_js_version;
    $local_app_js_version = '1.51';

    global $tsl_local_app_image_url;
    $tsl_local_app_image_url = plugin_dir_url(__FILE__);

    global $tsl_local_app_dir_path;
    $tsl_local_app_dir_path = plugin_dir_path(__FILE__);

    global $local_app_plugins_dir;
    $local_app_plugins_dir = plugin_dir_path(__DIR__);

    include_once 'includes/helper-classes/mam-classless-functions.php';
    include_once 'includes/user-roles/mam-user-roles.php';
    include_once 'includes/content-class-elements/general-elements.php';
    include_once 'includes/wp-local-app-functions.php';
    include_once 'includes/local-app-mobile-detect.php';
    include_once 'includes/local-app-language-mgr.php';
    include_once 'includes/plugin-update-checker/plugin-update-checker.php';
    include_once 'includes/local-app-notifications-manager.php';
    include_once 'includes/local-app-badge-shortcodes.php';
    include_once 'includes/grid-classes/debug-grid.php';
    include_once 'includes/grid-classes/test-users-grid.php';
    include_once 'includes/grid-classes/test-wp-users-grid.php';
    include_once 'includes/grid-classes/software-grid.php';
    include_once 'includes/grid-classes/notifications-grid.php';
    include_once 'includes/debug-handler.php';
    include_once 'includes/helper-classes/admin-settings-functions.php';
    include_once 'includes/helper-classes/update-cached-json-status.php';
    include_once 'includes/helper-classes/home-screen-stack-manager.php';
    include_once 'includes/setcron-manager/mam-setcron-manager.php';
    include_once 'includes/setcron-manager/admin-settings.php';
    include_once 'includes/setcron-manager/cron-list.php';
    include_once 'includes/setcron-manager/setcron-manager.php';
    require_once 'includes/app-settings/mam-app-settings.php';
    require_once 'includes/app-settings/admin-settings-page.php';
    require_once 'includes/app-settings/button-manager-class.php';
    require_once 'includes/app-settings/global-settings.php';
    require_once 'includes/app-settings/local-settings.php';
    include_once 'includes/app-settings/push-notifications-manager.php';
    require_once 'includes/notification-manager/mam-notification-manager.php';
    require_once 'includes/notification-manager/admin-settings.php';
    require_once 'includes/notification-manager/message-sender.php';
    require_once 'includes/notification-manager/outbox-viewer.php';
    require_once 'includes/notification-manager/sent-grid.php';
    require_once 'includes/notification-manager/opt-out-grid.php';
    require_once 'includes/notification-manager/custom-notifications-list.php';
    require_once 'includes/send-ios-push-notification.php';
    require_once 'includes/local-app-software-manager.php';
    require_once 'includes/comments-manager/post-comments.php';
    require_once 'includes/mam-object-caching.php';
    require_once 'includes/helper-classes/geocoding-class.php';
    require_once 'includes/support-manager/support-manager.php';
    require_once 'includes/support-manager/support-ticket-grid.php';
    require_once 'includes/helper-classes/user-manager-classes.php';
    include_once 'includes/local-app-content-class-manager.php';
    include_once 'includes/main-json-manager.php';
    include_once 'includes/form-cache-manager.php';
    include_once 'includes/setup-wizard/setup-wizard.php';
    include_once 'includes/codex-manager.php';
    include_once 'includes/rest-api-connection.php';
    include_once 'includes/qr-code-manager.php';
    include_once 'includes/get-icons-from-server.php';
    include_once 'includes/app-settings/app-settings-data-manager.php';
    include_once 'includes/database-manager.php';

    require_once 'includes/mail-cron-manager/mam-mail-manager.php';
    require_once 'includes/mail-cron-manager/mail-queue-class.php';
    require_once 'includes/mail-cron-manager/admin-settings.php';
    require_once 'includes/mail-cron-manager/mail-log-grid.php';


    require_once __DIR__ . '/vendor/autoload.php';

    add_action('plugins_loaded', array('tal_mam_main_manager', 'init'), 1);

}else{

    add_action('admin_notices', 'mam_php_version_error');
    function mam_php_version_error()
    {
        ?>
        <div class="error notice">
            <p><?php _e('Mobile App Manager requires PHP 8.1 or higher. Please upgrade your PHP version.', 'mam-main'); ?></p>
        </div>
        <?php
    }
}

use YahnisElsts\PluginUpdateChecker\v5\PucFactory;



class tal_mam_main_manager
{

    public $app_version = '1.0';
    public $plugin_slug = 'mam-main-plugin';
    public $plugins_update_required = array();

    public static function init()
    {
        $class = __CLASS__;
        new $class;
    }

    public function __construct()
    {

//        if( isset($_REQUEST['reset-acct-code']))
//        {
//            delete_option('local-app-account_code');
//            mamdebug('acct_code_reset', get_option('local-app-account_code'));
//        }

        global $mam_mam_plugin_folder_path, $mam_mam_plugin_folder_url;
        $mam_mam_plugin_folder_path = plugin_dir_path( __FILE__ );
        $mam_mam_plugin_folder_url = plugin_dir_url( __FILE__ );

        if( isset( $_REQUEST['pid'] )) $_REQUEST['original_pid'] = $_REQUEST['pid'];

        //vetted below this line
        add_filter( 'mam_main_is_active', '__return_true' );
        add_action( 'init', array( $this, 'initialize_debug_tracer' ), 1 );

        $database = new mam_main_database_manager();
        add_action( 'admin_init', array( $database, 'manage_tables' ) );

        $app_settings = new mam_app_settings_data_manager();
        add_action( 'admin_init', array($app_settings, 'manage_database'));

        add_filter( "plugin_action_links_mam-main/mam-main.php", array($this, 'mam_main_form_settings_link') );
        add_action( 'add_to_tracer' , array( $this, 'add_to_tracer'));
        add_action( 'mam_set_plugin_update_info' , array( $this, 'set_update_for_plugin_update_checker') , 1, 2 );
        add_filter( 'site_transient_update_plugins', array( $this, 'update_site_transient'), 10, 2 );
        add_action( 'admin_init', array( $this, 'local_app_load_plugin'));
        add_action( 'admin_init', array( $this, 'check_required_plugins'), 1000 );
        add_action( 'admin_menu', array( $this, 'add_admin_menus' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'local_app_enqueue_scripts' ));
        add_action( 'wp_ajax_mam_main_ajax', array( $this , 'mam_main_ajax' ));
        add_action( 'wp_ajax_nopriv_mam_main_ajax', array( $this , 'mam_main_ajax' ));
        add_action( 'admin_notices', array( $this, 'admin_alert_for_debug_log' ) );
        add_action( 'admin_footer', array( $this, 'support_submenu_page_redirect' ) );
        add_filter( 'mam_check_required_version', array( $this, 'check_required_version' ), 10, 2);
        add_filter( 'mam_main_add_to_home_category_list', array( $this, 'mam_main_add_to_home_category_list' ));
        add_filter( 'tsl_get_phone_data_before_send', array( $this , 'manage_phone_data') , 1000 );
        add_filter( 'mam_user_has_role', array( $this, 'mam_user_has_role'), 10 , 3 );
        add_action( 'wp_ajax_local_app_get_app', array($this, 'local_app_get_app') );
        add_action( 'wp_ajax_nopriv_local_app_get_app', array($this, 'local_app_get_app') );
        add_action( 'admin_bar_menu', array( $this, 'add_link_to_admin_bar'), 999);

        add_action( 'wp_ajax_mam_notification_opt_out', array( $this , 'mam_notification_opt_out' ));
        add_action( 'wp_ajax_nopriv_mam_notification_opt_out', array( $this , 'mam_notification_opt_out' ));

        //publishing actions move to separate class if relevant?
        add_action( 'wp_ajax_local_app_save_publish_app_settings', array($this, 'local_app_save_publish_app_settings') );
        add_action( 'wp_ajax_local_app_publish_in_phone', array($this, 'local_app_publish_in_phone') );
        add_action( 'wp_ajax_local_app_preview_in_phone', array($this, 'local_app_preview_in_phone') );
        add_action( 'wp_ajax_local_app_send_preview', array($this, 'local_app_send_preview') );

        $user_manager = new mam_user_manager_functions();
        add_action( 'init', array( $user_manager, 'mam_update_current_user') , 1 );
        add_filter( 'mam_get_user', array( $user_manager, 'mam_get_user'));
        add_filter( 'mam_get_user_id', array( $user_manager, 'mam_get_user_id'));
        add_action( 'mam_update_current_user', array( $user_manager, 'mam_update_current_user') );
        add_filter( 'convert_datetime_to_user_date_time', array( $user_manager, 'convert_datetime_to_user_date_time') , 10, 4 );
        add_action( 'mam_ping_item_view', array( $user_manager, 'mam_ping_item_view'));
        add_filter( 'user_row_actions', array( $user_manager, 'add_logout_user_link'), 10, 2);

        //convert to mam_get_user then delete
        add_filter( 'mam_get_current_app_user' , array( $user_manager, 'mam_get_user' ));

        $codex_handler = new mam_codex_manager();
        add_action( 'admin_init', array( $codex_handler, 'update_settings_status' ) );

        $admin_settings = new mam_setcron_manager_admin_settings();
        add_action( 'admin_init', array($admin_settings, 'mam_setcron_save_settings') );

        $mam_setcron_manager = new mam_setcron_manager();
        add_action( 'wp_ajax_mam_setcron_processor', array( $mam_setcron_manager, 'mam_setcron_processor' ));
        add_action( 'wp_ajax_nopriv_mam_setcron_processor', array( $mam_setcron_manager, 'mam_setcron_processor' ));

        $form_cache = new mam_form_manager_cache_manager();
        add_filter( 'mam_is_form_in_array' , array( $form_cache, 'is_form_in_array' ) , 10 , 2 );
        add_filter( 'mam_add_or_update_form_in_array' , array( $form_cache, 'add_or_update_form_in_array' ) , 10 , 2 );

        //phone validation
        $phone_valid = new mam_validate_phone_code();
        add_filter( 'mam_notification_list', array($phone_valid, 'mam_notification_list'));
        add_filter( 'mam_main_ajax', array($phone_valid, 'get_phone_validation_code'));

        //wizard functions
        $wizard = new mam_setup_wizard();
        add_filter( 'init', array( $wizard, 'setup_roles' ) , 100 );
        add_action( 'admin_init', array( $wizard, 'initial_launch' ) );

        //QR Code
        $qr_code = new mam_push_notifications_settings_manager();
        add_action( 'mam_get_main_qr_code', array( $qr_code, 'get_main_qr_code' ) );
        add_action( 'mam_create_qr_code', array( $qr_code, 'create_qr_code' ), 10, 2 );

        //maybe not needed///need to test new account setup to determine if needed
        add_action( 'wp_ajax_local_app_activate_api', array($this, 'local_app_activate_api') );
        add_action( 'wp_ajax_nopriv_local_app_activate_api', array($this, 'local_app_activate_api') );

        //most likely deletable
//        add_action( 'wp_ajax_local_app_set_onboarding_status', array($this, 'local_app_set_onboarding_status') );
//        add_action( 'wp_ajax_nopriv_local_app_set_onboarding_status', array($this, 'local_app_set_onboarding_status') );

        $mam_mail_manager = new tsl_mam_mail_manager();
        add_filter( 'mam_cron_manager' , array( $mam_mail_manager, 'mam_cron_manager' ));

        $mam_app_settings = new tsl_mam_app_settings_main();
        add_action( 'wp_ajax_advanced_settings_ajax_handler', array($mam_app_settings, 'local_app_ajax_handler'));
        add_action( 'wp_ajax_nopriv_advanced_settings_ajax_handler', array($mam_app_settings, 'local_app_ajax_handler'));
        add_filter( 'mam_main_content_class_settings', array( $mam_app_settings, 'mam_main_content_class_settings'), 10, 4);
        add_action( 'wp_ajax_mam_app_ajax_handler', array($mam_app_settings, 'local_app_ajax_handler'), 1);
        add_action( 'wp_ajax_nopriv_mam_app_ajax_handler', array($mam_app_settings, 'local_app_ajax_handler'), 1);
        add_filter( 'tsl_get_phone_data_before_send', array( $mam_app_settings , 'manage_phone_data'), 1 );
        add_filter( 'mam_final_button_settings', array( $mam_app_settings, 'mam_final_button_settings' ));
        add_action( 'admin_init' , array( $mam_app_settings, 'admin_init' ));
        add_action( 'wp_ajax_local_app_save_file', array($mam_app_settings, 'local_app_save_file'), 1);
        add_action( 'wp_ajax_nopriv_local_app_save_file', array($mam_app_settings, 'local_app_save_file'), 1);
        add_action( 'wp_ajax_local_app_save_plist_file', array($mam_app_settings, 'local_app_save_plist_file'), 1);
        add_action( 'wp_ajax_nopriv_local_app_save_plist_file', array($mam_app_settings, 'local_app_save_plist_file'), 1);
        add_action( 'wp_ajax_local_app_get_data', array($mam_app_settings, 'local_app_get_data'), 1);
        add_action( 'wp_ajax_nopriv_local_app_get_data', array($mam_app_settings, 'local_app_get_data'), 1);
        add_action( 'wp_ajax_local_app_save_social', array($mam_app_settings, 'local_app_save_social') );
        add_action( 'wp_ajax_local_app_update_image', array($mam_app_settings, 'local_app_update_image') );
        add_action( 'wp_ajax_local_app_update_button_data', array($mam_app_settings, 'local_app_update_button_data') );
        add_action( 'admin_enqueue_scripts', array($mam_app_settings, 'admin_manager_scripts'));
        add_action( 'wp_ajax_mam_get_home_cat_section_term', array($mam_app_settings, 'mam_get_home_cat_section_term'), 1);
        add_action( 'wp_ajax_nopriv_mam_get_home_cat_section_term', array($mam_app_settings, 'mam_get_home_cat_section_term'), 1);
        add_action( 'wp_ajax_mam_get_role_buttons', array($mam_app_settings, 'mam_get_role_buttons'), 1);
        add_action( 'wp_ajax_nopriv_mam_get_role_buttons', array($mam_app_settings, 'mam_get_role_buttons'), 1);
        add_action( 'wp_ajax_local_app_resize_image', array($mam_app_settings, 'local_app_resize_image'));
        add_action( 'wp_ajax_local_app_generate_launch_screen', array($mam_app_settings, 'local_app_generate_launch_screen'));

        $app_setting_handler = new mam_app_settings_admin_page();
        add_filter( 'mam_filter_settings_for_public_use', array( $app_setting_handler, 'filter_settings_for_public_use') );
        add_filter( 'mam_settings_set_help_desc', array( $app_setting_handler, 'set_description') );

        $mam_notification_manager = new mam_message_and_notification_manager();
        add_filter('admin_menu', array($mam_notification_manager, 'mam_admin_menu_manager'));
        add_filter('mam_notification_send_message', array($mam_notification_manager, 'send_message_handler'));
        add_action('admin_enqueue_scripts', array($mam_notification_manager, 'admin_manager_scripts'));
        add_shortcode('mam_mail_opt_out', array($mam_notification_manager, 'mail_opt_out'));
        add_action( 'mam_notification_send_pn', array( $mam_notification_manager , 'send_push_notification') , 10 , 1 );
        add_action( 'wp_ajax_nopriv_mam_main_send_test_not', array($mam_notification_manager, 'mam_main_send_test_not') );
        add_action( 'wp_ajax_mam_main_send_test_not', array($mam_notification_manager, 'mam_main_send_test_not') );
        add_action( 'wp_ajax_mam_send_test_email', array($mam_notification_manager, 'mam_send_test_email'));
        add_filter( 'mam_notification_list', array($mam_notification_manager, 'mam_notification_list'));
        add_filter( 'mam_cron_manager', array( $mam_notification_manager, 'mam_cron_manager' ));
        add_filter( 'mam_custom_notifications', array( $mam_notification_manager, 'mam_trigger_custom_notifications') );

        $handler = new mam_main_get_forum_post_comments();
        add_filter( 'mam_forum_post_comments_for_post', array( $handler, 'get_comments'), 10, 2 );
        add_action( 'mam_main_ajax', array( $handler, 'mam_main_ajax' ));

        $form_handler = new mam_user_role_form_manager();
        add_filter( 'mam_gf_get_form_settings', array( $form_handler, 'mam_gf_get_form_settings' ));
        add_filter( 'mam_for_gravity_forms_form_result_form_'.get_option( 'woocommerce_user_billing_address_form' ), array( $form_handler, 'save_user_profile') );
        add_filter( 'tsl_get_phone_data_before_send', array( $form_handler , 'process_last_json_check'), 199 );
        add_filter( 'tsl_update_form_before_sending', array( $form_handler, 'mam_populate_user_profile_form' ));
        add_action( 'pre_get_posts', array( $form_handler, 'mam_hide_user_avatar_media') );

        $user_roles = new mam_user_roles_manager();
		add_filter( 'tsl_get_phone_data_before_send' , array( $user_roles , 'manage_phone_data') );
        add_filter( 'mam_for_plugin_settings_section' , array( $user_roles , 'plugin_settings_section') ,10,1 );
		add_action( 'wp_ajax_mam_user_roles_ajax_handler', array($user_roles, 'ajax_handler' ));
		add_action( 'wp_ajax_nopriv_mam_user_roles_ajax_handler', array($user_roles,  'ajax_handler' ));
        add_action( 'wp_ajax_mam_user_roles_cred_handler', array($user_roles, 'ajax_cred_handler' ));
		add_action( 'wp_ajax_nopriv_mam_user_roles_cred_handler', array($user_roles,  'ajax_cred_handler' ));
		add_action( 'user_profile_update_errors', array( $user_roles , 'empty_email_error' ));
		add_filter( 'mam_notification_list', array($user_roles, 'mam_notification_list'));
        add_action( 'edit_user_profile', array( $user_roles , 'custom_user_profile_fields'));
        add_action( 'show_user_profile', array( $user_roles , 'custom_user_profile_fields'));
        add_action( 'mam_initial_phone_data', array( $user_roles , 'modify_initial_phone_data' ));
        add_action( 'wp_ajax_mam_clone_role_content', array($user_roles, 'mam_clone_role_content' ));

        add_filter( 'mam_not_logged_within_last_five_days', array( $user_roles, 'mam_not_logged_within_last_five_days') );
        add_filter( 'mam_cron_manager', array( $user_roles, 'mam_cron_manager' ));
        add_action( 'admin_enqueue_scripts', array($user_roles, 'admin_manager_scripts'));

        $support_manager_settings = new mam_support_manager_admin_settings();
        add_action( 'admin_init', array($support_manager_settings, 'mam_support_manager_save_settings') );

        //rest api
        $rest_api = new mam_rest_api_connection();
        add_action( 'rest_api_init', array( $rest_api, 'register_routes' ) );

        $icons = new mam_get_icons_after_install();
        add_action( 'admin_init', array( $icons, 'mam_main_get_icons'));

        $push_notification = new mam_main_send_ios_push_notification();
        add_action( 'wp_ajax_mam_send_voip_push_notification', array( $push_notification, 'send_voip_notification' ), 10 );


        add_action( 'admin_init', array( $this, 'test_app_icons'));

        add_filter( 'plugin_row_meta', array( $this, 'plugin_row_meta'), 10, 3 );

    }

    function plugin_row_meta($plugin_meta, $plugin_file, $plugin_data ) {

        if( isset($plugin_data['slug']) && strpos( $plugin_data['slug'] , 'mam-' ) !== false ) {

            foreach ($plugin_meta as $key => $meta) {
                if (strpos($meta, 'View details') !== false) unset($plugin_meta[$key]);
            }

            $file = WP_PLUGIN_DIR .  '/'. $plugin_data['slug'] . '/readme.html';
            if( file_exists( $file ) ) {

                $url = WP_PLUGIN_URL . '/' . $plugin_data['slug'] . '/readme.html';

                $plugin_meta[] = sprintf(
                    '<a href="%s" class="thickbox open-plugin-details-modal" aria-label="%s" data-title="%s">%s</a>',
                    add_query_arg('TB_iframe', 'true', $url),
                    esc_attr(sprintf(__('More information about %s'), $plugin_data['Name'])),
                    esc_attr($plugin_data['Name']),
                    __('View details')
                );
            }
        }
        return $plugin_meta;
    }

    function test_app_icons()
    {

//        $plugs = get_site_transient( 'update_plugins' );
//        mamdebug('update_plugins', $plugs );

        if( isset( $_REQUEST['test_app'] ) )
        {
            $_REQUEST['ostype'] = 1;
            $this->local_app_publish_in_phone();

//            $icons = new local_app_submit();
//            $icons->create_ios_app_icons();
//            $icons->create_android_app_icons();
        }

    }

    function local_app_get_app(){
mamdebug('ios', $_REQUEST );
        $detect = new Mobile_Detect_2;

        mamdebug('ios', get_option( 'mam-itunes-app-url' ) );
        if( $detect->isiOS() && get_option( 'mam-itunes-app-url' )){
            $appleID = get_option( 'mam-itunes-app-url' );
            mamdebug('ios', $appleID );
            if( isset($_REQUEST['client_code']) && strlen($_REQUEST['client_code']) > 0 ) $appleID .= '?client_code=' . $_REQUEST['client_code'];
mamdebug('ios', $appleID );
            if( strlen($appleID) > 10 ) wp_redirect($appleID);  //good redirect
            exit;
        }

        if( $detect->isAndroidOS() && get_option( 'mam-android-app-url' )){
            $androidID = get_option( 'mam-android-app-url' );
            if( isset($_REQUEST['client_code']) && strlen($_REQUEST['client_code']) > 0 ) $androidID .= '?client_code=' . $_REQUEST['client_code'];
            if( strlen($androidID) > 10 ) wp_redirect($androidID);
            exit;
        }

        $platform = get_site_url();
        wp_redirect($platform);    //good redirect
        exit;

    }

    function mam_notification_opt_out()
    {
        if( isset($_REQUEST['code'])) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'mam_email_opted_out';
            $opt_out = $wpdb->get_results("SELECT * FROM $table_name WHERE email_code = '" . $_REQUEST['code'] . "'", ARRAY_A);

            if (sizeof($opt_out) > 0)
            {
                mamdebug('optout', $opt_out );
                $wpdb->update($table_name, array('opted_out' => time()), array('email_code' => $_REQUEST['code']));
            }
        }
        if( get_option( 'mam-opt-out-complete-url' ) && strlen( get_option( 'mam-opt-out-complete-url' ) ) > 0 ) {
            wp_redirect(get_option('mam-opt-out-complete-url'));
        }else {
            wp_redirect(site_url() . '/?mam_email_opt_out=1');
        }
        die();
    }

    function mam_user_has_role( $has_role, $role, $user_id )
    {

        if( $user_id == 0 ) return false;
        $user = get_userdata( $user_id );
        if( $user )
        {
            if( !is_array( $role ) ) $role = array( $role );
            foreach( $role as $user_role  ) {
                if (in_array($user_role, $user->roles)) return true;
            }
        }
        return $has_role;
    }

    function mam_main_add_to_home_category_list( $category )
    {

        global $mam_main_add_to_home_category_list;
        if( ! is_array( $mam_main_add_to_home_category_list ) ) $mam_main_add_to_home_category_list = array();
        if( ! in_array( $category, $mam_main_add_to_home_category_list ) ) $mam_main_add_to_home_category_list[] = $category;

    }

    function manage_phone_data( $data_array )
    {
        do_action('add_to_tracer', 'mam_main ' . __LINE__);
        global $mam_main_add_to_home_category_list, $mam_user_role;
        $user_role = $mam_user_role;
        if( ! is_array( $mam_main_add_to_home_category_list ) ) $mam_main_add_to_home_category_list = array();
        if( sizeof($mam_main_add_to_home_category_list) > 0 ) {
            asort($mam_main_add_to_home_category_list);
            $data_array['home_cats'] = array();
            foreach ($mam_main_add_to_home_category_list as $cat) {
                if( strlen($cat) == 0 ) continue;
                if (!in_array($cat, $data_array['home_cats'])) $data_array['home_cats'][] = array('name' => $cat);
            }
        }

        $home_cat_sections = apply_filters( 'mam_app_settings_get_home_screen_categories', array() , $user_role );
        if(sizeof($home_cat_sections) > 0){
            foreach ($home_cat_sections as $cat) {
                if( strlen($cat['term']) == 0 ) continue;
                $data_array['home_cats'][] = array('name' => $cat['term'] , 'sort' => $cat['sort'] , 'order' => $cat['order']);
            }
        }
        if( ! isset($data_array['home_cats']) || ! is_array($data_array['home_cats']) ) $data_array['home_cats'] = array();

        $data_array['home_cats'] = apply_filters( 'mam_main_final_home_cats', $data_array['home_cats'] );

        if( ! is_array($data_array['home_cats']) || sizeof($data_array['home_cats']) == 0 ) unset( $data_array['home_cats'] );

        do_action('add_to_tracer', 'mam_main ' . __LINE__);

        $data_array['login_format_option_2'] = 'yes';

        return $data_array;
    }

    function add_link_to_admin_bar($admin_bar) {
        $args = array(
            'parent' => 'site-name',
            'id' => 'mobile-app-manager',
            'title' => 'Mobile App Manager',
            'href' => site_url() . '/wp-admin/admin.php?page=mam-app-settings',
            'meta' => false
        );
        $admin_bar->add_node($args);
    }


    function check_required_version( $requires = '' , $plugin = 'MAM Main')
    {
        if ( is_admin() ) {
            if (!function_exists('get_plugin_data')) {
                require_once(ABSPATH . 'wp-admin/includes/plugin.php');
            }
            $plugin_data = get_plugin_data(__FILE__);

            // Extract the version from the plugin data
            $version = $plugin_data['Version'];

            // Compare the version
            $message = 'The plugin ' . $plugin . ' requires MAM Main with version ' . $requires . ' or higher. Your current version is ' . $version;
            if (version_compare($version, $requires, '<')) {
                mamdebug('admin_alert', $message);
            }else{
                global $wpdb;
                $table_name = $wpdb->prefix . 'tsl_debug_items';
                $wpdb->get_results("DELETE FROM $table_name WHERE message_info LIKE '".$message."'");
            }

            return $version;
        }
        return '1.9.1';

    }

    function check_required_plugins()
    {

        $plugins = apply_filters( 'mam_required_plugins', array() );
        if( ! is_array( $plugins ) ) $plugins = array();
        global $wpdb;
        $table_name = $wpdb->prefix . 'tsl_debug_items';

        foreach( $plugins as $plugin )
        {
            $message = 'Required plugin not active: ' . $plugin['name'] .'. Get the plugin here: <a href="' . $plugin['url'] . '" target="_blank">' . $plugin['name'] . '</a>';
            if( ! is_plugin_active( $plugin['slug'] ) )
            {
                mamdebug('admin_alert', $message);
            }else{
                $wpdb->get_results("DELETE FROM $table_name WHERE message_info LIKE '".$message."'");
            }
        }
    }

    function initialize_debug_tracer()
    {
        global $mam_time_start;
        if( ! $mam_time_start ) {
            $mam_time_start = microtime(true);
        }
    }

    function mam_main_form_settings_link($links) {
        $mylinks = array( '<a href="' . admin_url( 'admin.php?page=local_app_setup' ) . '">Settings</a>', );
        return array_merge( $mylinks , $links );
    }

    function local_app_load_plugin() {

        if(!get_option('local-app-account_code')) {
            if (is_admin() && !get_option('local-app-processing')) {

                update_option('local-app-processing', '1');

                delete_option('local-app-activated-plugin');
                $local_host = 0;
                if (!local_app_is_connected_to_internet()) $local_host = 1;

                $querystring = 'https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=tsl_is_connected&aff_code=none&local_host=' . $local_host . '&ajax_url=' . admin_url('admin-ajax.php');

                $response = wp_remote_get($querystring);

                if (is_array($response) && !is_wp_error($response)) {
                    $data = json_decode(wp_remote_retrieve_body($response), true);
                    if (isset($data['account_code'])) update_option('local-app-account_code', $data['account_code']);
                    if ($local_host == 1) update_option('local-app-account_code', 'LOCALHOST');
                    delete_option('local-app-processing');
                }
            }
        }
    }

    function admin_alert_for_debug_log()
    {

        if( get_option('local-app-admin-message_nag') )
        {

            global $wpdb;
            $table_name = $wpdb->prefix . 'tsl_debug_items';

            $data = $wpdb->get_results("SELECT message_from FROM $table_name  WHERE message_from LIKE 'admin_alert'", ARRAY_A);
            if (sizeof($data) > 0) {
                $class = 'notice notice-info';
                $message = 'You have Mobile App Manger admin notifications in the Debug Log';
                printf('<div id="local-app-config-message" class="%1$s"><p>%2$s <span style="float:right">| <a href="/wp-admin/admin.php?page=mam-debug-logger-manager-settings&tag=admin_alert">Open Log</a></span></p></div>', $class, $message);
            }
        }
    }

    function update_site_transient($value, $transient)
    {

        if( ! is_object($value) ) return $value;

        $mam_plugins_for_update = get_option('mam_plugins_for_update');
        if (!is_array($mam_plugins_for_update)) $mam_plugins_for_update = array();

        foreach( $mam_plugins_for_update as $slug => $file_path ) {
            $plugin_name = $slug . '/' . $slug . '.php';
            $value_response = (array)$value->response;
            if ( !array_key_exists($plugin_name, $value_response)) {
                if( !empty( get_transient('update_info_' . $slug)) && is_object(get_transient('update_info_' . $slug))) {
                    $value->response[$plugin_name] = get_transient('update_info_' . $slug);
                } else {
                    $apiKey = get_option('mam-main-software-manager-api');
                    $plugin_data = get_plugin_data($file_path);
                    $plugin_version = $plugin_data['Version'];
                    $response = wp_remote_get('https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=wc_private_repo_check_update&slug=' . $slug . '&url=' . site_url() . '&api_key=' . $apiKey . '&installed_version=' . $plugin_version);
                    if (is_array($response) && !is_wp_error($response)) {
                        $body = $response['body'];
                        $data = json_decode($body, true);
                        if ( isset($data['new_version']) && $data['new_version'] != $plugin_version) {
                            $value->response[$plugin_name] = (object)$data;
                        }
                        set_transient('update_info_' . $slug, (object)$data, 43200); //12 hours
                    }
                    return $value;  //only do one at a time to manage performance
                }
            }
        }

        return $value;
    }

    function set_update_for_plugin_update_checker( $slug , $file_path )
    {

        if( get_option('mam-main-software-manager-api') || $slug == 'mam-main' ){
            $mam_plugins_for_update = get_option('mam_plugins_for_update');
            if( ! is_array($mam_plugins_for_update) ) $mam_plugins_for_update = array();
            $mam_plugins_for_update[$slug] = $file_path;
            update_option('mam_plugins_for_update', $mam_plugins_for_update);
            if( strpos($_SERVER['REQUEST_URI'], 'plugins.php') ) {
                $apiKey = get_option('mam-main-software-manager-api');
                $plugin_data = get_plugin_data( $file_path );
                $plugin_version = $plugin_data['Version'];
                if ($slug == 'mam-main' && !get_option('mam-main-software-manager-api')) $apiKey = 'notrequired';
//root cause is here.  in PUC the timeout is set to 300ms and this script times out.  So I set it to a static json
                PucFactory::buildUpdateChecker('https://tinyscreenlabs.com/wp-content/plugins/woocommerce-private-repo/update_json/'.$slug.'.json', $file_path, $slug);
//                PucFactory::buildUpdateChecker('https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=wc_private_repo_check_update&slug=' . $slug . '&url=' . site_url() . '&api_key=' . $apiKey . '&installed_version=' . $plugin_version, $file_path, $slug);
            }
        }
    }

    function add_to_tracer( $tag )
    {

        global $mam_time_start, $time_tracers;
        if( ! is_array($time_tracers))
        {
            $time_tracers = array();
        }
        if( ! $mam_time_start ) {
            $mam_time_start = microtime(true);
        }
        $time_tracers[$tag] = microtime(true) - $mam_time_start;

    }

    function mam_main_ajax()
    {

        if( isset( $_REQUEST['subaction'] )){

            if( isset($_REQUEST['subaction'] ) && $_REQUEST['subaction'] == 'admin_clone' )
            {

                $admin_id = mam_user_id();
                if( isset( $_REQUEST['original_pid']))
                {
                    $users = get_users(array('meta_key' => 'mam_app_token', 'meta_value' => $_REQUEST['original_pid'] ));
                    if( $users == 1)
                    {
                        $admin_id = $users[0]->ID;
                    }
                }

                $users = get_users(array('meta_key' => 'admin_is_cloning', 'value' => $admin_id ));
                foreach( $users as $app_user ) {
                    delete_user_meta( $app_user->ID, 'admin_is_cloning' );
                }
                update_user_meta( $_REQUEST['clone_user'], 'admin_is_cloning', $admin_id );
                $data_array['status'] = 'success';
                $data_array['jsonData'] = local_app_get_phone_data(true);
                wp_send_json( $data_array );
                die();

            }else if( isset($_REQUEST['subaction'] ) && $_REQUEST['subaction'] == 'reset_clone' ) {

                $admin_id = get_user_meta( mam_user_id(), 'admin_is_cloning', true );
                $users = get_users(array('meta_key' => 'admin_is_cloning', 'value' => $admin_id ));
                foreach( $users as $app_user ) {
                    delete_user_meta( $app_user->ID, 'admin_is_cloning' );
                }
                $_REQUEST['pid'] = get_user_meta( $admin_id, 'mam_app_token', true );
                global $is_cloning;
                $is_cloning = false;

                $data_array['status'] = 'success';
                $data_array['jsonData'] = local_app_get_phone_data(true);
                wp_send_json( $data_array );
                die();
            }else if( isset($_REQUEST['subaction'] ) && $_REQUEST['subaction'] == 'mam_entered_background' ) {
                $data_array['status'] = 'success';
                wp_send_json( $data_array );
                die();
            }else if( $_REQUEST['subaction'] == 'mam_update_json' ){
                $data_array['status'] = 'success';
                $data_array['jsonData'] = local_app_get_phone_data(true);
                wp_send_json( $data_array );
                die();
            }else if ( has_action( $_REQUEST['subaction'] ) ) {

                do_action( $_REQUEST['subaction'] );
                if( isset($_REQUEST['reload'])){

                    $data_array['status'] = 'success';
                    $data_array['jsonData'] = local_app_get_phone_data(true);
                    wp_send_json( $data_array );
                    die();
                }
            }
        }

        do_action( 'mam_main_ajax' );

    }

    function local_app_is_settings_page(){

        if(isset($_REQUEST['page'])) {
            if (strstr($_REQUEST['page'], 'local_app')) return true;
        }
        return false;

    }

    function local_app_enqueue_scripts(){

        global $local_app_js_version;
        $local_app_js_version = uniqid();

        if( isset( $_REQUEST['page'] ) && $_REQUEST['page'] == 'mam-setup-wizard' )
        {
            wp_register_style('mam-wizard-css', plugins_url("css/mam-wizard-setup.css", __FILE__), array(), $local_app_js_version, 'screen');
            wp_enqueue_style(array( 'mam-wizard-css'));
        }

        wp_register_script('localapp',      plugins_url( 'js/wp-local-app.js', __FILE__ ), array(), $local_app_js_version, true);
        wp_register_script('chart',         plugins_url( 'js/Chart.js', __FILE__ ), array(), $local_app_js_version, true);
        wp_register_script('validate',      plugins_url( 'js/validate.js', __FILE__ ), array(), $local_app_js_version, true);

        wp_register_script('qtip',          plugins_url( 'js/jquery.qtip.min.js', __FILE__ ), array(), $local_app_js_version, true);
        wp_register_script('gifplayer',     plugins_url( 'js/jquery.gifplayer.js', __FILE__ ), array(), $local_app_js_version, true);
        wp_register_script('printthis',     plugins_url( 'js/jquery.printthis.js', __FILE__ ), array(), $local_app_js_version, true);

        wp_register_script('manage-home-stack', plugins_url( 'js/home-screen-stack-manager.js', __FILE__ ), array(), $local_app_js_version, true);

        wp_register_style('local-app-css', plugins_url( "css/wp-local-app.css", __FILE__ ), array(), $local_app_js_version, 'screen');
        wp_register_style('manage-home-stack',   plugins_url( "css/home-screen-stack-manager.css", __FILE__ ), array(), $local_app_js_version, 'screen');

        wp_register_style('qtip',          plugins_url( "css/jquery.qtip.min.css", __FILE__ ), array(), $local_app_js_version, 'screen');
        wp_register_style('css-gifplayer', plugins_url( "css/gifplayer.css", __FILE__ ), array(), $local_app_js_version, 'screen');
        wp_register_style('tsl-general-css', plugins_url("css/tsl-general-css.css", __FILE__), array(), $local_app_js_version, 'screen');


        wp_enqueue_style( 'font-awesome-free', '//use.fontawesome.com/releases/v5.2.0/css/all.css' );

        if(tal_mam_main_manager::local_app_is_settings_page()){

            wp_localize_script( 'localapp', 'localapp_editor', "1" );
            wp_localize_script( 'localapp', 'localapp_status', tal_mam_main_manager::local_app_onboarding_complete() );
            wp_enqueue_script(array('jquery', 'jquery-ui-core', 'jquery-ui-dialog', 'jquery-ui-datepicker', 'chart', 'validate', 'qtip', 'gifplayer', 'localapp', 'printthis' , 'manage-home-stack' ));
            wp_enqueue_style(array('css-gifplayer', 'wp-jquery-ui-dialog', 'fontawesome', 'local-app-css',  'qtip' , 'tsl-general-css' , 'manage-home-stack' ));

        }else{

            wp_localize_script('localapp', 'localapp_status', tal_mam_main_manager::local_app_onboarding_complete());
            wp_localize_script('localapp', 'localapp_editor', "0");
            tal_mam_main_manager::local_app_setup_notifications_meta();

            wp_enqueue_script(array('jquery', 'jquery-ui-core', 'jquery-ui-dialog', 'jquery-ui-datepicker', 'chart', 'validate', 'qtip', 'gifplayer', 'localapp', 'printthis' , 'manage-home-stack' ));
            wp_enqueue_style(array('css-gifplayer', 'wp-jquery-ui-dialog', 'fontawesome', 'local-app-css', 'qtip' , 'tsl-general-css', 'manage-home-stack'));

        }

        wp_enqueue_style( 'jquery-ui-datepicker-style' , '//ajax.googleapis.com/ajax/libs/jqueryui/1.10.4/themes/smoothness/jquery-ui.css');

        $image_url = plugin_dir_url(__FILE__);

        wp_localize_script('localapp', 'local_app_image_url', $image_url);

        if(local_app_get_api_key()){
            wp_localize_script('localapp', 'local_app_status', array('has_acct' => 1, 'ios_publish_status' => get_option( 'local_app_apple_status' ) , 'android_publish_status' => get_option( 'local_app_android_status' ) ));
        }else{
            wp_localize_script('localapp', 'local_app_status', array('has_acct' => 0));
        }

        wp_enqueue_media();

        if(isset($_REQUEST['debug'])) local_app_clear_options();

    }

    function add_admin_menus(){

        $app_settings = new tsl_mam_app_settings_main();

        add_menu_page(__('Mobile App Mgr', 'local_app'), __('Mobile App Mgr', 'local_app') , 'manage_options', 'local_app_setup', 'local_app_setup_page', 'dashicons-tablet' , 10);
        add_submenu_page('local_app_setup', 'App Navigation', 'App Navigation ', 'manage_options', 'local_app_setup', 'local_app_setup_page');
        add_submenu_page('local_app_setup', 'App Settings', 'App Settings', 'activate_plugins', 'mam-app-settings', array($app_settings, 'add_settings_page'));
        add_submenu_page('local_app_setup', 'Setup Wizard', 'Setup Wizard', 'manage_options', 'mam-setup-wizard', array($app_settings, 'add_setup_wizard_page'));
        add_submenu_page('local_app_setup', 'Push Notifications', 'Push Notifications ', 'manage_options', 'local_app_notifications','local_app_setup_page');


        if( current_user_can('update_plugins') ) {
            $software_manager = new mam_software_manager();
            add_submenu_page('local_app_setup', 'Software', 'Software', 'manage_options', 'mam-software-manager', array($software_manager, 'add_software_manager_page'));
        }

        $available_crons = apply_filters( 'mam_cron_manager', array() );
        if(!empty($available_crons)) {
            $setcron_manager = new mam_setcron_manager();
            add_submenu_page('local_app_setup', 'Scheduled Jobs', 'Scheduled Jobs', 'manage_options', 'mam-setcron-manager', array($setcron_manager, 'add_setcron_manager_page'));
        }

        $support_manager = new mam_support_manager_admin_settings();
        add_submenu_page('local_app_setup', 'Support <span class="dashicons dashicons-external"></span>', 'Support <span class="dashicons dashicons-external"></span>', 'manage_options', 'mam-support-manager', array($support_manager, 'add_support_manager_page'));
        add_submenu_page('local_app_setup', 'Publish App', 'Publish App', 'manage_options', 'local_app_publish', 'local_app_setup_page');

//        add_submenu_page('local_app_setup', 'Support', 'Support', 'manage_options', 'mam-support-manager', array( $this, 'support_submenu_page_callback' ));

    }

    function support_submenu_page_redirect() {
        ?>
        <script type="text/javascript">
            jQuery("#adminmenuwrap #adminmenu #toplevel_page_local_app_setup ul.wp-submenu.wp-submenu-wrap li").each(function(k,v){
                var href = jQuery(v).find('a').attr('href');
                if(href){
                    if(href.indexOf('mam-support-manager') != -1) {
                        jQuery(v).find('a').attr('href', 'https://tinyscreenlabs.com/dashboard');
                        jQuery(v).find('a').attr('target', '_blank');
                    }
                }
            });
        </script>
        <?php
    }

    function local_app_setup_notifications_meta(){

        global $local_app_notification_id;
        global $local_app_notification_type;
        $local_app_notification_id = null;
        $local_app_notification_type = null;

        if(get_the_ID() > 0) {
            $local_app_notification_id = get_the_ID();
            $local_app_notification_type = 'posts';
            wp_localize_script('localapp', 'localapp_notification_id', array('id' => get_the_ID() , 'type' => 'posts'));
        }else{
            //check for connections
            if( is_plugin_active( 'connections/connections.php' ) ) {
                if(isset($_REQUEST['id']) && isset($_REQUEST['page']) && $_REQUEST['page']=='connections_manage') {
                    $local_app_notification_id = $_REQUEST['id'];
                    $local_app_notification_type = 'connections';
                    wp_localize_script('localapp', 'localapp_notification_id', array('id' => $_REQUEST['id'], 'type' => 'connections'));
                }
            }
        }
    }

    function local_app_onboarding_complete(){

        if (get_option('local-app-onboarding-status')) {
            $data['status'] = 1;
        }else{
            $data['status'] = 0;
        }

        if(isset($_REQUEST['page']) && $_REQUEST['page'] == 'local_app_setup') {
            if (isset($_REQUEST['tab']) && $_REQUEST['tab'] != 'homepage') {
                $data['status'] = 1;
            }
        }

        return $data;
    }

    function local_app_activate_api(){

        if(current_user_can( 'manage_options' )) {
            require_once 'includes/local-app-submit-class.php';

            if($_REQUEST['product_id']==0) {
                $module = new local_app_submit();
                $result = $module->local_app_activate_api_key($_REQUEST['api_key'], $_REQUEST['email']);
                $update_status = $result;
            }else{
                //get product id and submit
                $product_id = tal_mam_main_manager::local_app_get_addon_product_id($_REQUEST['product_id']);
                $module = new local_app_submit();
                $result = $module->local_app_activate_api_key($_REQUEST['api_key'], $_REQUEST['email'], $product_id);
                $update_status = $result;
            }
        }else{
            $update_status = array('status' => 'Shove Off.');
        }
        wp_send_json( $update_status );
        die();

    }

    function local_app_get_addon_product_id($product_id){

        $addons = tal_mam_main_manager::local_app_premium_add_ons();

        for($x=0;$x<sizeof($addons);$x++){
            if($addon[$x]['id'] = $product_id) return $addons[$x]['product_id'];
        }
    }

    function local_app_premium_add_ons(){

        $addon = array();

        //$addon[] = array('id' => 1, 'name' => 'Geofilter for Directories', 'product_id' => '999');

        for($x=0;$x<sizeof($addon);$x++){
            $apikey = get_option('local-app-adon-apikey-' . $addon[$x]['id']);
            $addon[$x]['api_key'] = $apikey;
        }

        return $addon;
    }


    function local_app_set_onboarding_status(){

        if(current_user_can( 'manage_options' )) {
            update_option('local-app-onboarding-status' , 1);
            $update_status = array('status' => 'done');
        }else{
            $update_status = array('status' => 'Shove Off.');
        }

        wp_send_json( $update_status );
        die();
    }


    function local_app_send_preview(){

        if(current_user_can( 'manage_options' )) {

            $local_host = 0;
            if(!local_app_is_connected_to_internet()) $local_host = 1;

            $querystring = 'https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=tsl_preview_request&local_host='.$local_host.'&email_address='.$_REQUEST['email_address'].'&ajax_url='.admin_url('admin-ajax.php');

            $response = wp_remote_get($querystring);

            if (is_array($response) && !is_wp_error($response)) {
                $update_status = array('status' => 'Preview sent'  );
            }else{
                $update_status = array('status' => 'Network Error - Please try again');
            }

        }else{
            $update_status = array('status' => 'Shove Off.');
        }

        wp_send_json( $update_status );
        die();
    }



    function local_app_preview_in_phone(){

//        if(current_user_can( 'manage_options' )) {
//
//            $ostype = '';
//            if($_REQUEST['ostype']==1) $ostype = 'ios';
//            if($_REQUEST['ostype']==2) $ostype = 'android';
//
//            $querystring = 'https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=tsl_publish_request&activity=preview&os_type='.$ostype.'&email_address='.$_REQUEST['email_address'].'&ajax_url='.admin_url('admin-ajax.php');
//
//            $response = wp_remote_get($querystring);
//
//            if (is_array($response) && !is_wp_error($response)) {
//                $update_status = array('status' => 'TestFlight App Requested' );
//            }else{
//                $update_status = array('status' => 'Network Error - Please try again');
//            }
//
//        }else{
//            $update_status = array('status' => 'Shove Off.');
//        }
//
//        wp_send_json( $update_status );
//        die();
    }

    function local_app_publish_in_phone(){

        if(current_user_can( 'manage_options' )) {

            if( strlen( get_option('mam-main-software-manager-api') ) > 0 ){
                $apikey = get_option('mam-main-software-manager-api');
            }

            if(! $apikey){
                $update_status = array('status' => 'No API Key.');
            }else {

                if($_REQUEST['ostype'] == 1){
                    $_REQUEST['ostype'] = 'ios';
                }else{
                    $_REQUEST['ostype'] = 'android';
                }

                if($_REQUEST['save_action'] == 1){
                    $update_status = array('status' => 'Draft Saved' );
                    wp_send_json( $update_status );
                    die();
                }
                //$email = get_option('local-app-api-email');
                $current_user = wp_get_current_user();
                $email = $current_user->user_email;


                //$querystring = 'https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=tsl_publish_request&activity=publish&apikey=' . $apikey . '&os_type=' . $_REQUEST['ostype'] . '&email_address=' . $email . '&ajax_url=' . admin_url('admin-ajax.php');
                $querystring = 'https://tinyscreenlabs.com/wp-admin/admin-ajax.php?action=tsl_publish_request&activity=publish&apikey=' . $apikey . '&os_type=' . $_REQUEST['ostype'] . '&email_address=' . $email . '&ajax_url=' . admin_url('admin-ajax.php');
mamdebug('publish', $querystring);
                $response = wp_remote_get($querystring);

                if (is_array($response) && !is_wp_error($response)) {
                    if($_REQUEST['ostype'] == 'ios') {
                        update_option('local_app_apple_status' , "Publish requested on " . date('M d, Y'));
                    }else{
                        update_option('local_app_android_status' , "Publish requested on " . date('M d, Y'));
                    }
                    $update_status = array('status' => "Publish requested on " . date('M d, Y'));
                    update_option('local-app-is-pushed' , true);
                } else {
                    $update_status = array('status' => 'Network Error - Please try again');
                    if($_REQUEST['ostype'] == 'ios') {
                        update_option('local_app_apple_status', 'Network Error - Please Submit again');
                    }else{
                        update_option('local_app_android_status', 'Network Error - Please Submit again');
                    }
                }
            }
        }else{
            $update_status = array('status' => 'Shove Off.');
        }

        wp_send_json( $update_status );
        die();
    }

    function local_app_save_publish_app_settings(){

        update_option('local-app-ios-app-name', trim($_POST['ios_app_name']));
        update_option('ios_pn_app_bundle_id', trim($_POST['ios_bundle_id']));
        update_option('android_pn_app_bundle_id', trim($_POST['android_bundle_id']));
        update_option('ios_pn_team_id', trim($_POST['ios_team_id']));
        update_option('ios_app_camera_message', trim($_POST['ios_camera_message']));
        update_option('ios_app_gps_message', trim($_POST['ios_gps_message']));
        update_option('ios_app_contacts_message', trim($_POST['ios_contacts_message']));
        update_option('ios_app_nfc_message', trim($_POST['ios_nfc_message']));
        update_option('ios_app_google_plist', trim($_POST['ios_google_plist']));
        update_option('mam_ios_version_number', trim($_POST['ios_version_number']));
        update_option('mam_android_version_number', trim($_POST['android_version_number']));
        update_option('mam_ios_build_number', trim($_POST['ios_build_number']));
        update_option('mam_android_build_number', trim($_POST['android_build_number']));
        update_option('ios_app_google_places_api', trim($_POST['ios_google_places_api']));
        update_option('ios_app_launch_screen_bg', trim($_POST['ios_app_launch_screen_bg']));
        update_option('local-app-images-app-icon', trim($_POST['app_icon_image']));
        update_option('local-app-app-icon-bg', trim($_POST['app_icon_bg']));
        update_option('local-app-images-launch-screen-1', trim($_POST['launch_screen_1']));
        update_option('local-app-images-launch-screen-2', trim($_POST['launch_screen_2']));
        update_option('local-app-ios-app-channel', trim($_POST['app_channel']));

        parse_str($_POST['ios_entitlements'], $entitlements);
        if(!empty($entitlements)){
            update_option('saved_entitlements', $entitlements['entitlements']);
        }else{
            update_option('saved_entitlements', array());
        }

        $update_status = array('status' => 'done');
        wp_send_json($update_status);
        die();

    }
}

function local_app_is_connected_to_internet(){

    foreach (array('HTTP_CLIENT_IP', 'HTTP_X_FORWARDED_FOR', 'HTTP_X_FORWARDED', 'HTTP_X_CLUSTER_CLIENT_IP', 'HTTP_FORWARDED_FOR', 'HTTP_FORWARDED', 'REMOTE_ADDR') as $key){
        if (array_key_exists($key, $_SERVER) === true){
            foreach (explode(',', $_SERVER[$key]) as $ip){
                $ip = trim($ip); // just to be safe

                if (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE) !== false){
                    return true;
                }else{
                    return false;
                }
            }
        }
    }

}

//delete after refactoring
function local_app_get_api_key()
{
    return get_option('local-app-api');
}


if(!function_exists('mamdebug')) {
	function mamdebug( $from, $message , $plugin = '') {
        if(function_exists('mam_log_item')) {
            mam_log_item( $from, $message , $plugin = '');
	    }
    }
}

if( ! function_exists( 'tsl_mam_get_option_value' )) {


    function tsl_mam_get_option_value($id = null, $tab_id = 'general-settings', $user_role = null)
    {

        if (!$user_role) {

            global $mam_user_role;
            $user_role = $mam_user_role;
            if (!$mam_user_role) $user_role = 'mam-all';

        }

        $option_value = get_option($id);

        if ($option_value) {

            if (is_array($option_value)) {

                if (isset($option_value[$user_role][$tab_id])) {
                    return $option_value[$user_role][$tab_id];
                } else if (isset($option_value['mam-all'][$tab_id])) {
                    return $option_value['mam-all'][$tab_id];
                }

                $tab_id = 'general-settings';

                if (isset($option_value[$user_role][$tab_id])) {
                    return $option_value[$user_role][$tab_id];
                } else if (isset($option_value['mam-all'][$tab_id])) {
                    return $option_value['mam-all'][$tab_id];
                }

                return null;
            }

            return $option_value;

        }

    }

}

function mam_distance_between( $latitudeFrom, $longitudeFrom, $latitudeTo, $longitudeTo, $earthRadius = 3959)
{

    if( $latitudeFrom == 0 ) return 0;
    if( $longitudeFrom == 0 ) return 0;
    if( $latitudeTo == 0 ) return 0;
    if( $longitudeTo == 0 ) return 0;

    // convert from degrees to radians
    $latFrom = deg2rad((float)$latitudeFrom);
    $lonFrom = deg2rad((float)$longitudeFrom);
    $latTo = deg2rad((float)$latitudeTo);
    $lonTo = deg2rad((float)$longitudeTo);

    $latDelta = $latTo - $latFrom;
    $lonDelta = $lonTo - $lonFrom;

    $angle = 2 * asin(sqrt(pow(sin($latDelta / 2), 2) +
            cos($latFrom) * cos($latTo) * pow(sin($lonDelta / 2), 2)));
    return $angle * $earthRadius;
}

//this functio is utlzied in a lot pf places so must keep until they are converted
function mam_update_current_user()
{
    apply_filters( 'mam_update_current_user' , 0 );
}
